@@ -1,6 +1,6 @@
 /*
  **********************************************************************
- *   Copyright (C) 1996-2006, International Business Machines
+ *   Copyright (C) 1996-2011, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  **********************************************************************
  *
@@ -26,7 +26,18 @@
  */
 
 #include "locmap.h"
+#include "unicode/uloc.h"
 #include "cstring.h"
+#include "cmemory.h"
+
+#if defined(U_WINDOWS) && defined(_MSC_VER) && (_MSC_VER >= 1500)
+#define USE_WINDOWS_LOCALE_API
+#endif
+
+#ifdef USE_WINDOWS_LOCALE_API
+#include <windows.h>
+#include <winnls.h>
+#endif
 
 /*
  * Note:
@@ -66,24 +77,36 @@ typedef struct ILcidPosixMap
 /////////////////////////////////////////////////
 */
 
-/*
- The standard one language/one country mapping for LCID.
- The first element must be the language, and the following
- elements are the language with the country.
+/**
+ * The standard one language/one country mapping for LCID.
+ * The first element must be the language, and the following
+ * elements are the language with the country.
+ * @param hostID LCID in host format such as 0x044d
+ * @param languageID posix ID of just the language such as 'de'
+ * @param posixID posix ID of the language_TERRITORY such as 'de_CH'
  */
 #define ILCID_POSIX_ELEMENT_ARRAY(hostID, languageID, posixID) \
-static const ILcidPosixElement languageID[] = { \
+static const ILcidPosixElement locmap_ ## languageID [] = { \
     {LANGUAGE_LCID(hostID), #languageID},     /* parent locale */ \
     {hostID, #posixID}, \
 };
 
-/*
- Create the map for the posixID. This macro supposes that the language string
- name is the same as the global variable name, and that the first element
- in the ILcidPosixElement is just the language.
+/**
+ * Define a subtable by ID
+ * @param id the POSIX ID, either a language or language_TERRITORY
+ */
+#define ILCID_POSIX_SUBTABLE(id) \
+static const ILcidPosixElement locmap_ ## id [] =
+
+
+/**
+ * Create the map for the posixID. This macro supposes that the language string
+ * name is the same as the global variable name, and that the first element
+ * in the ILcidPosixElement is just the language.
+ * @param _posixID the full POSIX ID for this entry. 
  */
 #define ILCID_POSIX_MAP(_posixID) \
-    {sizeof(_posixID)/sizeof(ILcidPosixElement), _posixID}
+    {sizeof(locmap_ ## _posixID)/sizeof(ILcidPosixElement), locmap_ ## _posixID}
 
 /*
 ////////////////////////////////////////////
@@ -94,12 +117,16 @@ static const ILcidPosixElement languageID[] = { \
 // Keep static locale variables inside the function so that
 // it can be created properly during static init.
 //
+// Note: This table should be updated periodically. Check the National Lanaguage Support API Reference Website.
+//       Microsoft is moving away from LCID in favor of locale name as of Vista.  This table needs to be
+//       maintained for support of older Windows version.
+//       Update: Windows 7 (091130)
 ////////////////////////////////////////////
 */
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0436, af, af_ZA)
 
-static const ILcidPosixElement ar[] = {
+ILCID_POSIX_SUBTABLE(ar) {
     {0x01,   "ar"},
     {0x3801, "ar_AE"},
     {0x3c01, "ar_BH"},
@@ -111,6 +138,7 @@ static const ILcidPosixElement ar[] = {
     {0x3001, "ar_LB"},
     {0x1001, "ar_LY"},
     {0x1801, "ar_MA"},
+    {0x1801, "ar_MO"},
     {0x2001, "ar_OM"},
     {0x4001, "ar_QA"},
     {0x0401, "ar_SA"},
@@ -123,19 +151,19 @@ ILCID_POSIX_ELEMENT_ARRAY(0x044d, as, as_IN)
 ILCID_POSIX_ELEMENT_ARRAY(0x045e, am, am_ET)
 ILCID_POSIX_ELEMENT_ARRAY(0x047a, arn,arn_CL)
 
-static const ILcidPosixElement az[] = {
+ILCID_POSIX_SUBTABLE(az) {
     {0x2c,   "az"},
     {0x082c, "az_Cyrl_AZ"},  /* Cyrillic based */
-    {0x082c, "az_Cyrl"},  /* Cyrillic based */
+    {0x742c, "az_Cyrl"},  /* Cyrillic based */
     {0x042c, "az_Latn_AZ"}, /* Latin based */
-    {0x042c, "az_Latn"}, /* Latin based */
+    {0x782c, "az_Latn"}, /* Latin based */
     {0x042c, "az_AZ"} /* Latin based */
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x046d, ba, ba_RU)
 ILCID_POSIX_ELEMENT_ARRAY(0x0423, be, be_BY)
 
-static const ILcidPosixElement ber[] = {
+ILCID_POSIX_SUBTABLE(ber) {
     {0x5f,   "ber"},
     {0x045f, "ber_Arab_DZ"},
     {0x045f, "ber_Arab"},
@@ -145,13 +173,15 @@ static const ILcidPosixElement ber[] = {
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0402, bg, bg_BG)
 
-static const ILcidPosixElement bn[] = {
+ILCID_POSIX_ELEMENT_ARRAY(0x0466, bin, bin_NG)
+
+ILCID_POSIX_SUBTABLE(bn) {
     {0x45,   "bn"},
     {0x0845, "bn_BD"},
     {0x0445, "bn_IN"}
 };
 
-static const ILcidPosixElement bo[] = {
+ILCID_POSIX_SUBTABLE(bo) {
     {0x51,   "bo"},
     {0x0851, "bo_BT"},
     {0x0451, "bo_CN"}
@@ -163,15 +193,12 @@ ILCID_POSIX_ELEMENT_ARRAY(0x0483, co, co_FR)
 ILCID_POSIX_ELEMENT_ARRAY(0x045c, chr,chr_US)
 
 /* Declared as cs_CZ to get around compiler errors on z/OS, which defines cs as a function */
-static const ILcidPosixElement cs_CZ[] = {
-    {0x05,   "cs"},
-    {0x0405, "cs_CZ"},
-};
+ILCID_POSIX_ELEMENT_ARRAY(0x0405, cs, cs_CZ)
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0452, cy, cy_GB)
 ILCID_POSIX_ELEMENT_ARRAY(0x0406, da, da_DK)
 
-static const ILcidPosixElement de[] = {
+ILCID_POSIX_SUBTABLE(de) {
     {0x07,   "de"},
     {0x0c07, "de_AT"},
     {0x0807, "de_CH"},
@@ -185,12 +212,14 @@ static const ILcidPosixElement de[] = {
 ILCID_POSIX_ELEMENT_ARRAY(0x0465, dv, dv_MV)
 ILCID_POSIX_ELEMENT_ARRAY(0x0408, el, el_GR)
 
-static const ILcidPosixElement en[] = {
+ILCID_POSIX_SUBTABLE(en) {
     {0x09,   "en"},
     {0x0c09, "en_AU"},
     {0x2809, "en_BZ"},
     {0x1009, "en_CA"},
     {0x0809, "en_GB"},
+    {0x3c09, "en_HK"},
+    {0x3809, "en_ID"},
     {0x1809, "en_IE"},
     {0x4009, "en_IN"},
     {0x2009, "en_JM"},
@@ -204,6 +233,7 @@ static const ILcidPosixElement en[] = {
     {0x2409, "en_VI"},  /* Virgin Islands AKA Caribbean Islands (en_CB). */
     {0x1c09, "en_ZA"},
     {0x3009, "en_ZW"},
+    {0x2409, "en_029"},
     {0x0409, "en_AS"},  /* Alias for en_US. Leave last. */
     {0x0409, "en_GU"},  /* Alias for en_US. Leave last. */
     {0x0409, "en_MH"},  /* Alias for en_US. Leave last. */
@@ -211,11 +241,11 @@ static const ILcidPosixElement en[] = {
     {0x0409, "en_UM"}   /* Alias for en_US. Leave last. */
 };
 
-static const ILcidPosixElement en_US_POSIX[] = {
+ILCID_POSIX_SUBTABLE(en_US_POSIX) {
     {0x007f, "en_US_POSIX"} /* duplicate for roundtripping */
 };
 
-static const ILcidPosixElement es[] = {
+ILCID_POSIX_SUBTABLE(es) {
     {0x0a,   "es"},
     {0x2c0a, "es_AR"},
     {0x400a, "es_BO"},
@@ -237,6 +267,7 @@ static const ILcidPosixElement es[] = {
     {0x540a, "es_US"},
     {0x380a, "es_UY"},
     {0x200a, "es_VE"},
+    {0xe40a, "es_419"},
     {0x040a, "es_ES@collation=traditional"},
     {0x040a, "es@collation=traditional"}
 };
@@ -245,14 +276,14 @@ ILCID_POSIX_ELEMENT_ARRAY(0x0425, et, et_EE)
 ILCID_POSIX_ELEMENT_ARRAY(0x042d, eu, eu_ES)
 
 /* ISO-639 doesn't distinguish between Persian and Dari.*/
-static const ILcidPosixElement fa[] = {
+ILCID_POSIX_SUBTABLE(fa) {
     {0x29,   "fa"},
     {0x0429, "fa_IR"},  /* Persian/Farsi (Iran) */
     {0x048c, "fa_AF"}   /* Persian/Dari (Afghanistan) */
 };
 
 /* duplicate for roundtripping */
-static const ILcidPosixElement fa_AF[] = {
+ILCID_POSIX_SUBTABLE(fa_AF) {
     {0x8c,   "fa_AF"},  /* Persian/Dari (Afghanistan) */
     {0x048c, "fa_AF"}   /* Persian/Dari (Afghanistan) */
 };
@@ -261,11 +292,12 @@ ILCID_POSIX_ELEMENT_ARRAY(0x040b, fi, fi_FI)
 ILCID_POSIX_ELEMENT_ARRAY(0x0464, fil,fil_PH)
 ILCID_POSIX_ELEMENT_ARRAY(0x0438, fo, fo_FO)
 
-static const ILcidPosixElement fr[] = {
+ILCID_POSIX_SUBTABLE(fr) {
     {0x0c,   "fr"},
     {0x080c, "fr_BE"},
     {0x0c0c, "fr_CA"},
     {0x240c, "fr_CD"},
+    {0x240c, "fr_CG"},
     {0x100c, "fr_CH"},
     {0x300c, "fr_CI"},
     {0x2c0c, "fr_CM"},
@@ -276,67 +308,85 @@ static const ILcidPosixElement fr[] = {
     {0x180c, "fr_MC"},
     {0x340c, "fr_ML"},
     {0x200c, "fr_RE"},
-    {0x280c, "fr_SN"}
+    {0x280c, "fr_SN"},
+    {0xe40c, "fr_015"},
+    {0x1c0c, "fr_029"}
 };
 
+ILCID_POSIX_ELEMENT_ARRAY(0x0467, fuv, fuv_NG)
+
 ILCID_POSIX_ELEMENT_ARRAY(0x0462, fy, fy_NL)
 
-/* This LCID is really two different locales.*/
-static const ILcidPosixElement ga[] = {
+ILCID_POSIX_SUBTABLE(ga) { /* Gaelic (Ireland) */
     {0x3c,   "ga"},
-    {0x3c,   "gd"},
-    {0x083c, "ga_IE"},  /* Gaelic (Ireland) */
-    {0x043c, "gd_GB"}   /* Gaelic (Scotland) */
+    {0x083c, "ga_IE"},
+    {0x043c, "gd_GB"}
+};
+
+ILCID_POSIX_SUBTABLE(gd) { /* Gaelic (Scotland) */
+    {0x91,   "gd"},
+    {0x0491, "gd_GB"}
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0456, gl, gl_ES)
 ILCID_POSIX_ELEMENT_ARRAY(0x0447, gu, gu_IN)
 ILCID_POSIX_ELEMENT_ARRAY(0x0474, gn, gn_PY)
 ILCID_POSIX_ELEMENT_ARRAY(0x0484, gsw,gsw_FR)
-ILCID_POSIX_ELEMENT_ARRAY(0x0468, ha, ha_NG) /* ha_Latn_NG? */
+
+ILCID_POSIX_SUBTABLE(ha) {
+    {0x68,   "ha"},
+    {0x7c68, "ha_Latn"},
+    {0x0468, "ha_Latn_NG"},
+};
+
 ILCID_POSIX_ELEMENT_ARRAY(0x0475, haw,haw_US)
 ILCID_POSIX_ELEMENT_ARRAY(0x040d, he, he_IL)
 ILCID_POSIX_ELEMENT_ARRAY(0x0439, hi, hi_IN)
 
 /* This LCID is really four different locales.*/
-static const ILcidPosixElement hr[] = {
+ILCID_POSIX_SUBTABLE(hr) {
     {0x1a,   "hr"},
     {0x141a, "bs_Latn_BA"},  /* Bosnian, Bosnia and Herzegovina */
-    {0x141a, "bs_Latn"},  /* Bosnian, Bosnia and Herzegovina */
+    {0x681a, "bs_Latn"},  /* Bosnian, Bosnia and Herzegovina */
     {0x141a, "bs_BA"},  /* Bosnian, Bosnia and Herzegovina */
-    {0x141a, "bs"},     /* Bosnian */
+    {0x781a, "bs"},     /* Bosnian */
     {0x201a, "bs_Cyrl_BA"},  /* Bosnian, Bosnia and Herzegovina */
-    {0x201a, "bs_Cyrl"},  /* Bosnian, Bosnia and Herzegovina */
+    {0x641a, "bs_Cyrl"},  /* Bosnian, Bosnia and Herzegovina */
     {0x101a, "hr_BA"},  /* Croatian in Bosnia */
     {0x041a, "hr_HR"},  /* Croatian*/
+    {0x2c1a, "sr_Latn_ME"},
+    {0x241a, "sr_Latn_RS"},
     {0x181a, "sr_Latn_BA"}, /* Serbo-Croatian in Bosnia */
     {0x081a, "sr_Latn_CS"}, /* Serbo-Croatian*/
-    {0x081a, "sr_Latn"},    /* It's 0x1a or 0x081a, pick one to make the test program happy. */
+    {0x701a, "sr_Latn"},    /* It's 0x1a or 0x081a, pick one to make the test program happy. */
     {0x1c1a, "sr_Cyrl_BA"}, /* Serbo-Croatian in Bosnia */
     {0x0c1a, "sr_Cyrl_CS"}, /* Serbian*/
-    {0x0c1a, "sr_Cyrl"},    /* It's 0x1a or 0x0c1a, pick one to make the test program happy. */
-    {0x0c1a, "sr"}          /* In CLDR sr is sr_Cyrl. */
+    {0x301a, "sr_Cyrl_ME"},
+    {0x281a, "sr_Cyrl_RS"},
+    {0x6c1a, "sr_Cyrl"},    /* It's 0x1a or 0x0c1a, pick one to make the test program happy. */
+    {0x7c1a, "sr"}          /* In CLDR sr is sr_Cyrl. */
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x040e, hu, hu_HU)
 ILCID_POSIX_ELEMENT_ARRAY(0x042b, hy, hy_AM)
+ILCID_POSIX_ELEMENT_ARRAY(0x0469, ibb, ibb_NG)
 ILCID_POSIX_ELEMENT_ARRAY(0x0421, id, id_ID)
 ILCID_POSIX_ELEMENT_ARRAY(0x0470, ig, ig_NG)
 ILCID_POSIX_ELEMENT_ARRAY(0x0478, ii, ii_CN)
 ILCID_POSIX_ELEMENT_ARRAY(0x040f, is, is_IS)
 
-static const ILcidPosixElement it[] = {
+ILCID_POSIX_SUBTABLE(it) {
     {0x10,   "it"},
     {0x0810, "it_CH"},
     {0x0410, "it_IT"}
 };
 
-static const ILcidPosixElement iu[] = {
+ILCID_POSIX_SUBTABLE(iu) {
     {0x5d,   "iu"},
     {0x045d, "iu_Cans_CA"},
-    {0x045d, "iu_Cans"},
+    {0x785d, "iu_Cans"},
     {0x085d, "iu_Latn_CA"},
-    {0x085d, "iu_Latn"}
+    {0x7c5d, "iu_Latn"}
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x040d, iw, iw_IL)    /*Left in for compatibility*/
@@ -347,7 +397,7 @@ ILCID_POSIX_ELEMENT_ARRAY(0x046f, kl, kl_GL)
 ILCID_POSIX_ELEMENT_ARRAY(0x0453, km, km_KH)
 ILCID_POSIX_ELEMENT_ARRAY(0x044b, kn, kn_IN)
 
-static const ILcidPosixElement ko[] = {
+ILCID_POSIX_SUBTABLE(ko) {
     {0x12,   "ko"},
     {0x0812, "ko_KP"},
     {0x0412, "ko_KR"}
@@ -356,10 +406,11 @@ static const ILcidPosixElement ko[] = {
 ILCID_POSIX_ELEMENT_ARRAY(0x0457, kok, kok_IN)
 ILCID_POSIX_ELEMENT_ARRAY(0x0471, kr,  kr_NG)
 
-static const ILcidPosixElement ks[] = {         /* We could add PK and CN too */
+ILCID_POSIX_SUBTABLE(ks) {         /* We could add PK and CN too */
     {0x60,   "ks"},
     {0x0860, "ks_IN"},              /* Documentation doesn't mention script */
-    {0x0460, "ks_Arab_IN"}
+    {0x0460, "ks_Arab_IN"},
+    {0x0860, "ks_Deva_IN"}
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0440, ky, ky_KG)   /* Kyrgyz is spoken in Kyrgyzstan */
@@ -372,17 +423,20 @@ ILCID_POSIX_ELEMENT_ARRAY(0x0481, mi, mi_NZ)
 ILCID_POSIX_ELEMENT_ARRAY(0x042f, mk, mk_MK)
 ILCID_POSIX_ELEMENT_ARRAY(0x044c, ml, ml_IN)
 
-static const ILcidPosixElement mn[] = {
+ILCID_POSIX_SUBTABLE(mn) {
     {0x50,   "mn"},
+    {0x0450, "mn_MN"},
+    {0x7c50, "mn_Mong"},
+    {0x0850, "mn_Mong_CN"},
     {0x0850, "mn_CN"},
-    {0x0450, "mn_MN"}
+    {0x7850, "mn_Cyrl"}
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0458, mni,mni_IN)
 ILCID_POSIX_ELEMENT_ARRAY(0x047c, moh,moh_CA)
 ILCID_POSIX_ELEMENT_ARRAY(0x044e, mr, mr_IN)
 
-static const ILcidPosixElement ms[] = {
+ILCID_POSIX_SUBTABLE(ms) {
     {0x3e,   "ms"},
     {0x083e, "ms_BN"},   /* Brunei Darussalam*/
     {0x043e, "ms_MY"}    /* Malaysia*/
@@ -391,87 +445,111 @@ static const ILcidPosixElement ms[] = {
 ILCID_POSIX_ELEMENT_ARRAY(0x043a, mt, mt_MT)
 ILCID_POSIX_ELEMENT_ARRAY(0x0455, my, my_MM)
 
-static const ILcidPosixElement ne[] = {
+ILCID_POSIX_SUBTABLE(ne) {
     {0x61,   "ne"},
     {0x0861, "ne_IN"},   /* India*/
     {0x0461, "ne_NP"}    /* Nepal*/
 };
 
-static const ILcidPosixElement nl[] = {
+ILCID_POSIX_SUBTABLE(nl) {
     {0x13,   "nl"},
     {0x0813, "nl_BE"},
     {0x0413, "nl_NL"}
 };
 
 /* The "no" locale split into nb and nn.  By default in ICU, "no" is nb.*/
-static const ILcidPosixElement no[] = {
-    {0x14,   "nb"},     /* really nb */
+ILCID_POSIX_SUBTABLE(no) {
+    {0x14,   "no"},     /* really nb_NO */
+    {0x7c14, "nb"},     /* really nb */
     {0x0414, "nb_NO"},  /* really nb_NO. Keep first in the 414 list. */
-    {0x0414, "no"},     /* really nb_NO */
     {0x0414, "no_NO"},  /* really nb_NO */
     {0x0814, "nn_NO"},  /* really nn_NO. Keep first in the 814 list.  */
-    {0x0814, "nn"},     /* It's 0x14 or 0x814, pick one to make the test program happy. */
+    {0x7814, "nn"},     /* It's 0x14 or 0x814, pick one to make the test program happy. */
     {0x0814, "no_NO_NY"}/* really nn_NO */
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x046c, nso,nso_ZA)   /* TODO: Verify the ISO-639 code */
 ILCID_POSIX_ELEMENT_ARRAY(0x0482, oc, oc_FR)
-ILCID_POSIX_ELEMENT_ARRAY(0x0472, om, om_ET)    /* TODO: Verify the country */
+
+ILCID_POSIX_SUBTABLE(om) { /* TODO: Verify the country */
+    {0x72,   "om"},
+    {0x0472, "om_ET"},
+    {0x0472, "gaz_ET"}
+};
 
 /* Declared as or_IN to get around compiler errors*/
-static const ILcidPosixElement or_IN[] = {
+ILCID_POSIX_SUBTABLE(or_IN) {
     {0x48,   "or"},
     {0x0448, "or_IN"},
 };
 
-static const ILcidPosixElement pa[] = {
+
+ILCID_POSIX_SUBTABLE(pa) {
     {0x46,   "pa"},
     {0x0446, "pa_IN"},
     {0x0846, "pa_PK"}
 };
 
+ILCID_POSIX_ELEMENT_ARRAY(0x0479, pap, pap_AN)
 ILCID_POSIX_ELEMENT_ARRAY(0x0415, pl, pl_PL)
 ILCID_POSIX_ELEMENT_ARRAY(0x0463, ps, ps_AF)
 
-static const ILcidPosixElement pt[] = {
+ILCID_POSIX_SUBTABLE(pt) {
     {0x16,   "pt"},
     {0x0416, "pt_BR"},
     {0x0816, "pt_PT"}
 };
 
-static const ILcidPosixElement qu[] = {
+ILCID_POSIX_SUBTABLE(qu) {
     {0x6b,   "qu"},
     {0x046b, "qu_BO"},
     {0x086b, "qu_EC"},
-    {0x0C6b, "qu_PE"}
+    {0x0C6b, "qu_PE"},
+    {0x046b, "quz_BO"},
+    {0x086b, "quz_EC"},
+    {0x0C6b, "quz_PE"}
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0486, qut, qut_GT) /* qut is an ISO-639-3 code */
 ILCID_POSIX_ELEMENT_ARRAY(0x0417, rm, rm_CH)
-ILCID_POSIX_ELEMENT_ARRAY(0x0418, ro, ro_RO)
 
-static const ILcidPosixElement root[] = {
+ILCID_POSIX_SUBTABLE(ro) {
+    {0x18,   "ro"},
+    {0x0418, "ro_RO"},
+    {0x0818, "ro_MD"}
+};
+
+ILCID_POSIX_SUBTABLE(root) {
     {0x00,   "root"}
 };
 
-ILCID_POSIX_ELEMENT_ARRAY(0x0419, ru, ru_RU)
+ILCID_POSIX_SUBTABLE(ru) {
+    {0x19,   "ru"},
+    {0x0419, "ru_RU"},
+    {0x0819, "ru_MD"}
+};
+
 ILCID_POSIX_ELEMENT_ARRAY(0x0487, rw, rw_RW)
 ILCID_POSIX_ELEMENT_ARRAY(0x044f, sa, sa_IN)
 ILCID_POSIX_ELEMENT_ARRAY(0x0485, sah,sah_RU)
 
-static const ILcidPosixElement sd[] = {
+ILCID_POSIX_SUBTABLE(sd) {
     {0x59,   "sd"},
     {0x0459, "sd_IN"},
     {0x0859, "sd_PK"}
 };
 
-static const ILcidPosixElement se[] = {
+ILCID_POSIX_SUBTABLE(se) {
     {0x3b,   "se"},
     {0x0c3b, "se_FI"},
     {0x043b, "se_NO"},
     {0x083b, "se_SE"},
+    {0x783b, "sma"},
     {0x183b, "sma_NO"},
     {0x1c3b, "sma_SE"},
+    {0x7c3b, "smj"},
+    {0x703b, "smn"},
+    {0x743b, "sms"},
     {0x103b, "smj_NO"},
     {0x143b, "smj_SE"},
     {0x243b, "smn_FI"},
@@ -481,10 +559,17 @@ static const ILcidPosixElement se[] = {
 ILCID_POSIX_ELEMENT_ARRAY(0x045b, si, si_LK)
 ILCID_POSIX_ELEMENT_ARRAY(0x041b, sk, sk_SK)
 ILCID_POSIX_ELEMENT_ARRAY(0x0424, sl, sl_SI)
-ILCID_POSIX_ELEMENT_ARRAY(0x0477, so, so_ET)    /* TODO: Verify the country */
-ILCID_POSIX_ELEMENT_ARRAY(0x041c, sq, sq_AL)
 
-static const ILcidPosixElement sv[] = {
+ILCID_POSIX_SUBTABLE(so) { /* TODO: Verify the country */
+    {0x77,   "so"},
+    {0x0477, "so_ET"},
+    {0x0477, "so_SO"}
+};
+
+ILCID_POSIX_ELEMENT_ARRAY(0x041c, sq, sq_AL)
+ILCID_POSIX_ELEMENT_ARRAY(0x0430, st, st_ZA)
+
+ILCID_POSIX_SUBTABLE(sv) {
     {0x1d,   "sv"},
     {0x081d, "sv_FI"},
     {0x041d, "sv_SE"}
@@ -494,56 +579,93 @@ ILCID_POSIX_ELEMENT_ARRAY(0x0441, sw, sw_KE)
 ILCID_POSIX_ELEMENT_ARRAY(0x045A, syr, syr_SY)
 ILCID_POSIX_ELEMENT_ARRAY(0x0449, ta, ta_IN)
 ILCID_POSIX_ELEMENT_ARRAY(0x044a, te, te_IN)
-ILCID_POSIX_ELEMENT_ARRAY(0x0428, tg, tg_TJ) /* Cyrillic based by default */
+
+/* Cyrillic based by default */
+ILCID_POSIX_SUBTABLE(tg) {
+    {0x28,   "tg"},
+    {0x7c28, "tg_Cyrl"},
+    {0x0428, "tg_Cyrl_TJ"}
+};
+
 ILCID_POSIX_ELEMENT_ARRAY(0x041e, th, th_TH)
 
-static const ILcidPosixElement ti[] = {
+ILCID_POSIX_SUBTABLE(ti) {
     {0x73,   "ti"},
+    {0x0473, "ti_ER"},
     {0x0873, "ti_ER"},
+    {0x0873, "ti_ET"},
     {0x0473, "ti_ET"}
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0442, tk, tk_TM)
-ILCID_POSIX_ELEMENT_ARRAY(0x0432, tn, tn_BW)
+
+ILCID_POSIX_SUBTABLE(tn) {
+    {0x32,   "tn"},
+    {0x0432, "tn_BW"},
+    {0x0432, "tn_ZA"}
+};
+
 ILCID_POSIX_ELEMENT_ARRAY(0x041f, tr, tr_TR)
+ILCID_POSIX_ELEMENT_ARRAY(0x0431, ts, ts_ZA)
 ILCID_POSIX_ELEMENT_ARRAY(0x0444, tt, tt_RU)
-ILCID_POSIX_ELEMENT_ARRAY(0x0480, ug, ug_CN)
+
+ILCID_POSIX_SUBTABLE(tzm) {
+    {0x5f,   "tzm"},
+    {0x7c5f, "tzm_Latn"},
+    {0x085f, "tzm_Latn_DZ"},
+    {0x045f, "tmz"}
+};
+
+ILCID_POSIX_SUBTABLE(ug) {
+    {0x80,   "ug"},
+    {0x0480, "ug_CN"},
+    {0x0480, "ug_Arab_CN"}
+};
+
 ILCID_POSIX_ELEMENT_ARRAY(0x0422, uk, uk_UA)
 
-static const ILcidPosixElement ur[] = {
+ILCID_POSIX_SUBTABLE(ur) {
     {0x20,   "ur"},
     {0x0820, "ur_IN"},
     {0x0420, "ur_PK"}
 };
 
-static const ILcidPosixElement uz[] = {
+ILCID_POSIX_SUBTABLE(uz) {
     {0x43,   "uz"},
     {0x0843, "uz_Cyrl_UZ"},  /* Cyrillic based */
-    {0x0843, "uz_Cyrl"},  /* Cyrillic based */
+    {0x7843, "uz_Cyrl"},  /* Cyrillic based */
     {0x0843, "uz_UZ"},  /* Cyrillic based */
     {0x0443, "uz_Latn_UZ"}, /* Latin based */
-    {0x0443, "uz_Latn"} /* Latin based */
+    {0x7c43, "uz_Latn"} /* Latin based */
+};
+
+ILCID_POSIX_SUBTABLE(ve) { /* TODO: Verify the country */
+    {0x33,   "ve"},
+    {0x0433, "ve_ZA"},
+    {0x0433, "ven_ZA"}
 };
 
-ILCID_POSIX_ELEMENT_ARRAY(0x0433, ve, ve_ZA)    /* TODO: Verify the country */
 ILCID_POSIX_ELEMENT_ARRAY(0x042a, vi, vi_VN)
 
-static const ILcidPosixElement wen[] = {
+ILCID_POSIX_SUBTABLE(wen) {
     {0x2E,   "wen"},
     {0x042E, "wen_DE"},
     {0x042E, "hsb_DE"},
-    {0x082E, "dsb_DE"}
+    {0x082E, "dsb_DE"},
+    {0x7C2E, "dsb"},
+    {0x2E,   "hsb"}
 };
 
 ILCID_POSIX_ELEMENT_ARRAY(0x0488, wo, wo_SN)
 ILCID_POSIX_ELEMENT_ARRAY(0x0434, xh, xh_ZA)
+ILCID_POSIX_ELEMENT_ARRAY(0x043d, yi, yi)
 ILCID_POSIX_ELEMENT_ARRAY(0x046a, yo, yo_NG)
 
-static const ILcidPosixElement zh[] = {
-    {0x04,   "zh"},
-    {0x0804, "zh_Hans_CN"},
-    {0x0804, "zh_Hans"},
+ILCID_POSIX_SUBTABLE(zh) {
+    {0x0004, "zh_Hans"},
+    {0x7804, "zh"},
     {0x0804, "zh_CN"},
+    {0x0804, "zh_Hans_CN"},
     {0x0c04, "zh_Hant_HK"},
     {0x0c04, "zh_HK"},
     {0x1404, "zh_Hant_MO"},
@@ -551,12 +673,15 @@ static const ILcidPosixElement zh[] = {
     {0x1004, "zh_Hans_SG"},
     {0x1004, "zh_SG"},
     {0x0404, "zh_Hant_TW"},
-    {0x0404, "zh_Hant"},
+    {0x7c04, "zh_Hant"},
     {0x0404, "zh_TW"},
     {0x30404,"zh_Hant_TW"},     /* Bopomofo order */
     {0x30404,"zh_TW"},          /* Bopomofo order */
+    {0x20004,"zh@collation=stroke"},
+    {0x20404,"zh_Hant@collation=stroke"},
     {0x20404,"zh_Hant_TW@collation=stroke"},
     {0x20404,"zh_TW@collation=stroke"},
+    {0x20804,"zh_Hans@collation=stroke"},
     {0x20804,"zh_Hans_CN@collation=stroke"},
     {0x20804,"zh_CN@collation=stroke"}
 };
@@ -564,14 +689,6 @@ static const ILcidPosixElement zh[] = {
 ILCID_POSIX_ELEMENT_ARRAY(0x0435, zu, zu_ZA)
 
 /* This must be static and grouped by LCID. */
-
-/* non-existent ISO-639-2 codes */
-/*
-0x466   Edo
-0x467   Fulfulde - Nigeria
-0x486   K'iche - Guatemala
-0x430   Sutu
-*/
 static const ILcidPosixMap gPosixIDmap[] = {
     ILCID_POSIX_MAP(af),    /*  af  Afrikaans                 0x36 */
     ILCID_POSIX_MAP(am),    /*  am  Amharic                   0x5e */
@@ -581,15 +698,16 @@ static const ILcidPosixMap gPosixIDmap[] = {
     ILCID_POSIX_MAP(az),    /*  az  Azerbaijani               0x2c */
     ILCID_POSIX_MAP(ba),    /*  ba  Bashkir                   0x6d */
     ILCID_POSIX_MAP(be),    /*  be  Belarusian                0x23 */
-    ILCID_POSIX_MAP(ber),   /*  ber Berber/Tamazight          0x5f */
+/*    ILCID_POSIX_MAP(ber),     ber Berber/Tamazight          0x5f */
     ILCID_POSIX_MAP(bg),    /*  bg  Bulgarian                 0x02 */
+    ILCID_POSIX_MAP(bin),   /*  bin Edo                       0x66 */
     ILCID_POSIX_MAP(bn),    /*  bn  Bengali; Bangla           0x45 */
     ILCID_POSIX_MAP(bo),    /*  bo  Tibetan                   0x51 */
     ILCID_POSIX_MAP(br),    /*  br  Breton                    0x7e */
     ILCID_POSIX_MAP(ca),    /*  ca  Catalan                   0x03 */
     ILCID_POSIX_MAP(chr),   /*  chr Cherokee                  0x5c */
     ILCID_POSIX_MAP(co),    /*  co  Corsican                  0x83 */
-    ILCID_POSIX_MAP(cs_CZ), /*  cs  Czech                     0x05 */
+    ILCID_POSIX_MAP(cs),    /*  cs  Czech                     0x05 */
     ILCID_POSIX_MAP(cy),    /*  cy  Welsh                     0x52 */
     ILCID_POSIX_MAP(da),    /*  da  Danish                    0x06 */
     ILCID_POSIX_MAP(de),    /*  de  German                    0x07 */
@@ -606,8 +724,10 @@ static const ILcidPosixMap gPosixIDmap[] = {
     ILCID_POSIX_MAP(fil),   /*  fil Filipino                  0x64 */
     ILCID_POSIX_MAP(fo),    /*  fo  Faroese                   0x38 */
     ILCID_POSIX_MAP(fr),    /*  fr  French                    0x0c */
+    ILCID_POSIX_MAP(fuv),   /*  fuv Fulfulde - Nigeria        0x67 */
     ILCID_POSIX_MAP(fy),    /*  fy  Frisian                   0x62 */
     ILCID_POSIX_MAP(ga),    /*  *   Gaelic (Ireland,Scotland) 0x3c */
+    ILCID_POSIX_MAP(gd),    /*  gd  Gaelic (United Kingdom)   0x91 */
     ILCID_POSIX_MAP(gl),    /*  gl  Galician                  0x56 */
     ILCID_POSIX_MAP(gn),    /*  gn  Guarani                   0x74 */
     ILCID_POSIX_MAP(gsw),   /*  gsw Alemanic/Alsatian/Swiss German 0x84 */
@@ -619,6 +739,7 @@ static const ILcidPosixMap gPosixIDmap[] = {
     ILCID_POSIX_MAP(hr),    /*  *   Croatian and others       0x1a */
     ILCID_POSIX_MAP(hu),    /*  hu  Hungarian                 0x0e */
     ILCID_POSIX_MAP(hy),    /*  hy  Armenian                  0x2b */
+    ILCID_POSIX_MAP(ibb),   /*  ibb Ibibio - Nigeria          0x69 */
     ILCID_POSIX_MAP(id),    /*  id  Indonesian (formerly in)  0x21 */
     ILCID_POSIX_MAP(ig),    /*  ig  Igbo                      0x70 */
     ILCID_POSIX_MAP(ii),    /*  ii  Sichuan Yi                0x78 */
@@ -662,6 +783,7 @@ static const ILcidPosixMap gPosixIDmap[] = {
     ILCID_POSIX_MAP(om),    /*  om  Oromo                     0x72 */
     ILCID_POSIX_MAP(or_IN), /*  or  Oriya                     0x48 */
     ILCID_POSIX_MAP(pa),    /*  pa  Punjabi                   0x46 */
+    ILCID_POSIX_MAP(pap),   /*  pap Papiamentu                0x79 */
     ILCID_POSIX_MAP(pl),    /*  pl  Polish                    0x15 */
     ILCID_POSIX_MAP(ps),    /*  ps  Pashto                    0x63 */
     ILCID_POSIX_MAP(pt),    /*  pt  Portuguese                0x16 */
@@ -683,6 +805,7 @@ static const ILcidPosixMap gPosixIDmap[] = {
     ILCID_POSIX_MAP(so),    /*  so  Somali                    0x77 */
     ILCID_POSIX_MAP(sq),    /*  sq  Albanian                  0x1c */
 /*    ILCID_POSIX_MAP(sr),    //  sr  Serbian                   0x1a */
+    ILCID_POSIX_MAP(st),    /*  st  Sutu                      0x30 */
     ILCID_POSIX_MAP(sv),    /*  sv  Swedish                   0x1d */
     ILCID_POSIX_MAP(sw),    /*  sw  Swahili                   0x41 */
     ILCID_POSIX_MAP(syr),   /*  syr Syriac                    0x5A */
@@ -694,7 +817,9 @@ static const ILcidPosixMap gPosixIDmap[] = {
     ILCID_POSIX_MAP(tk),    /*  tk  Turkmen                   0x42 */
     ILCID_POSIX_MAP(tn),    /*  tn  Tswana                    0x32 */
     ILCID_POSIX_MAP(tr),    /*  tr  Turkish                   0x1f */
+    ILCID_POSIX_MAP(ts),    /*  ts  Tsonga                    0x31 */
     ILCID_POSIX_MAP(tt),    /*  tt  Tatar                     0x44 */
+    ILCID_POSIX_MAP(tzm),   /*  tzm Tamazight                 0x5f */
     ILCID_POSIX_MAP(ug),    /*  ug  Uighur                    0x80 */
     ILCID_POSIX_MAP(uk),    /*  uk  Ukrainian                 0x22 */
     ILCID_POSIX_MAP(ur),    /*  ur  Urdu                      0x20 */
@@ -704,6 +829,7 @@ static const ILcidPosixMap gPosixIDmap[] = {
     ILCID_POSIX_MAP(wen),   /*  wen Sorbian                   0x2e */
     ILCID_POSIX_MAP(wo),    /*  wo  Wolof                     0x88 */
     ILCID_POSIX_MAP(xh),    /*  xh  Xhosa                     0x34 */
+    ILCID_POSIX_MAP(yi),    /*  yi  Yiddish                   0x3d */
     ILCID_POSIX_MAP(yo),    /*  yo  Yoruba                    0x6a */
     ILCID_POSIX_MAP(zh),    /*  zh  Chinese                   0x04 */
     ILCID_POSIX_MAP(zu),    /*  zu  Zulu                      0x35 */
@@ -793,18 +919,57 @@ getPosixID(const ILcidPosixMap *this_0, uint32_t hostID)
 //
 /////////////////////////////////////
 */
+#ifdef USE_WINDOWS_LOCALE_API
+/*
+ * Change the tag separator from '-' to '_'
+ */
+#define FIX_LOCALE_ID_TAG_SEPARATOR(buffer, len, i) \
+    for(i = 0; i < len; i++) \
+        if (buffer[i] == '-') buffer[i] = '_';
 
+/*
+ * Various language tags needs to be changed:
+ * quz -> qu
+ * prs -> fa
+ */
+#define FIX_LANGUAGE_ID_TAG(buffer, len) \
+    if (len >= 3) { \
+        if (buffer[0] == 'q' && buffer[1] == 'u' && buffer[2] == 'z') {\
+            buffer[2] = 0; \
+            uprv_strcat(buffer, buffer+3); \
+        } else if (buffer[0] == 'p' && buffer[1] == 'r' && buffer[2] == 's') {\
+            buffer[0] = 'f'; buffer[1] = 'a'; buffer[2] = 0; \
+            uprv_strcat(buffer, buffer+3); \
+        } \
+    }
+
+static char gPosixFromLCID[ULOC_FULLNAME_CAPACITY];
+#endif
 U_CAPI const char *
 uprv_convertToPosix(uint32_t hostid, UErrorCode* status)
 {
-    uint16_t langID = LANGUAGE_LCID(hostid);
-    uint32_t index;
+    uint16_t langID;
+    uint32_t localeIndex;
+#ifdef USE_WINDOWS_LOCALE_API
+    int32_t ret = 0;
 
-    for (index = 0; index < gLocaleCount; index++)
+    uprv_memset(gPosixFromLCID, 0, sizeof(gPosixFromLCID));
+
+    ret = GetLocaleInfoA(hostid, LOCALE_SNAME, (LPSTR)gPosixFromLCID, sizeof(gPosixFromLCID));
+    if (ret > 1) {
+        FIX_LOCALE_ID_TAG_SEPARATOR(gPosixFromLCID, (uint32_t)ret, localeIndex)
+        FIX_LANGUAGE_ID_TAG(gPosixFromLCID, ret)
+
+        return gPosixFromLCID;
+    }
+#endif
+    langID = LANGUAGE_LCID(hostid);
+
+    for (localeIndex = 0; localeIndex < gLocaleCount; localeIndex++)
     {
-        if (langID == gPosixIDmap[index].regionMaps->hostID)
+        if (langID == gPosixIDmap[localeIndex].regionMaps->hostID)
         {
-            return getPosixID(&gPosixIDmap[index], hostid);
+            return getPosixID(&gPosixIDmap[localeIndex], hostid);
         }
     }
 
