@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2009-2010, International Business Machines
+*   Copyright (C) 2009-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -25,8 +25,10 @@
 #include "mutex.h"
 #include "normalizer2impl.h"
 #include "uassert.h"
+#include "uhash.h"
 #include "uset_imp.h"
 #include "utrie2.h"
+#include "uvector.h"
 
 U_NAMESPACE_BEGIN
 
@@ -241,15 +243,24 @@ void ReorderingBuffer::insert(UChar32 c, uint8_t cc) {
 
 // Normalizer2Impl --------------------------------------------------------- ***
 
+struct CanonIterData : public UMemory {
+    CanonIterData(UErrorCode &errorCode);
+    ~CanonIterData();
+    void addToStartSet(UChar32 origin, UChar32 decompLead, UErrorCode &errorCode);
+    UTrie2 *trie;
+    UVector canonStartSets;  // contains UnicodeSet *
+};
+
 Normalizer2Impl::~Normalizer2Impl() {
     udata_close(memory);
     utrie2_close(normTrie);
     UTrie2Singleton(fcdTrieSingleton).deleteInstance();
+    delete (CanonIterData *)canonIterDataSingleton.fInstance;
 }
 
 UBool U_CALLCONV
 Normalizer2Impl::isAcceptable(void *context,
-                              const char *type, const char *name,
+                              const char * /* type */, const char * /*name*/,
                               const UDataInfo *pInfo) {
     if(
         pInfo->size>=20 &&
@@ -333,10 +344,15 @@ enumPropertyStartsRange(const void *context, UChar32 start, UChar32 /*end*/, uin
     return TRUE;
 }
 
+static uint32_t U_CALLCONV
+segmentStarterMapper(const void * /*context*/, uint32_t value) {
+    return value&CANON_NOT_SEGMENT_STARTER;
+}
+
 U_CDECL_END
 
 void
-Normalizer2Impl::addPropertyStarts(const USetAdder *sa, UErrorCode &errorCode) const {
+Normalizer2Impl::addPropertyStarts(const USetAdder *sa, UErrorCode & /*errorCode*/) const {
     /* add the start code point of each same-value range of each trie */
     utrie2_enum(normTrie, NULL, enumPropertyStartsRange, sa);
 
@@ -348,6 +364,16 @@ Normalizer2Impl::addPropertyStarts(const USetAdder *sa, UErrorCode &errorCode) c
     sa->add(sa->set, Hangul::HANGUL_LIMIT); /* add Hangul+1 to continue with other properties */
 }
 
+void
+Normalizer2Impl::addCanonIterPropertyStarts(const USetAdder *sa, UErrorCode &errorCode) const {
+    /* add the start code point of each same-value range of the canonical iterator data trie */
+    if(ensureCanonIterData(errorCode)) {
+        // currently only used for the SEGMENT_STARTER property
+        utrie2_enum(((CanonIterData *)canonIterDataSingleton.fInstance)->trie,
+                    segmentStarterMapper, enumPropertyStartsRange, sa);
+    }
+}
+
 const UChar *
 Normalizer2Impl::copyLowPrefixFromNulTerminated(const UChar *src,
                                                 UChar32 minNeedDataCP,
@@ -544,8 +570,10 @@ Normalizer2Impl::getDecomposition(UChar32 c, UChar buffer[4], int32_t &length) c
 
 void Normalizer2Impl::decomposeAndAppend(const UChar *src, const UChar *limit,
                                          UBool doDecompose,
+                                         UnicodeString &safeMiddle,
                                          ReorderingBuffer &buffer,
                                          UErrorCode &errorCode) const {
+    buffer.copyReorderableSuffixTo(safeMiddle);
     if(doDecompose) {
         decompose(src, limit, &buffer, errorCode);
         return;
@@ -558,6 +586,9 @@ void Normalizer2Impl::decomposeAndAppend(const UChar *src, const UChar *limit,
         prevCC=cc;
         cc=getCC(iter.next16());
     };
+    if(limit==NULL) {  // appendZeroCC() needs limit!=NULL
+        limit=u_strchr(iter.codePointStart, 0);
+    }
     buffer.append(src, (int32_t)(iter.codePointStart-src), firstCC, prevCC, errorCode) &&
         buffer.appendZeroCC(iter.codePointStart, limit, errorCode);
 }
@@ -643,8 +674,8 @@ int32_t Normalizer2Impl::combine(const uint16_t *list, UChar32 trail) {
         // trail character is 3400..10FFFF
         // result entry has 3 units
         key1=(uint16_t)(COMP_1_TRAIL_LIMIT+
-                        ((trail>>COMP_1_TRAIL_SHIFT))&
-                         ~COMP_1_TRIPLE);
+                        (((trail>>COMP_1_TRAIL_SHIFT))&
+                          ~COMP_1_TRIPLE));
         uint16_t key2=(uint16_t)(trail<<COMP_2_TRAIL_SHIFT);
         uint16_t secondUnit;
         for(;;) {
@@ -670,6 +701,30 @@ int32_t Normalizer2Impl::combine(const uint16_t *list, UChar32 trail) {
     return -1;
 }
 
+/**
+  * @param list some character's compositions list
+  * @param set recursively receives the composites from these compositions
+  */
+void Normalizer2Impl::addComposites(const uint16_t *list, UnicodeSet &set) const {
+    uint16_t firstUnit;
+    int32_t compositeAndFwd;
+    do {
+        firstUnit=*list;
+        if((firstUnit&COMP_1_TRIPLE)==0) {
+            compositeAndFwd=list[1];
+            list+=2;
+        } else {
+            compositeAndFwd=(((int32_t)list[1]&~COMP_2_TRAIL_MASK)<<16)|list[2];
+            list+=3;
+        }
+        UChar32 composite=compositeAndFwd>>1;
+        if((compositeAndFwd&1)!=0) {
+            addComposites(getCompositionsListForComposite(getNorm16(composite)), set);
+        }
+        set.add(composite);
+    } while((firstUnit&COMP_1_LAST_TUPLE)==0);
+}
+
 /*
  * Recomposes the buffer text starting at recomposeStartIndex
  * (which is in NFD - decomposed and canonically ordered),
@@ -854,17 +909,6 @@ Normalizer2Impl::compose(const UChar *src, const UChar *limit,
                          UBool doCompose,
                          ReorderingBuffer &buffer,
                          UErrorCode &errorCode) const {
-    UChar32 minNoMaybeCP=minCompNoMaybeCP;
-    if(limit==NULL) {
-        src=copyLowPrefixFromNulTerminated(src, minNoMaybeCP,
-                                           doCompose ? &buffer : NULL,
-                                           errorCode);
-        if(U_FAILURE(errorCode)) {
-            return FALSE;
-        }
-        limit=u_strchr(src, 0);
-    }
-
     /*
      * prevBoundary points to the last character before the current one
      * that has a composition boundary before it with ccc==0 and quick check "yes".
@@ -878,6 +922,21 @@ Normalizer2Impl::compose(const UChar *src, const UChar *limit,
      * must correspond 1:1 to destination units at the end of the destination buffer.
      */
     const UChar *prevBoundary=src;
+    UChar32 minNoMaybeCP=minCompNoMaybeCP;
+    if(limit==NULL) {
+        src=copyLowPrefixFromNulTerminated(src, minNoMaybeCP,
+                                           doCompose ? &buffer : NULL,
+                                           errorCode);
+        if(U_FAILURE(errorCode)) {
+            return FALSE;
+        }
+        if(prevBoundary<src) {
+            // Set prevBoundary to the last character in the prefix.
+            prevBoundary=src-1;
+        }
+        limit=u_strchr(src, 0);
+    }
+
     const UChar *prevSrc;
     UChar32 c=0;
     uint16_t norm16=0;
@@ -1108,18 +1167,22 @@ const UChar *
 Normalizer2Impl::composeQuickCheck(const UChar *src, const UChar *limit,
                                    UBool onlyContiguous,
                                    UNormalizationCheckResult *pQCResult) const {
-    UChar32 minNoMaybeCP=minCompNoMaybeCP;
-    if(limit==NULL) {
-        UErrorCode errorCode=U_ZERO_ERROR;
-        src=copyLowPrefixFromNulTerminated(src, minNoMaybeCP, NULL, errorCode);
-        limit=u_strchr(src, 0);
-    }
-
     /*
      * prevBoundary points to the last character before the current one
      * that has a composition boundary before it with ccc==0 and quick check "yes".
      */
     const UChar *prevBoundary=src;
+    UChar32 minNoMaybeCP=minCompNoMaybeCP;
+    if(limit==NULL) {
+        UErrorCode errorCode=U_ZERO_ERROR;
+        src=copyLowPrefixFromNulTerminated(src, minNoMaybeCP, NULL, errorCode);
+        if(prevBoundary<src) {
+            // Set prevBoundary to the last character in the prefix.
+            prevBoundary=src-1;
+        }
+        limit=u_strchr(src, 0);
+    }
+
     const UChar *prevSrc;
     UChar32 c=0;
     uint16_t norm16=0;
@@ -1213,6 +1276,7 @@ Normalizer2Impl::composeQuickCheck(const UChar *src, const UChar *limit,
 void Normalizer2Impl::composeAndAppend(const UChar *src, const UChar *limit,
                                        UBool doCompose,
                                        UBool onlyContiguous,
+                                       UnicodeString &safeMiddle,
                                        ReorderingBuffer &buffer,
                                        UErrorCode &errorCode) const {
     if(!buffer.isEmpty()) {
@@ -1220,9 +1284,10 @@ void Normalizer2Impl::composeAndAppend(const UChar *src, const UChar *limit,
         if(src!=firstStarterInSrc) {
             const UChar *lastStarterInDest=findPreviousCompBoundary(buffer.getStart(),
                                                                     buffer.getLimit());
-            UnicodeString middle(lastStarterInDest,
-                                 (int32_t)(buffer.getLimit()-lastStarterInDest));
-            buffer.removeSuffix((int32_t)(buffer.getLimit()-lastStarterInDest));
+            int32_t destSuffixLength=(int32_t)(buffer.getLimit()-lastStarterInDest);
+            UnicodeString middle(lastStarterInDest, destSuffixLength);
+            buffer.removeSuffix(destSuffixLength);
+            safeMiddle=middle;
             middle.append(src, (int32_t)(firstStarterInSrc-src));
             const UChar *middleStart=middle.getBuffer();
             compose(middleStart, middleStart+middle.length(), onlyContiguous,
@@ -1236,6 +1301,9 @@ void Normalizer2Impl::composeAndAppend(const UChar *src, const UChar *limit,
     if(doCompose) {
         compose(src, limit, onlyContiguous, TRUE, buffer, errorCode);
     } else {
+        if(limit==NULL) {  // appendZeroCC() needs limit!=NULL
+            limit=u_strchr(src, 0);
+        }
         buffer.appendZeroCC(src, limit, errorCode);
     }
 }
@@ -1356,7 +1424,7 @@ enumRangeHandler(const void *context, UChar32 start, UChar32 end, uint32_t value
 // Collect (OR together) the FCD values for a range of supplementary characters,
 // for their lead surrogate code unit.
 static UBool U_CALLCONV
-enumRangeOrValue(const void *context, UChar32 start, UChar32 end, uint32_t value) {
+enumRangeOrValue(const void *context, UChar32 /*start*/, UChar32 /*end*/, uint32_t value) {
     *((uint32_t *)context)|=value;
     return TRUE;
 }
@@ -1450,11 +1518,24 @@ const UChar *
 Normalizer2Impl::makeFCD(const UChar *src, const UChar *limit,
                          ReorderingBuffer *buffer,
                          UErrorCode &errorCode) const {
+    // Tracks the last FCD-safe boundary, before lccc=0 or after properly-ordered tccc<=1.
+    // Similar to the prevBoundary in the compose() implementation.
+    const UChar *prevBoundary=src;
+    int32_t prevFCD16=0;
     if(limit==NULL) {
         src=copyLowPrefixFromNulTerminated(src, MIN_CCC_LCCC_CP, buffer, errorCode);
         if(U_FAILURE(errorCode)) {
             return src;
         }
+        if(prevBoundary<src) {
+            prevBoundary=src;
+            // We know that the previous character's lccc==0.
+            // Fetching the fcd16 value was deferred for this below-U+0300 code point.
+            prevFCD16=getFCD16FromSingleLead(*(src-1));
+            if(prevFCD16>1) {
+                --prevBoundary;
+            }
+        }
         limit=u_strchr(src, 0);
     }
 
@@ -1466,12 +1547,8 @@ Normalizer2Impl::makeFCD(const UChar *src, const UChar *limit,
 
     const UTrie2 *trie=fcdTrie();
 
-    // Tracks the last FCD-safe boundary, before lccc=0 or after properly-ordered tccc<=1.
-    // Similar to the prevBoundary in the compose() implementation.
-    const UChar *prevBoundary=src;
     const UChar *prevSrc;
     UChar32 c=0;
-    int32_t prevFCD16=0;
     uint16_t fcd16=0;
 
     for(;;) {
@@ -1583,6 +1660,7 @@ Normalizer2Impl::makeFCD(const UChar *src, const UChar *limit,
 
 void Normalizer2Impl::makeFCDAndAppend(const UChar *src, const UChar *limit,
                                        UBool doMakeFCD,
+                                       UnicodeString &safeMiddle,
                                        ReorderingBuffer &buffer,
                                        UErrorCode &errorCode) const {
     if(!buffer.isEmpty()) {
@@ -1590,9 +1668,10 @@ void Normalizer2Impl::makeFCDAndAppend(const UChar *src, const UChar *limit,
         if(src!=firstBoundaryInSrc) {
             const UChar *lastBoundaryInDest=findPreviousFCDBoundary(buffer.getStart(),
                                                                     buffer.getLimit());
-            UnicodeString middle(lastBoundaryInDest,
-                                 (int32_t)(buffer.getLimit()-lastBoundaryInDest));
-            buffer.removeSuffix((int32_t)(buffer.getLimit()-lastBoundaryInDest));
+            int32_t destSuffixLength=(int32_t)(buffer.getLimit()-lastBoundaryInDest);
+            UnicodeString middle(lastBoundaryInDest, destSuffixLength);
+            buffer.removeSuffix(destSuffixLength);
+            safeMiddle=middle;
             middle.append(src, (int32_t)(firstBoundaryInSrc-src));
             const UChar *middleStart=middle.getBuffer();
             makeFCD(middleStart, middleStart+middle.length(), &buffer, errorCode);
@@ -1605,6 +1684,9 @@ void Normalizer2Impl::makeFCDAndAppend(const UChar *src, const UChar *limit,
     if(doMakeFCD) {
         makeFCD(src, limit, &buffer, errorCode);
     } else {
+        if(limit==NULL) {  // appendZeroCC() needs limit!=NULL
+            limit=u_strchr(src, 0);
+        }
         buffer.appendZeroCC(src, limit, errorCode);
     }
 }
@@ -1627,6 +1709,217 @@ const UChar *Normalizer2Impl::findNextFCDBoundary(const UChar *p, const UChar *l
     return iter.codePointStart;
 }
 
+// CanonicalIterator data -------------------------------------------------- ***
+
+CanonIterData::CanonIterData(UErrorCode &errorCode) :
+        trie(utrie2_open(0, 0, &errorCode)),
+        canonStartSets(uhash_deleteUObject, NULL, errorCode) {}
+
+CanonIterData::~CanonIterData() {
+    utrie2_close(trie);
+}
+
+void CanonIterData::addToStartSet(UChar32 origin, UChar32 decompLead, UErrorCode &errorCode) {
+    uint32_t canonValue=utrie2_get32(trie, decompLead);
+    if((canonValue&(CANON_HAS_SET|CANON_VALUE_MASK))==0 && origin!=0) {
+        // origin is the first character whose decomposition starts with
+        // the character for which we are setting the value.
+        utrie2_set32(trie, decompLead, canonValue|origin, &errorCode);
+    } else {
+        // origin is not the first character, or it is U+0000.
+        UnicodeSet *set;
+        if((canonValue&CANON_HAS_SET)==0) {
+            set=new UnicodeSet;
+            if(set==NULL) {
+                errorCode=U_MEMORY_ALLOCATION_ERROR;
+                return;
+            }
+            UChar32 firstOrigin=(UChar32)(canonValue&CANON_VALUE_MASK);
+            canonValue=(canonValue&~CANON_VALUE_MASK)|CANON_HAS_SET|(uint32_t)canonStartSets.size();
+            utrie2_set32(trie, decompLead, canonValue, &errorCode);
+            canonStartSets.addElement(set, errorCode);
+            if(firstOrigin!=0) {
+                set->add(firstOrigin);
+            }
+        } else {
+            set=(UnicodeSet *)canonStartSets[(int32_t)(canonValue&CANON_VALUE_MASK)];
+        }
+        set->add(origin);
+    }
+}
+
+class CanonIterDataSingleton {
+public:
+    CanonIterDataSingleton(SimpleSingleton &s, Normalizer2Impl &ni, UErrorCode &ec) :
+        singleton(s), impl(ni), errorCode(ec) {}
+    CanonIterData *getInstance(UErrorCode &errorCode) {
+        void *duplicate;
+        CanonIterData *instance=
+            (CanonIterData *)singleton.getInstance(createInstance, this, duplicate, errorCode);
+        delete (CanonIterData *)duplicate;
+        return instance;
+    }
+    static void *createInstance(const void *context, UErrorCode &errorCode);
+    UBool rangeHandler(UChar32 start, UChar32 end, uint32_t value) {
+        if(value!=0) {
+            impl.makeCanonIterDataFromNorm16(start, end, (uint16_t)value, *newData, errorCode);
+        }
+        return U_SUCCESS(errorCode);
+    }
+
+private:
+    SimpleSingleton &singleton;
+    Normalizer2Impl &impl;
+    CanonIterData *newData;
+    UErrorCode &errorCode;
+};
+
+U_CDECL_BEGIN
+
+// Call Normalizer2Impl::makeCanonIterDataFromNorm16() for a range of same-norm16 characters.
+static UBool U_CALLCONV
+enumCIDRangeHandler(const void *context, UChar32 start, UChar32 end, uint32_t value) {
+    return ((CanonIterDataSingleton *)context)->rangeHandler(start, end, value);
+}
+
+U_CDECL_END
+
+void *CanonIterDataSingleton::createInstance(const void *context, UErrorCode &errorCode) {
+    CanonIterDataSingleton *me=(CanonIterDataSingleton *)context;
+    me->newData=new CanonIterData(errorCode);
+    if(me->newData==NULL) {
+        errorCode=U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
+    if(U_SUCCESS(errorCode)) {
+        utrie2_enum(me->impl.getNormTrie(), NULL, enumCIDRangeHandler, me);
+        utrie2_freeze(me->newData->trie, UTRIE2_32_VALUE_BITS, &errorCode);
+        if(U_SUCCESS(errorCode)) {
+            return me->newData;
+        }
+    }
+    delete me->newData;
+    return NULL;
+}
+
+void Normalizer2Impl::makeCanonIterDataFromNorm16(UChar32 start, UChar32 end, uint16_t norm16,
+                                                  CanonIterData &newData,
+                                                  UErrorCode &errorCode) const {
+    if(norm16==0 || (minYesNo<=norm16 && norm16<minNoNo)) {
+        // Inert, or 2-way mapping (including Hangul syllable).
+        // We do not write a canonStartSet for any yesNo character.
+        // Composites from 2-way mappings are added at runtime from the
+        // starter's compositions list, and the other characters in
+        // 2-way mappings get CANON_NOT_SEGMENT_STARTER set because they are
+        // "maybe" characters.
+        return;
+    }
+    for(UChar32 c=start; c<=end; ++c) {
+        uint32_t oldValue=utrie2_get32(newData.trie, c);
+        uint32_t newValue=oldValue;
+        if(norm16>=minMaybeYes) {
+            // not a segment starter if it occurs in a decomposition or has cc!=0
+            newValue|=CANON_NOT_SEGMENT_STARTER;
+            if(norm16<MIN_NORMAL_MAYBE_YES) {
+                newValue|=CANON_HAS_COMPOSITIONS;
+            }
+        } else if(norm16<minYesNo) {
+            newValue|=CANON_HAS_COMPOSITIONS;
+        } else {
+            // c has a one-way decomposition
+            UChar32 c2=c;
+            uint16_t norm16_2=norm16;
+            while(limitNoNo<=norm16_2 && norm16_2<minMaybeYes) {
+                c2=mapAlgorithmic(c2, norm16_2);
+                norm16_2=getNorm16(c2);
+            }
+            if(minYesNo<=norm16_2 && norm16_2<limitNoNo) {
+                // c decomposes, get everything from the variable-length extra data
+                const uint16_t *mapping=getMapping(norm16_2);
+                uint16_t firstUnit=*mapping++;
+                int32_t length=firstUnit&MAPPING_LENGTH_MASK;
+                if((firstUnit&MAPPING_HAS_CCC_LCCC_WORD)!=0) {
+                    if(c==c2 && (*mapping&0xff)!=0) {
+                        newValue|=CANON_NOT_SEGMENT_STARTER;  // original c has cc!=0
+                    }
+                    ++mapping;
+                }
+                // Skip empty mappings (no characters in the decomposition).
+                if(length!=0) {
+                    // add c to first code point's start set
+                    int32_t i=0;
+                    U16_NEXT_UNSAFE(mapping, i, c2);
+                    newData.addToStartSet(c, c2, errorCode);
+                    // Set CANON_NOT_SEGMENT_STARTER for each remaining code point of a
+                    // one-way mapping. A 2-way mapping is possible here after
+                    // intermediate algorithmic mapping.
+                    if(norm16_2>=minNoNo) {
+                        while(i<length) {
+                            U16_NEXT_UNSAFE(mapping, i, c2);
+                            uint32_t c2Value=utrie2_get32(newData.trie, c2);
+                            if((c2Value&CANON_NOT_SEGMENT_STARTER)==0) {
+                                utrie2_set32(newData.trie, c2, c2Value|CANON_NOT_SEGMENT_STARTER,
+                                             &errorCode);
+                            }
+                        }
+                    }
+                }
+            } else {
+                // c decomposed to c2 algorithmically; c has cc==0
+                newData.addToStartSet(c, c2, errorCode);
+            }
+        }
+        if(newValue!=oldValue) {
+            utrie2_set32(newData.trie, c, newValue, &errorCode);
+        }
+    }
+}
+
+UBool Normalizer2Impl::ensureCanonIterData(UErrorCode &errorCode) const {
+    // Logically const: Synchronized instantiation.
+    Normalizer2Impl *me=const_cast<Normalizer2Impl *>(this);
+    CanonIterDataSingleton(me->canonIterDataSingleton, *me, errorCode).getInstance(errorCode);
+    return U_SUCCESS(errorCode);
+}
+
+int32_t Normalizer2Impl::getCanonValue(UChar32 c) const {
+    return (int32_t)utrie2_get32(((CanonIterData *)canonIterDataSingleton.fInstance)->trie, c);
+}
+
+const UnicodeSet &Normalizer2Impl::getCanonStartSet(int32_t n) const {
+    return *(const UnicodeSet *)(
+        ((CanonIterData *)canonIterDataSingleton.fInstance)->canonStartSets[n]);
+}
+
+UBool Normalizer2Impl::isCanonSegmentStarter(UChar32 c) const {
+    return getCanonValue(c)>=0;
+}
+
+UBool Normalizer2Impl::getCanonStartSet(UChar32 c, UnicodeSet &set) const {
+    int32_t canonValue=getCanonValue(c)&~CANON_NOT_SEGMENT_STARTER;
+    if(canonValue==0) {
+        return FALSE;
+    }
+    set.clear();
+    int32_t value=canonValue&CANON_VALUE_MASK;
+    if((canonValue&CANON_HAS_SET)!=0) {
+        set.addAll(getCanonStartSet(value));
+    } else if(value!=0) {
+        set.add(value);
+    }
+    if((canonValue&CANON_HAS_COMPOSITIONS)!=0) {
+        uint16_t norm16=getNorm16(c);
+        if(norm16==JAMO_L) {
+            UChar32 syllable=
+                (UChar32)(Hangul::HANGUL_BASE+(c-Hangul::JAMO_L_BASE)*Hangul::JAMO_VT_COUNT);
+            set.add(syllable, syllable+Hangul::JAMO_VT_COUNT-1);
+        } else {
+            addComposites(getCompositionsList(norm16), set);
+        }
+    }
+    return TRUE;
+}
+
 U_NAMESPACE_END
 
 // Normalizer2 data swapping ----------------------------------------------- ***
@@ -1678,7 +1971,7 @@ unorm2_swap(const UDataSwapper *ds,
 
     if(length>=0) {
         length-=headerSize;
-        if(length<sizeof(indexes)) {
+        if(length<(int32_t)sizeof(indexes)) {
             udata_printError(ds, "unorm2_swap(): too few bytes (%d after header) for Normalizer2 data\n",
                              length);
             *pErrorCode=U_INDEX_OUTOFBOUNDS_ERROR;
