@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2009-2010, International Business Machines
+*   Copyright (C) 2009-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -31,6 +31,8 @@
 
 U_NAMESPACE_BEGIN
 
+struct CanonIterData;
+
 class Hangul {
 public:
     /* Korean Hangul and Jamo constants */
@@ -149,6 +151,9 @@ public:
         reorderStart=limit=newLimit;
         lastCC=0;
     }
+    void copyReorderableSuffixTo(UnicodeString &s) const {
+        s.setTo(reorderStart, (int32_t)(limit-reorderStart));
+    }
 private:
     /*
      * TODO: Revisit whether it makes sense to track reorderStart.
@@ -193,18 +198,22 @@ class U_COMMON_API Normalizer2Impl : public UMemory {
 public:
     Normalizer2Impl() : memory(NULL), normTrie(NULL) {
         fcdTrieSingleton.fInstance=NULL;
+        canonIterDataSingleton.fInstance=NULL;
     }
     ~Normalizer2Impl();
 
     void load(const char *packageName, const char *name, UErrorCode &errorCode);
 
     void addPropertyStarts(const USetAdder *sa, UErrorCode &errorCode) const;
+    void addCanonIterPropertyStarts(const USetAdder *sa, UErrorCode &errorCode) const;
 
     // low-level properties ------------------------------------------------ ***
 
     const UTrie2 *getNormTrie() const { return normTrie; }
     const UTrie2 *getFCDTrie(UErrorCode &errorCode) const ;
 
+    UBool ensureCanonIterData(UErrorCode &errorCode) const;
+
     uint16_t getNorm16(UChar32 c) const { return UTRIE2_GET16(normTrie, c); }
 
     UNormalizationCheckResult getCompQuickCheck(uint16_t norm16) const {
@@ -246,6 +255,9 @@ public:
     void setFCD16FromNorm16(UChar32 start, UChar32 end, uint16_t norm16,
                             UTrie2 *newFCDTrie, UErrorCode &errorCode) const;
 
+    void makeCanonIterDataFromNorm16(UChar32 start, UChar32 end, uint16_t norm16,
+                                     CanonIterData &newData, UErrorCode &errorCode) const;
+
     /**
      * Get the decomposition for one code point.
      * @param c code point
@@ -255,6 +267,9 @@ public:
      */
     const UChar *getDecomposition(UChar32 c, UChar buffer[4], int32_t &length) const;
 
+    UBool isCanonSegmentStarter(UChar32 c) const;
+    UBool getCanonStartSet(UChar32 c, UnicodeSet &set) const;
+
     enum {
         MIN_CCC_LCCC_CP=0x300
     };
@@ -316,6 +331,7 @@ public:
                            ReorderingBuffer *buffer, UErrorCode &errorCode) const;
     void decomposeAndAppend(const UChar *src, const UChar *limit,
                             UBool doDecompose,
+                            UnicodeString &safeMiddle,
                             ReorderingBuffer &buffer,
                             UErrorCode &errorCode) const;
     UBool compose(const UChar *src, const UChar *limit,
@@ -329,12 +345,14 @@ public:
     void composeAndAppend(const UChar *src, const UChar *limit,
                           UBool doCompose,
                           UBool onlyContiguous,
+                          UnicodeString &safeMiddle,
                           ReorderingBuffer &buffer,
                           UErrorCode &errorCode) const;
     const UChar *makeFCD(const UChar *src, const UChar *limit,
                          ReorderingBuffer *buffer, UErrorCode &errorCode) const;
     void makeFCDAndAppend(const UChar *src, const UChar *limit,
                           UBool doMakeFCD,
+                          UnicodeString &safeMiddle,
                           ReorderingBuffer &buffer,
                           UErrorCode &errorCode) const;
 
@@ -426,6 +444,15 @@ private:
             (*list&MAPPING_LENGTH_MASK)+  // + mapping length
             ((*list>>7)&1);  // +1 if MAPPING_HAS_CCC_LCCC_WORD
     }
+    /**
+     * @param c code point must have compositions
+     * @return compositions list pointer
+     */
+    const uint16_t *getCompositionsList(uint16_t norm16) const {
+        return isDecompYes(norm16) ?
+                getCompositionsListForDecompYes(norm16) :
+                getCompositionsListForComposite(norm16);
+    }
 
     const UChar *copyLowPrefixFromNulTerminated(const UChar *src,
                                                 UChar32 minNeedDataCP,
@@ -437,6 +464,7 @@ private:
                     ReorderingBuffer &buffer, UErrorCode &errorCode) const;
 
     static int32_t combine(const uint16_t *list, UChar32 trail);
+    void addComposites(const uint16_t *list, UnicodeSet &set) const;
     void recompose(ReorderingBuffer &buffer, int32_t recomposeStartIndex,
                    UBool onlyContiguous) const;
 
@@ -449,6 +477,9 @@ private:
     const UChar *findPreviousFCDBoundary(const UChar *start, const UChar *p) const;
     const UChar *findNextFCDBoundary(const UChar *p, const UChar *limit) const;
 
+    int32_t getCanonValue(UChar32 c) const;
+    const UnicodeSet &getCanonStartSet(int32_t n) const;
+
     UDataMemory *memory;
     UVersionInfo dataVersion;
 
@@ -467,8 +498,15 @@ private:
     const uint16_t *extraData;  // mappings and/or compositions for yesYes, yesNo & noNo characters
 
     SimpleSingleton fcdTrieSingleton;
+    SimpleSingleton canonIterDataSingleton;
 };
 
+// bits in canonIterData
+#define CANON_NOT_SEGMENT_STARTER 0x80000000
+#define CANON_HAS_COMPOSITIONS 0x40000000
+#define CANON_HAS_SET 0x200000
+#define CANON_VALUE_MASK 0x1fffff
+
 /**
  * ICU-internal shortcut for quick access to standard Unicode normalization.
  */
