@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 1999-2008, International Business Machines
+*   Copyright (C) 1999-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -15,6 +15,7 @@
 #include "ruleiter.h"
 #include "cmemory.h"
 #include "cstring.h"
+#include "patternprops.h"
 #include "uhash.h"
 #include "util.h"
 #include "uvector.h"
@@ -1926,7 +1927,7 @@ escapeUnprintable) {
         break;
     default:
         // Escape whitespace
-        if (uprv_isRuleWhiteSpace(c)) {
+        if (PatternProps::isWhiteSpace(c)) {
             buf.append(BACKSLASH);
         }
         break;
@@ -2208,7 +2209,7 @@ int32_t UnicodeSet::spanUTF8(const char *s, int32_t length, USetSpanCondition sp
         return (int32_t)(bmpSet->spanUTF8(s0, length, spanCondition)-s0);
     }
     if(length<0) {
-        length=uprv_strlen(s);
+        length=(int32_t)uprv_strlen(s);
     }
     if(length==0) {
         return 0;
@@ -2249,7 +2250,7 @@ int32_t UnicodeSet::spanBackUTF8(const char *s, int32_t length, USetSpanConditio
         return bmpSet->spanBackUTF8(s0, length, spanCondition);
     }
     if(length<0) {
-        length=uprv_strlen(s);
+        length=(int32_t)uprv_strlen(s);
     }
     if(length==0) {
         return 0;
