@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2007, International Business Machines
+*   Copyright (C) 1999-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -33,12 +33,14 @@
 #include "uvector.h"
 #include "uprops.h"
 #include "propname.h"
-#include "unormimp.h"
+#include "normalizer2impl.h"
 #include "ucase.h"
 #include "ubidi_props.h"
 #include "uinvchar.h"
+#include "uprops.h"
 #include "charstr.h"
 #include "cstring.h"
+#include "mutex.h"
 #include "umutex.h"
 #include "uassert.h"
 #include "hash.h"
@@ -91,10 +93,44 @@ static const char ASSIGNED[] = "Assigned"; // [:^Cn:]
  */
 //static const UChar CATEGORY_CLOSE[] = {COLON, SET_CLOSE, 0x0000}; /* ":]" */
 
+// Cached sets ------------------------------------------------------------- ***
+
+U_CDECL_BEGIN
+static UBool U_CALLCONV uset_cleanup();
+U_CDECL_END
+
+// Not a TriStateSingletonWrapper because we think the UnicodeSet constructor
+// can only fail with an out-of-memory error
+// if we have a correct pattern and the properties data is hardcoded and always available.
+class UnicodeSetSingleton : public SimpleSingletonWrapper<UnicodeSet> {
+public:
+    UnicodeSetSingleton(SimpleSingleton &s, const char *pattern) :
+            SimpleSingletonWrapper<UnicodeSet>(s), fPattern(pattern) {}
+    UnicodeSet *getInstance(UErrorCode &errorCode) {
+        return SimpleSingletonWrapper<UnicodeSet>::getInstance(createInstance, fPattern, errorCode);
+    }
+private:
+    static void *createInstance(const void *context, UErrorCode &errorCode) {
+        UnicodeString pattern((const char *)context, -1, US_INV);
+        UnicodeSet *set=new UnicodeSet(pattern, errorCode);
+        if(set==NULL) {
+            errorCode=U_MEMORY_ALLOCATION_ERROR;
+            return NULL;
+        }
+        set->freeze();
+        ucln_common_registerCleanup(UCLN_COMMON_USET, uset_cleanup);
+        return set;
+    }
+
+    const char *fPattern;
+};
+
 U_CDECL_BEGIN
 
 static UnicodeSet *INCLUSIONS[UPROPS_SRC_COUNT] = { NULL }; // cached getInclusions()
 
+STATIC_SIMPLE_SINGLETON(uni32Singleton);
+
 //----------------------------------------------------------------
 // Inclusions list
 //----------------------------------------------------------------
@@ -128,7 +164,7 @@ static UBool U_CALLCONV uset_cleanup(void) {
             INCLUSIONS[i] = NULL;
         }
     }
-
+    UnicodeSetSingleton(uni32Singleton, NULL).deleteInstance();
     return TRUE;
 }
 
@@ -136,7 +172,14 @@ U_CDECL_END
 
 U_NAMESPACE_BEGIN
 
-static const UnicodeSet* getInclusions(int32_t src, UErrorCode &status) {
+/*
+Reduce excessive reallocation, and make it easier to detect initialization
+problems.
+Usually you don't see smaller sets than this for Unicode 5.0.
+*/
+#define DEFAULT_INCLUSION_CAPACITY 3072
+
+const UnicodeSet* UnicodeSet::getInclusions(int32_t src, UErrorCode &status) {
     UBool needInit;
     UMTX_CHECK(NULL, (INCLUSIONS[src] == NULL), needInit);
     if (needInit) {
@@ -146,10 +189,11 @@ static const UnicodeSet* getInclusions(int32_t src, UErrorCode &status) {
             _set_add,
             _set_addRange,
             _set_addString,
-            NULL // don't need remove()
+            NULL, // don't need remove()
+            NULL // don't need removeRange()
         };
-
         if (incl != NULL) {
+            incl->ensureCapacity(DEFAULT_INCLUSION_CAPACITY, status);
             switch(src) {
             case UPROPS_SRC_CHAR:
                 uchar_addPropertyStarts(&sa, &status);
@@ -161,19 +205,49 @@ static const UnicodeSet* getInclusions(int32_t src, UErrorCode &status) {
                 uchar_addPropertyStarts(&sa, &status);
                 upropsvec_addPropertyStarts(&sa, &status);
                 break;
-            case UPROPS_SRC_HST:
-                uhst_addPropertyStarts(&sa, &status);
-                break;
 #if !UCONFIG_NO_NORMALIZATION
-            case UPROPS_SRC_NORM:
-                unorm_addPropertyStarts(&sa, &status);
+            case UPROPS_SRC_CASE_AND_NORM: {
+                const Normalizer2Impl *impl=Normalizer2Factory::getNFCImpl(status);
+                if(U_SUCCESS(status)) {
+                    impl->addPropertyStarts(&sa, status);
+                }
+                ucase_addPropertyStarts(ucase_getSingleton(), &sa, &status);
                 break;
+            }
+            case UPROPS_SRC_NFC: {
+                const Normalizer2Impl *impl=Normalizer2Factory::getNFCImpl(status);
+                if(U_SUCCESS(status)) {
+                    impl->addPropertyStarts(&sa, status);
+                }
+                break;
+            }
+            case UPROPS_SRC_NFKC: {
+                const Normalizer2Impl *impl=Normalizer2Factory::getNFKCImpl(status);
+                if(U_SUCCESS(status)) {
+                    impl->addPropertyStarts(&sa, status);
+                }
+                break;
+            }
+            case UPROPS_SRC_NFKC_CF: {
+                const Normalizer2Impl *impl=Normalizer2Factory::getNFKC_CFImpl(status);
+                if(U_SUCCESS(status)) {
+                    impl->addPropertyStarts(&sa, status);
+                }
+                break;
+            }
+            case UPROPS_SRC_NFC_CANON_ITER: {
+                const Normalizer2Impl *impl=Normalizer2Factory::getNFCImpl(status);
+                if(U_SUCCESS(status)) {
+                    impl->addCanonIterPropertyStarts(&sa, status);
+                }
+                break;
+            }
 #endif
             case UPROPS_SRC_CASE:
-                ucase_addPropertyStarts(ucase_getSingleton(&status), &sa, &status);
+                ucase_addPropertyStarts(ucase_getSingleton(), &sa, &status);
                 break;
             case UPROPS_SRC_BIDI:
-                ubidi_addPropertyStarts(ubidi_getSingleton(&status), &sa, &status);
+                ubidi_addPropertyStarts(ubidi_getSingleton(), &sa, &status);
                 break;
             default:
                 status = U_INTERNAL_PROGRAM_ERROR;
@@ -198,6 +272,13 @@ static const UnicodeSet* getInclusions(int32_t src, UErrorCode &status) {
     return INCLUSIONS[src];
 }
 
+// Cache some sets for other services -------------------------------------- ***
+
+U_CFUNC UnicodeSet *
+uniset_getUnicode32Instance(UErrorCode &errorCode) {
+    return UnicodeSetSingleton(uni32Singleton, "[:age=3.2:]").getInstance(errorCode);
+}
+
 // helper functions for matching of pattern syntax pieces ------------------ ***
 // these functions are parallel to the PERL_OPEN etc. strings above
 
@@ -249,7 +330,8 @@ isPOSIXClose(const UnicodeString &pattern, int32_t pos) {
 UnicodeSet::UnicodeSet(const UnicodeString& pattern,
                        UErrorCode& status) :
     len(0), capacity(START_EXTRA), list(0), bmpSet(0), buffer(0),
-    bufferCapacity(0), patLen(0), pat(NULL), strings(NULL), stringSpan(NULL)
+    bufferCapacity(0), patLen(0), pat(NULL), strings(NULL), stringSpan(NULL),
+    fFlags(0)
 {   
     if(U_SUCCESS(status)){
         list = (UChar32*) uprv_malloc(sizeof(UChar32) * capacity);
@@ -277,7 +359,8 @@ UnicodeSet::UnicodeSet(const UnicodeString& pattern,
                        const SymbolTable* symbols,
                        UErrorCode& status) :
     len(0), capacity(START_EXTRA), list(0), bmpSet(0), buffer(0),
-    bufferCapacity(0), patLen(0), pat(NULL), strings(NULL), stringSpan(NULL)
+    bufferCapacity(0), patLen(0), pat(NULL), strings(NULL), stringSpan(NULL),
+    fFlags(0)
 {   
     if(U_SUCCESS(status)){
         list = (UChar32*) uprv_malloc(sizeof(UChar32) * capacity);
@@ -297,7 +380,8 @@ UnicodeSet::UnicodeSet(const UnicodeString& pattern, ParsePosition& pos,
                        const SymbolTable* symbols,
                        UErrorCode& status) :
     len(0), capacity(START_EXTRA), list(0), bmpSet(0), buffer(0),
-    bufferCapacity(0), patLen(0), pat(NULL), strings(NULL), stringSpan(NULL)
+    bufferCapacity(0), patLen(0), pat(NULL), strings(NULL), stringSpan(NULL),
+    fFlags(0)
 {
     if(U_SUCCESS(status)){
         list = (UChar32*) uprv_malloc(sizeof(UChar32) * capacity);
@@ -316,20 +400,6 @@ UnicodeSet::UnicodeSet(const UnicodeString& pattern, ParsePosition& pos,
 // Public API
 //----------------------------------------------------------------
 
-/**
- * Modifies this set to represent the set specified by the given
- * pattern, optionally ignoring white space.  See the class
- * description for the syntax of the pattern language.
- * @param pattern a string specifying what characters are in the set
- * @param ignoreSpaces if <code>true</code>, all spaces in the
- * pattern are ignored.  Spaces are those characters for which
- * <code>uprv_isRuleWhiteSpace()</code> is <code>true</code>.
- * Characters preceded by '\\' are escaped, losing any special
- * meaning they otherwise have.  Spaces may be included by
- * escaping them.
- * @exception <code>IllegalArgumentException</code> if the pattern
- * contains a syntax error.
- */
 UnicodeSet& UnicodeSet::applyPattern(const UnicodeString& pattern,
                                      UErrorCode& status) {
     return applyPattern(pattern, USET_IGNORE_SPACE, NULL, status);
@@ -533,13 +603,14 @@ void UnicodeSet::applyPattern(RuleCharacterIterator& chars,
             } else if (symbols != 0) {
                 const UnicodeFunctor *m = symbols->lookupMatcher(c);
                 if (m != 0) {
-                    if (m->getDynamicClassID() != UnicodeSet::getStaticClassID()) {
+                    const UnicodeSet *ms = dynamic_cast<const UnicodeSet *>(m);
+                    if (ms == NULL) {
                         ec = U_MALFORMED_SET;
                         return;
                     }
                     // casting away const, but `nested' won't be modified
                     // (important not to modify stored set)
-                    nested = (UnicodeSet*) m;
+                    nested = const_cast<UnicodeSet*>(ms);
                     setMode = 3;
                 }
             }
@@ -825,6 +896,10 @@ void UnicodeSet::applyPattern(RuleCharacterIterator& chars,
     } else {
         _generatePattern(rebuiltPat, FALSE);
     }
+    if (isBogus() && U_SUCCESS(ec)) {
+        // We likely ran out of memory. AHHH!
+        ec = U_MEMORY_ALLOCATION_ERROR;
+    }
 }
 
 //----------------------------------------------------------------
@@ -841,9 +916,10 @@ static UBool generalCategoryMaskFilter(UChar32 ch, void* context) {
 }
 
 static UBool versionFilter(UChar32 ch, void* context) {
-    UVersionInfo v, none = { 0, 0, 0, 0};
-    UVersionInfo* version = (UVersionInfo*)context;
+    static const UVersionInfo none = { 0, 0, 0, 0 };
+    UVersionInfo v;
     u_charAge(ch, v);
+    UVersionInfo* version = (UVersionInfo*)context;
     return uprv_memcmp(&v, &none, sizeof(v)) > 0 && uprv_memcmp(&v, version, sizeof(v)) <= 0;
 }
 
@@ -857,6 +933,9 @@ static UBool intPropertyFilter(UChar32 ch, void* context) {
     return u_getIntPropertyValue((UChar32) ch, c->prop) == c->value;
 }
 
+static UBool scriptExtensionsFilter(UChar32 ch, void* context) {
+    return uscript_hasScript(ch, *(UScriptCode*)context);
+}
 
 /**
  * Generic filter-based scanning code for UCD property UnicodeSets.
@@ -865,20 +944,17 @@ void UnicodeSet::applyFilter(UnicodeSet::Filter filter,
                              void* context,
                              int32_t src,
                              UErrorCode &status) {
-    // Walk through all Unicode characters, noting the start
+    if (U_FAILURE(status)) return;
+
+    // Logically, walk through all Unicode characters, noting the start
     // and end of each range for which filter.contain(c) is
     // true.  Add each range to a set.
     //
-    // To improve performance, use the INCLUSIONS set, which
+    // To improve performance, use an inclusions set which
     // encodes information about character ranges that are known
-    // to have identical properties. INCLUSIONS contains
-    // only the first characters of such ranges.
-    //
-    // TODO Where possible, instead of scanning over code points,
-    // use internal property data to initialize UnicodeSets for
-    // those properties.  Scanning code points is slow.
-    if (U_FAILURE(status)) return;
-
+    // to have identical properties.
+    // getInclusions(src) contains exactly the first characters of
+    // same-value ranges for the given properties "source".
     const UnicodeSet* inclusions = getInclusions(src, status);
     if (U_FAILURE(status)) {
         return;
@@ -887,7 +963,7 @@ void UnicodeSet::applyFilter(UnicodeSet::Filter filter,
     clear();
 
     UChar32 startHasProperty = -1;
-    int limitRange = inclusions->getRangeCount();
+    int32_t limitRange = inclusions->getRangeCount();
 
     for (int j=0; j<limitRange; ++j) {
         // get current range
@@ -911,6 +987,10 @@ void UnicodeSet::applyFilter(UnicodeSet::Filter filter,
     if (startHasProperty >= 0) {
         add((UChar32)startHasProperty, (UChar32)0x10FFFF);
     }
+    if (isBogus() && U_SUCCESS(status)) {
+        // We likely ran out of memory. AHHH!
+        status = U_MEMORY_ALLOCATION_ERROR;
+    }
 }
 
 static UBool mungeCharName(char* dst, const char* src, int32_t dstCapacity) {
@@ -942,6 +1022,9 @@ UnicodeSet::applyIntPropertyValue(UProperty prop, int32_t value, UErrorCode& ec)
 
     if (prop == UCHAR_GENERAL_CATEGORY_MASK) {
         applyFilter(generalCategoryMaskFilter, &value, UPROPS_SRC_CHAR, ec);
+    } else if (prop == UCHAR_SCRIPT_EXTENSIONS) {
+        UScriptCode script = (UScriptCode)value;
+        applyFilter(scriptExtensionsFilter, &script, UPROPS_SRC_PROPSVEC, ec);
     } else {
         IntPropertyContext c = {prop, value};
         applyFilter(intPropertyFilter, &c, uprops_getSource(prop), ec);
@@ -966,15 +1049,17 @@ UnicodeSet::applyPropertyAlias(const UnicodeString& prop,
     ) {
         FAIL(ec);
     }
-    CharString pname(prop);
-    CharString vname(value);
+    CharString pname, vname;
+    pname.appendInvariantChars(prop, ec);
+    vname.appendInvariantChars(value, ec);
+    if (U_FAILURE(ec)) return *this;
 
     UProperty p;
     int32_t v;
     UBool mustNotBeEmpty = FALSE, invert = FALSE;
 
     if (value.length() > 0) {
-        p = u_getPropertyEnum(pname);
+        p = u_getPropertyEnum(pname.data());
         if (p == UCHAR_INVALID_CODE) FAIL(ec);
 
         // Treat gc as gcm
@@ -985,14 +1070,14 @@ UnicodeSet::applyPropertyAlias(const UnicodeString& prop,
         if ((p >= UCHAR_BINARY_START && p < UCHAR_BINARY_LIMIT) ||
             (p >= UCHAR_INT_START && p < UCHAR_INT_LIMIT) ||
             (p >= UCHAR_MASK_START && p < UCHAR_MASK_LIMIT)) {
-            v = u_getPropertyValueEnum(p, vname);
+            v = u_getPropertyValueEnum(p, vname.data());
             if (v == UCHAR_INVALID_CODE) {
                 // Handle numeric CCC
                 if (p == UCHAR_CANONICAL_COMBINING_CLASS ||
                     p == UCHAR_TRAIL_CANONICAL_COMBINING_CLASS ||
                     p == UCHAR_LEAD_CANONICAL_COMBINING_CLASS) {
                     char* end;
-                    double value = uprv_strtod(vname, &end);
+                    double value = uprv_strtod(vname.data(), &end);
                     v = (int32_t) value;
                     if (v != value || v < 0 || *end != 0) {
                         // non-integral or negative value, or trailing junk
@@ -1013,7 +1098,7 @@ UnicodeSet::applyPropertyAlias(const UnicodeString& prop,
             case UCHAR_NUMERIC_VALUE:
                 {
                     char* end;
-                    double value = uprv_strtod(vname, &end);
+                    double value = uprv_strtod(vname.data(), &end);
                     if (*end != 0) {
                         FAIL(ec);
                     }
@@ -1027,7 +1112,7 @@ UnicodeSet::applyPropertyAlias(const UnicodeString& prop,
                     // Must munge name, since u_charFromName() does not do
                     // 'loose' matching.
                     char buf[128]; // it suffices that this be > uprv_getMaxCharNameLength
-                    if (!mungeCharName(buf, vname, sizeof(buf))) FAIL(ec);
+                    if (!mungeCharName(buf, vname.data(), sizeof(buf))) FAIL(ec);
                     UCharNameChoice choice = (p == UCHAR_NAME) ?
                         U_EXTENDED_CHAR_NAME : U_UNICODE_10_CHAR_NAME;
                     UChar32 ch = u_charFromName(choice, buf, &ec);
@@ -1045,13 +1130,20 @@ UnicodeSet::applyPropertyAlias(const UnicodeString& prop,
                     // Must munge name, since u_versionFromString() does not do
                     // 'loose' matching.
                     char buf[128];
-                    if (!mungeCharName(buf, vname, sizeof(buf))) FAIL(ec);
+                    if (!mungeCharName(buf, vname.data(), sizeof(buf))) FAIL(ec);
                     UVersionInfo version;
                     u_versionFromString(version, buf);
                     applyFilter(versionFilter, &version, UPROPS_SRC_PROPSVEC, ec);
                     return *this;
                 }
                 break;
+            case UCHAR_SCRIPT_EXTENSIONS:
+                v = u_getPropertyValueEnum(UCHAR_SCRIPT, vname.data());
+                if (v == UCHAR_INVALID_CODE) {
+                    FAIL(ec);
+                }
+                // fall through to calling applyIntPropertyValue()
+                break;
             default:
                 // p is a non-binary, non-enumerated property that we
                 // don't support (yet).
@@ -1064,21 +1156,21 @@ UnicodeSet::applyPropertyAlias(const UnicodeString& prop,
         // value is empty.  Interpret as General Category, Script, or
         // Binary property.
         p = UCHAR_GENERAL_CATEGORY_MASK;
-        v = u_getPropertyValueEnum(p, pname);
+        v = u_getPropertyValueEnum(p, pname.data());
         if (v == UCHAR_INVALID_CODE) {
             p = UCHAR_SCRIPT;
-            v = u_getPropertyValueEnum(p, pname);
+            v = u_getPropertyValueEnum(p, pname.data());
             if (v == UCHAR_INVALID_CODE) {
-                p = u_getPropertyEnum(pname);
+                p = u_getPropertyEnum(pname.data());
                 if (p >= UCHAR_BINARY_START && p < UCHAR_BINARY_LIMIT) {
                     v = 1;
-                } else if (0 == uprv_comparePropertyNames(ANY, pname)) {
+                } else if (0 == uprv_comparePropertyNames(ANY, pname.data())) {
                     set(MIN_VALUE, MAX_VALUE);
                     return *this;
-                } else if (0 == uprv_comparePropertyNames(ASCII, pname)) {
+                } else if (0 == uprv_comparePropertyNames(ASCII, pname.data())) {
                     set(0, 0x7F);
                     return *this;
-                } else if (0 == uprv_comparePropertyNames(ASSIGNED, pname)) {
+                } else if (0 == uprv_comparePropertyNames(ASSIGNED, pname.data())) {
                     // [:Assigned:]=[:^Cn:]
                     p = UCHAR_GENERAL_CATEGORY_MASK;
                     v = U_GC_CN_MASK;
@@ -1089,7 +1181,7 @@ UnicodeSet::applyPropertyAlias(const UnicodeString& prop,
             }
         }
     }
-    
+
     applyIntPropertyValue(p, v, ec);
     if(invert) {
         complement();
@@ -1101,6 +1193,10 @@ UnicodeSet::applyPropertyAlias(const UnicodeString& prop,
         ec = U_ILLEGAL_ARGUMENT_ERROR;
     }
 
+    if (isBogus() && U_SUCCESS(ec)) {
+        // We likely ran out of memory. AHHH!
+        ec = U_MEMORY_ALLOCATION_ERROR;
+    }
     return *this;
 }
 
@@ -1293,21 +1389,21 @@ addCaseMapping(UnicodeSet &set, int32_t result, const UChar *full, UnicodeString
 }
 
 UnicodeSet& UnicodeSet::closeOver(int32_t attribute) {
-    if (isFrozen()) {
+    if (isFrozen() || isBogus()) {
         return *this;
     }
     if (attribute & (USET_CASE_INSENSITIVE | USET_ADD_CASE_MAPPINGS)) {
-        UErrorCode status = U_ZERO_ERROR;
-        const UCaseProps *csp = ucase_getSingleton(&status);
-        if (U_SUCCESS(status)) {
+        const UCaseProps *csp = ucase_getSingleton();
+        {
             UnicodeSet foldSet(*this);
             UnicodeString str;
             USetAdder sa = {
-                (USet *)&foldSet,
+                foldSet.toUSet(),
                 _set_add,
                 _set_addRange,
                 _set_addString,
-                NULL // don't need remove()
+                NULL, // don't need remove()
+                NULL // don't need removeRange()
             };
 
             // start with input set to guarantee inclusion
@@ -1361,9 +1457,10 @@ UnicodeSet& UnicodeSet::closeOver(int32_t attribute) {
                 } else {
                     Locale root("");
 #if !UCONFIG_NO_BREAK_ITERATION
+                    UErrorCode status = U_ZERO_ERROR;
                     BreakIterator *bi = BreakIterator::createWordInstance(root, status);
-#endif
                     if (U_SUCCESS(status)) {
+#endif
                         const UnicodeString *pStr;
 
                         for (int32_t j=0; j<strings->size(); ++j) {
@@ -1379,8 +1476,8 @@ UnicodeSet& UnicodeSet::closeOver(int32_t attribute) {
                             (str = *pStr).foldCase();
                             foldSet.add(str);
                         }
-                    }
 #if !UCONFIG_NO_BREAK_ITERATION
+                    }
                     delete bi;
 #endif
                 }
