@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2001-2010, International Business Machines
+*   Copyright (C) 2001-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -184,7 +184,7 @@ unorm_cmpEquivFold(const UChar *s1, int32_t length1,
         nfcImpl=NULL;
     }
     if((options&U_COMPARE_IGNORE_CASE)!=0) {
-        csp=ucase_getSingleton(pErrorCode);
+        csp=ucase_getSingleton();
     } else {
         csp=NULL;
     }
@@ -591,6 +591,9 @@ unorm_compare(const UChar *s1, int32_t length1,
         } else {
             uni32=NULL;  // unused
         }
+        if(U_FAILURE(*pErrorCode)) {
+            return 0;
+        }
         FilteredNormalizer2 fn2(*n2, *uni32);
         if(normOptions&UNORM_UNICODE_3_2) {
             n2=&fn2;
