@@ -1,6 +1,6 @@
 /*  
 **********************************************************************
-*   Copyright (C) 1999-2007, International Business Machines
+*   Copyright (C) 1999-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   file name:  ustr_imp.h
@@ -22,17 +22,31 @@
 /** Simple declaration for u_strToTitle() to avoid including unicode/ubrk.h. */
 #ifndef UBRK_TYPEDEF_UBREAK_ITERATOR
 #   define UBRK_TYPEDEF_UBREAK_ITERATOR
-    typedef void UBreakIterator;
+    typedef struct UBreakIterator UBreakIterator;
 #endif
 
+#ifndef U_COMPARE_IGNORE_CASE
+/* see also unorm.h */
+/**
+ * Option bit for unorm_compare:
+ * Perform case-insensitive comparison.
+ */
+#define U_COMPARE_IGNORE_CASE       0x10000
+#endif
+
+/**
+ * Internal option for unorm_cmpEquivFold() for strncmp style.
+ * If set, checks for both string length and terminating NUL.
+ */
+#define _STRNCMP_STYLE 0x1000
+
 /**
  * Compare two strings in code point order or code unit order.
  * Works in strcmp style (both lengths -1),
  * strncmp style (lengths equal and >=0, flag TRUE),
  * and memcmp/UnicodeString style (at least one length >=0).
- * @internal
  */
-U_CAPI int32_t U_EXPORT2
+U_CFUNC int32_t U_EXPORT2
 uprv_strCompare(const UChar *s1, int32_t length1,
                 const UChar *s2, int32_t length2,
                 UBool strncmpStyle, UBool codePointOrder);
@@ -41,7 +55,6 @@ uprv_strCompare(const UChar *s1, int32_t length1,
  * Internal API, used by u_strcasecmp() etc.
  * Compare strings case-insensitively,
  * in code point order or code unit order.
- * @internal
  */
 U_CFUNC int32_t
 u_strcmpFold(const UChar *s1, int32_t length1,
@@ -53,9 +66,7 @@ u_strcmpFold(const UChar *s1, int32_t length1,
  * Are the Unicode properties loaded?
  * This must be used before internal functions are called that do
  * not perform this check.
- * Generate a debug assertion failure if data is not loaded, to flag the fact
- *   that u_init() wasn't called first, before trying to access character properties.
- * @internal
+ * Generate a debug assertion failure if data is not loaded.
  */
 U_CFUNC UBool
 uprv_haveProperties(UErrorCode *pErrorCode);
@@ -65,42 +76,15 @@ uprv_haveProperties(UErrorCode *pErrorCode);
   * Intended primarily for use from u_init().
   * Has no effect if property data is already loaded.
   * NOT thread safe.
-  * @internal
   */
 /*U_CFUNC int8_t
 uprv_loadPropsData(UErrorCode *errorCode);*/
 
-/**
- * Type of a function that may be passed to the internal case mapping functions
- * or similar for growing the destination buffer.
- * @internal
- */
-typedef UBool U_CALLCONV
-UGrowBuffer(void *context,      /* opaque pointer for this function */
-            UChar **pBuffer,    /* in/out destination buffer pointer */
-            int32_t *pCapacity, /* in/out buffer capacity in numbers of UChars */
-            int32_t reqCapacity,/* requested capacity */
-            int32_t length);    /* number of UChars to be copied to new buffer */
-
-/**
- * Default implementation of UGrowBuffer.
- * Takes a static buffer as context, allocates a new buffer,
- * and releases the old one if it is not the same as the one passed as context.
- * @internal
- */
-U_CAPI UBool /* U_CALLCONV U_EXPORT2 */
-u_growBufferFromStatic(void *context,
-                       UChar **pBuffer, int32_t *pCapacity, int32_t reqCapacity,
-                       int32_t length);
-
 /*
  * Internal string casing functions implementing
  * ustring.h/ustrcase.c and UnicodeString case mapping functions.
  */
 
-/**
- * @internal
- */
 struct UCaseMap {
     const UCaseProps *csp;
 #if !UCONFIG_NO_BREAK_ITERATION
@@ -115,9 +99,6 @@ struct UCaseMap {
 typedef struct UCaseMap UCaseMap;
 #endif
 
-/**
- * @internal
- */
 enum {
     TO_LOWER,
     TO_UPPER,
@@ -125,9 +106,6 @@ enum {
     FOLD_CASE
 };
 
-/**
- * @internal
- */
 U_CFUNC int32_t
 ustr_toLower(const UCaseProps *csp,
              UChar *dest, int32_t destCapacity,
@@ -135,9 +113,6 @@ ustr_toLower(const UCaseProps *csp,
              const char *locale,
              UErrorCode *pErrorCode);
 
-/**
- * @internal
- */
 U_CFUNC int32_t
 ustr_toUpper(const UCaseProps *csp,
              UChar *dest, int32_t destCapacity,
@@ -147,9 +122,6 @@ ustr_toUpper(const UCaseProps *csp,
 
 #if !UCONFIG_NO_BREAK_ITERATION
 
-/**
- * @internal
- */
 U_CFUNC int32_t
 ustr_toTitle(const UCaseProps *csp,
              UChar *dest, int32_t destCapacity,
@@ -162,7 +134,6 @@ ustr_toTitle(const UCaseProps *csp,
 
 /**
  * Internal case folding function.
- * @internal
  */
 U_CFUNC int32_t
 ustr_foldCase(const UCaseProps *csp,
@@ -182,7 +153,6 @@ ustr_foldCase(const UCaseProps *csp,
  * @param length Number of UChars that were (to be) written to dest.
  * @param pErrorCode ICU error code.
  * @return length
- * @internal
  */
 U_CAPI int32_t U_EXPORT2
 u_terminateUChars(UChar *dest, int32_t destCapacity, int32_t length, UErrorCode *pErrorCode);
