@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 1999-2001, International Business Machines
+*   Copyright (C) 1999-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -13,10 +13,31 @@
 #define UVECTOR_H
 
 #include "unicode/utypes.h"
+#include "unicode/uobject.h"
+#include "uarrsort.h"
 #include "uhash.h"
 
 U_NAMESPACE_BEGIN
 
+/**
+ * A token comparison function.
+ * @param tok1 A token (object or integer)
+ * @param tok2 A token (object or integer)
+ * @return 0 if the two tokens are equal, -1 if tok1 is < tok2, or
+ * +1 if tok1 is > tok2.
+ */
+typedef int8_t U_CALLCONV USortComparator(UHashTok tok1,
+                                          UHashTok tok2);
+
+/**
+ * A token assignment function.  It may copy an integer, copy
+ * a pointer, or clone a pointer, as appropriate.
+ * @param dst The token to be assigned to
+ * @param src The token to assign from
+ */
+typedef void U_CALLCONV UTokenAssigner(UHashTok *dst,
+                                       UHashTok *src);
+
 /**
  * <p>Ultralightweight C++ implementation of a <tt>void*</tt> vector
  * that is (mostly) compatible with java.util.Vector.
@@ -67,7 +88,7 @@ U_NAMESPACE_BEGIN
  *
  * @author Alan Liu
  */
-class U_COMMON_API UVector {
+class U_COMMON_API UVector : public UObject {
     // NOTE: UVector uses the UHashKey (union of void* and int32_t) as
     // its basic storage type.  It uses UKeyComparator as its
     // comparison function.  It uses UObjectDeleter as its deleter
@@ -82,19 +103,38 @@ private:
 
     UHashTok* elements;
 
-    UObjectDeleter deleter;
+    UObjectDeleter *deleter;
 
-    UKeyComparator comparer;
+    UKeyComparator *comparer;
 
 public:
     UVector(UErrorCode &status);
 
     UVector(int32_t initialCapacity, UErrorCode &status);
 
-    UVector(UObjectDeleter d, UKeyComparator c, UErrorCode &status);
+    UVector(UObjectDeleter *d, UKeyComparator *c, UErrorCode &status);
 
-    UVector(UObjectDeleter d, UKeyComparator c, int32_t initialCapacity, UErrorCode &status);
-    ~UVector();
+    UVector(UObjectDeleter *d, UKeyComparator *c, int32_t initialCapacity, UErrorCode &status);
+
+    virtual ~UVector();
+
+    /**
+     * Assign this object to another (make this a copy of 'other').
+     * Use the 'assign' function to assign each element.
+     */
+    void assign(const UVector& other, UTokenAssigner *assign, UErrorCode &ec);
+
+    /**
+     * Compare this vector with another.  They will be considered
+     * equal if they are of the same size and all elements are equal,
+     * as compared using this object's comparer.
+     */
+    UBool operator==(const UVector& other);
+
+    /**
+     * Equivalent to !operator==()
+     */
+    inline UBool operator!=(const UVector& other);
 
     //------------------------------------------------------------
     // java.util.Vector API
@@ -110,10 +150,14 @@ public:
 
     void insertElementAt(void* obj, int32_t index, UErrorCode &status);
 
+    void insertElementAt(int32_t elem, int32_t index, UErrorCode &status);
+    
     void* elementAt(int32_t index) const;
 
     int32_t elementAti(int32_t index) const;
 
+    UBool equals(const UVector &other) const;
+
     void* firstElement(void) const;
 
     void* lastElement(void) const;
@@ -128,6 +172,12 @@ public:
 
     UBool contains(int32_t obj) const;
 
+    UBool containsAll(const UVector& other) const;
+
+    UBool removeAll(const UVector& other);
+
+    UBool retainAll(const UVector& other);
+
     void removeElementAt(int32_t index);
 
     UBool removeElement(void* obj);
@@ -144,9 +194,9 @@ public:
      * Change the size of this vector as follows: If newSize is
      * smaller, then truncate the array, possibly deleting held
      * elements for i >= newSize.  If newSize is larger, grow the
-     * array, filling in new slows with NULL.
+     * array, filling in new slots with NULL.
      */
-    void setSize(int32_t newSize);
+    void setSize(int32_t newSize, UErrorCode &status);
 
     /**
      * Fill in the given array with all elements of this vector.
@@ -157,9 +207,9 @@ public:
     // New API
     //------------------------------------------------------------
 
-    UObjectDeleter setDeleter(UObjectDeleter d);
+    UObjectDeleter *setDeleter(UObjectDeleter *d);
 
-    UKeyComparator setComparer(UKeyComparator c);
+    UKeyComparator *setComparer(UKeyComparator *c);
 
     void* operator[](int32_t index) const;
 
@@ -174,14 +224,73 @@ public:
      */
     void* orphanElementAt(int32_t index);
 
+    /**
+     * Returns true if this vector contains none of the elements
+     * of the given vector.
+     * @param other vector to be checked for containment
+     * @return true if the test condition is met
+     */
+    UBool containsNone(const UVector& other) const;
+
+    /**
+     * Insert the given object into this vector at its sorted position
+     * as defined by 'compare'.  The current elements are assumed to
+     * be sorted already.
+     */
+    void sortedInsert(void* obj, USortComparator *compare, UErrorCode& ec);
+
+    /**
+     * Insert the given integer into this vector at its sorted position
+     * as defined by 'compare'.  The current elements are assumed to
+     * be sorted already.
+     */
+    void sortedInsert(int32_t obj, USortComparator *compare, UErrorCode& ec);
+
+    /**
+     * Sort the contents of the vector, assuming that the contents of the
+     * vector are of type int32_t.
+     */
+    void sorti(UErrorCode &ec);
+
+    /**
+      * Sort the contents of this vector, using a caller-supplied function
+      * to do the comparisons.  (It's confusing that
+      *  UVector's USortComparator function is different from the
+      *  UComparator function type defined in uarrsort.h)
+      */
+    void sort(USortComparator *compare, UErrorCode &ec);
+
+    /**
+     * Sort the contents of this vector using a caller-supplied function
+     * of type UComparator to do the comparison.  Provides more flexibility
+     * than uvector::sort() because an additional user-parameter can be passed to
+     * the comparison function.
+     */
+    void sortWithUComparator(UComparator *compare, const void *context, UErrorCode &ec);
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     */
+    static UClassID U_EXPORT2 getStaticClassID();
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     */
+    virtual UClassID getDynamicClassID() const;
+
 private:
     void _init(int32_t initialCapacity, UErrorCode &status);
 
+    int32_t indexOf(UHashTok key, int32_t startIndex = 0, int8_t hint = 0) const;
+
+    void sortedInsert(UHashTok tok, USortComparator *compare, UErrorCode& ec);
+
     // Disallow
     UVector(const UVector&);
 
     // Disallow
     UVector& operator=(const UVector&);
+
 };
 
 
@@ -207,9 +316,11 @@ public:
 
     UStack(int32_t initialCapacity, UErrorCode &status);
 
-    UStack(UObjectDeleter d, UKeyComparator c, UErrorCode &status);
+    UStack(UObjectDeleter *d, UKeyComparator *c, UErrorCode &status);
 
-    UStack(UObjectDeleter d, UKeyComparator c, int32_t initialCapacity, UErrorCode &status);
+    UStack(UObjectDeleter *d, UKeyComparator *c, int32_t initialCapacity, UErrorCode &status);
+
+    virtual ~UStack();
 
     // It's okay not to have a virtual destructor (in UVector)
     // because UStack has no special cleanup to do.
@@ -228,8 +339,22 @@ public:
 
     int32_t push(int32_t i, UErrorCode &status);
 
+    /*
+    If the object o occurs as an item in this stack,
+    this method returns the 1-based distance from the top of the stack.
+    */
     int32_t search(void* obj) const;
 
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     */
+    static UClassID U_EXPORT2 getStaticClassID();
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     */
+    virtual UClassID getDynamicClassID() const;
+
 private:
     // Disallow
     UStack(const UStack&);
@@ -273,6 +398,10 @@ inline void* UVector::operator[](int32_t index) const {
     return elementAt(index);
 }
 
+inline UBool UVector::operator!=(const UVector& other) {
+    return !operator==(other);
+}
+
 // UStack inlines
 
 inline UBool UStack::empty(void) const {
