@@ -1,6 +1,6 @@
 /*****************************************************************************
 *
-*   Copyright (C) 1999-2006, International Business Machines
+*   Copyright (C) 1999-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************/
@@ -40,7 +40,7 @@
 
 U_NAMESPACE_USE
 
-#if (defined(U_WINDOWS) || defined(U_CYGWIN)) && !defined(__STRICT_ANSI__)
+#if (defined(U_WINDOWS) || defined(U_CYGWIN) || defined(U_MINGW)) && !defined(__STRICT_ANSI__)
 #include <io.h>
 #include <fcntl.h>
 #if defined(U_WINDOWS)
@@ -259,7 +259,7 @@ static int printConverters(const char *pname, const char *lookfor,
             }
             if (U_FAILURE(err)) {
                 u_wmsg(stderr, "cantGetTag", u_wmsg_errorName(err));
-                return -1;
+                goto error_cleanup;
             }
         }
         if (canon) {
@@ -291,7 +291,7 @@ static int printConverters(const char *pname, const char *lookfor,
             putchar('\t');
             u_wmsg(stderr, "cantGetAliases", str.getTerminatedBuffer(),
                 u_wmsg_errorName(err));
-            return -1;
+            goto error_cleanup;
         } else {
             uint16_t a, s, t;
 
@@ -305,7 +305,7 @@ static int printConverters(const char *pname, const char *lookfor,
                     putchar('\t');
                     u_wmsg(stderr, "cantGetAliases", str.getTerminatedBuffer(),
                         u_wmsg_errorName(err));
-                    return -1;
+                    goto error_cleanup;
                 }
 
                 /* Print the current alias so that it looks right. */
@@ -363,6 +363,9 @@ static int printConverters(const char *pname, const char *lookfor,
     /* Success. */
 
     return 0;
+error_cleanup:
+    uprv_free(stds);
+    return -1;
 }
 
 /* Print all available transliterators. If canon is non zero, print
@@ -374,56 +377,30 @@ static int printTransliterators(UBool canon)
     printf("no transliterators available because of UCONFIG_NO_TRANSLITERATION, see uconfig.h\n");
     return 1;
 #else
-    int32_t numtrans = utrans_countAvailableIDs(), i;
-    int buflen = 512;
-    char *buf = (char *) uprv_malloc(buflen);
-    char staticbuf[512];
+    UErrorCode status = U_ZERO_ERROR;
+    UEnumeration *ids = utrans_openIDs(&status);
+    int32_t i, numtrans = uenum_count(ids, &status);
 
     char sepchar = canon ? '\n' : ' ';
 
-    if (!buf) {
-        buf = staticbuf;
-        buflen = sizeof(staticbuf);
-    }
+    for (i = 0; U_SUCCESS(status)&& (i < numtrans); ++i) {
+    	int32_t len;
+    	const char *nextTrans = uenum_next(ids, &len, &status);
 
-    for (i = 0; i < numtrans; ++i) {
-        int32_t len = utrans_getAvailableID(i, buf, buflen);
-        if (len >= buflen - 1) {
-            if (buf != staticbuf) {
-                buflen <<= 1;
-                if (buflen < len) {
-                    buflen = len + 64;
-                }
-                buf = (char *) uprv_realloc(buf, buflen);
-                if (!buf) {
-                    buf = staticbuf;
-                    buflen = sizeof(staticbuf);
-                }
-            }
-            utrans_getAvailableID(i, buf, buflen);
-            if (len >= buflen) {
-                uprv_strcpy(buf + buflen - 4, "..."); /* Truncate the name. */
-            }
-        }
-
-        printf("%s", buf);
+        printf("%s", nextTrans);
         if (i < numtrans - 1) {
             putchar(sepchar);
         }
     }
 
+    uenum_close(ids);
+
     /* Add a terminating newline if needed. */
 
     if (sepchar != '\n') {
         putchar('\n');
     }
 
-    /* Free temporary data. */
-
-    if (buf != staticbuf) {
-        uprv_free(buf);
-    }
-
     /* Success. */
 
     return 0;
@@ -1390,7 +1367,11 @@ main(int argc, char **argv)
 
     goto normal_exit;
 error_exit:
+#if !UCONFIG_NO_LEGACY_CONVERSION
     ret = 1;
+#else 
+    fprintf(stderr, "uconv error: UCONFIG_NO_LEGACY_CONVERSION is on. See uconfig.h\n");
+#endif
 normal_exit:
 
     if (outfile != stdout) {
