@@ -1,6 +1,6 @@
 /*
  ******************************************************************************
- *   Copyright (C) 1996-2009, International Business Machines                 *
+ *   Copyright (C) 1996-2011, International Business Machines                 *
  *   Corporation and others.  All Rights Reserved.                            *
  ******************************************************************************
  */
@@ -234,10 +234,14 @@ void StringList::add(const UnicodeString *string, UErrorCode &status)
 
     if (listSize >= listMax) {
         int32_t newMax = listMax + STRING_LIST_BUFFER_SIZE;
-
         UnicodeString *newStrings = new UnicodeString[newMax];
-
-        uprv_memcpy(newStrings, strings, listSize * sizeof(UnicodeString));
+        if (newStrings == NULL) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return;
+        }
+        for (int32_t i=0; i<listSize; ++i) {
+            newStrings[i] = strings[i];
+        }
 
 #ifdef INSTRUMENT_STRING_LIST
         int32_t _h = listSize / STRING_LIST_BUFFER_SIZE;
@@ -463,9 +467,9 @@ private:
     static char *getKey(UCollator *collator, char *keyBuffer, int32_t *charBufferLength);
     static void deleteKey(char *key);
 
-    UMTX lock;
     UHashtable *cache;
 };
+static UMTX lock;
 
 U_CFUNC void deleteChars(void * /*obj*/)
 {
@@ -484,7 +488,7 @@ U_CFUNC void deleteCollDataCacheEntry(void *obj)
 }
 
 CollDataCache::CollDataCache(UErrorCode &status)
-    : lock(0), cache(NULL)
+    : cache(NULL)
 {
     if (U_FAILURE(status)) {
         return;
@@ -506,8 +510,6 @@ CollDataCache::~CollDataCache()
     uhash_close(cache);
     cache = NULL;
     umtx_unlock(&lock);
-
-    umtx_destroy(&lock);
 }
 
 CollData *CollDataCache::get(UCollator *collator, UErrorCode &status)
