@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (C) 1997-2006, International Business Machines Corporation and    *
+* Copyright (C) 1997-2011, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 *
@@ -33,6 +33,9 @@
 #include "unicode/ustring.h"
 #include "unicode/ucurr.h"
 #include "unicode/curramt.h"
+#include "unicode/numsys.h"
+#include "unicode/rbnf.h"
+#include "charstr.h"
 #include "winnmfmt.h"
 #include "uresimp.h"
 #include "uhash.h"
@@ -41,6 +44,9 @@
 #include "ucln_in.h"
 #include "cstring.h"
 #include "putilimp.h"
+#include "umutex.h"
+#include "mutex.h"
+#include "digitlst.h"
 #include <float.h>
 
 //#define FMT_DEBUG
@@ -72,20 +78,98 @@ static const UChar gLastResortPercentPat[] = {
 static const UChar gLastResortScientificPat[] = {
     0x23, 0x45, 0x30, 0 /* "#E0" */
 };
+static const UChar gLastResortIsoCurrencyPat[] = {
+    0xA4, 0xA4, 0x23, 0x30, 0x2E, 0x30, 0x30, 0x3B, 0x28, 0xA4, 0xA4, 0x23, 0x30, 0x2E, 0x30, 0x30, 0x29, 0 /* "\u00A4\u00A4#0.00;(\u00A4\u00A4#0.00)" */
+};
+static const UChar gLastResortPluralCurrencyPat[] = {
+    0x23, 0x30, 0x2E, 0x30, 0x30, 0xA0, 0xA4, 0xA4, 0xA4, 0 /* "#0.00\u00A0\u00A4\u00A4\u00A4*/
+};
+
+static const UChar gSingleCurrencySign[] = {0xA4, 0};
+static const UChar gDoubleCurrencySign[] = {0xA4, 0xA4, 0};
+
+static const UChar gSlash = 0x2f;
 
 // If the maximum base 10 exponent were 4, then the largest number would
 // be 99,999 which has 5 digits.
-static const int32_t gMaxIntegerDigits = DBL_MAX_10_EXP + 1; // Should be ~40 ? --srl
+// On IEEE754 systems gMaxIntegerDigits is 308 + possible denormalized 15 digits + rounding digit
+static const int32_t gMaxIntegerDigits = DBL_MAX_10_EXP + DBL_DIG + 1;
 static const int32_t gMinIntegerDigits = 127;
 
-static const UChar * const gLastResortNumberPatterns[] =
-{
-    gLastResortDecimalPat,
-    gLastResortCurrencyPat,
-    gLastResortPercentPat,
-    gLastResortScientificPat
+static const UChar * const gLastResortNumberPatterns[UNUM_FORMAT_STYLE_COUNT] = {
+    NULL,  // UNUM_PATTERN_DECIMAL
+    gLastResortDecimalPat,  // UNUM_DECIMAL
+    gLastResortCurrencyPat,  // UNUM_CURRENCY
+    gLastResortPercentPat,  // UNUM_PERCENT
+    gLastResortScientificPat,  // UNUM_SCIENTIFIC
+    NULL,  // UNUM_SPELLOUT
+    NULL,  // UNUM_ORDINAL
+    NULL,  // UNUM_DURATION
+    NULL,  // UNUM_NUMBERING_SYSTEM
+    NULL,  // UNUM_PATTERN_RULEBASED
+    gLastResortIsoCurrencyPat,  // UNUM_CURRENCY_ISO
+    gLastResortPluralCurrencyPat  // UNUM_CURRENCY_PLURAL
 };
 
+// Keys used for accessing resource bundles
+
+static const char *gNumberElements = "NumberElements";
+static const char *gLatn = "latn";
+static const char *gPatterns = "patterns";
+static const char *gFormatKeys[UNUM_FORMAT_STYLE_COUNT] = {
+    NULL,  // UNUM_PATTERN_DECIMAL
+    "decimalFormat",  // UNUM_DECIMAL
+    "currencyFormat",  // UNUM_CURRENCY
+    "percentFormat",  // UNUM_PERCENT
+    "scientificFormat",  // UNUM_SCIENTIFIC
+    NULL,  // UNUM_SPELLOUT
+    NULL,  // UNUM_ORDINAL
+    NULL,  // UNUM_DURATION
+    NULL,  // UNUM_NUMBERING_SYSTEM
+    NULL,  // UNUM_PATTERN_RULEBASED
+    // For UNUM_CURRENCY_ISO and UNUM_CURRENCY_PLURAL,
+    // the pattern is the same as the pattern of UNUM_CURRENCY
+    // except for replacing the single currency sign with
+    // double currency sign or triple currency sign.
+    "currencyFormat",  // UNUM_CURRENCY_ISO
+    "currencyFormat"  // UNUM_CURRENCY_PLURAL
+};
+
+// Static hashtable cache of NumberingSystem objects used by NumberFormat
+static UHashtable * NumberingSystem_cache = NULL;
+
+static UMTX nscacheMutex = NULL;
+
+#if !UCONFIG_NO_SERVICE
+static U_NAMESPACE_QUALIFIER ICULocaleService* gService = NULL;
+#endif
+
+/**
+ * Release all static memory held by Number Format.
+ */
+U_CDECL_BEGIN
+static void U_CALLCONV
+deleteNumberingSystem(void *obj) {
+    delete (U_NAMESPACE_QUALIFIER NumberingSystem *)obj;
+}
+
+static UBool U_CALLCONV numfmt_cleanup(void) {
+#if !UCONFIG_NO_SERVICE
+    if (gService) {
+        delete gService;
+        gService = NULL;
+    }
+#endif
+    if (NumberingSystem_cache) {
+        // delete NumberingSystem_cache;
+        uhash_close(NumberingSystem_cache);
+        NumberingSystem_cache = NULL;
+    }
+
+    return TRUE;
+}
+U_CDECL_END
+
 // *****************************************************************************
 // class NumberFormat
 // *****************************************************************************
@@ -111,7 +195,7 @@ UBool SimpleNumberFormatFactory::visible(void) const {
 }
 
 const UnicodeString *
-SimpleNumberFormatFactory::getSupportedIDs(int32_t &count, UErrorCode& status) const 
+SimpleNumberFormatFactory::getSupportedIDs(int32_t &count, UErrorCode& status) const
 {
     if (U_SUCCESS(status)) {
         count = 1;
@@ -130,7 +214,8 @@ NumberFormat::NumberFormat()
     fMinIntegerDigits(1),
     fMaxFractionDigits(3), // invariant, >= minFractionDigits
     fMinFractionDigits(0),
-    fParseIntegerOnly(FALSE)
+    fParseIntegerOnly(FALSE),
+    fLenient(FALSE)
 {
     fCurrency[0] = 0;
 }
@@ -158,6 +243,7 @@ NumberFormat::operator=(const NumberFormat& rhs)
 {
     if (this != &rhs)
     {
+        Format::operator=(rhs);
         fGroupingUsed = rhs.fGroupingUsed;
         fMaxIntegerDigits = rhs.fMaxIntegerDigits;
         fMinIntegerDigits = rhs.fMinIntegerDigits;
@@ -165,6 +251,7 @@ NumberFormat::operator=(const NumberFormat& rhs)
         fMinFractionDigits = rhs.fMinFractionDigits;
         fParseIntegerOnly = rhs.fParseIntegerOnly;
         u_strncpy(fCurrency, rhs.fCurrency, 4);
+        fLenient = rhs.fLenient;
     }
     return *this;
 }
@@ -207,7 +294,11 @@ NumberFormat::operator==(const Format& that) const
         if (first) { printf("[ "); first = FALSE; } else { printf(", "); }
         debug("fCurrency !=");
     }
-    if (!first) { printf(" ]"); }    
+    if (!(fLenient == other->fLenient)) {
+        if (first) { printf("[ "); first = FALSE; } else { printf(", "); }
+        debug("fLenient != ");
+    }
+    if (!first) { printf(" ]"); }
 #endif
 
     return ((this == &that) ||
@@ -218,13 +309,168 @@ NumberFormat::operator==(const Format& that) const
               fMinFractionDigits == other->fMinFractionDigits &&
               fGroupingUsed == other->fGroupingUsed &&
               fParseIntegerOnly == other->fParseIntegerOnly &&
-              u_strcmp(fCurrency, other->fCurrency) == 0)));
+              u_strcmp(fCurrency, other->fCurrency) == 0 &&
+              fLenient == other->fLenient)));
 }
 
-// -------------------------------------x
+// -------------------------------------
+// Default implementation sets unsupported error; subclasses should
+// override.
+
+UnicodeString&
+NumberFormat::format(double /* unused number */,
+                     UnicodeString& toAppendTo,
+                     FieldPositionIterator* /* unused posIter */,
+                     UErrorCode& status) const
+{
+    if (!U_FAILURE(status)) {
+        status = U_UNSUPPORTED_ERROR;
+    }
+    return toAppendTo;
+}
+
+// -------------------------------------
+// Default implementation sets unsupported error; subclasses should
+// override.
+
+UnicodeString&
+NumberFormat::format(int32_t /* unused number */,
+                     UnicodeString& toAppendTo,
+                     FieldPositionIterator* /* unused posIter */,
+                     UErrorCode& status) const
+{
+    if (!U_FAILURE(status)) {
+        status = U_UNSUPPORTED_ERROR;
+    }
+    return toAppendTo;
+}
+
+// -------------------------------------
+// Default implementation sets unsupported error; subclasses should
+// override.
+
+UnicodeString&
+NumberFormat::format(int64_t /* unused number */,
+                     UnicodeString& toAppendTo,
+                     FieldPositionIterator* /* unused posIter */,
+                     UErrorCode& status) const
+{
+    if (!U_FAILURE(status)) {
+        status = U_UNSUPPORTED_ERROR;
+    }
+    return toAppendTo;
+}
+
+// -------------------------------------
+// Decimal Number format() default implementation 
+// Subclasses do not normally override this function, but rather the DigitList
+// formatting functions..
+//   The expected call chain from here is
+//      this function ->
+//      NumberFormat::format(Formattable  ->
+//      DecimalFormat::format(DigitList    
+//
+//   Or, for subclasses of Formattable that do not know about DigitList,
+//       this Function ->
+//       NumberFormat::format(Formattable  ->
+//       NumberFormat::format(DigitList  ->
+//       XXXFormat::format(double
+
+UnicodeString&
+NumberFormat::format(const StringPiece &decimalNum,
+                     UnicodeString& toAppendTo,
+                     FieldPositionIterator* fpi,
+                     UErrorCode& status) const
+{
+    Formattable f;
+    f.setDecimalNumber(decimalNum, status);
+    format(f, toAppendTo, fpi, status);
+    return toAppendTo;
+}
+
+// -------------------------------------
 // Formats the number object and save the format
 // result in the toAppendTo string buffer.
 
+// utility to save/restore state, used in two overloads
+// of format(const Formattable&...) below.
+
+class ArgExtractor {
+  NumberFormat *ncnf;
+  const Formattable* num;
+  UBool setCurr;
+  UChar save[4];
+
+ public:
+  ArgExtractor(const NumberFormat& nf, const Formattable& obj, UErrorCode& status);
+  ~ArgExtractor();
+
+  const Formattable* number(void) const;
+};
+
+inline const Formattable*
+ArgExtractor::number(void) const {
+  return num;
+}
+
+ArgExtractor::ArgExtractor(const NumberFormat& nf, const Formattable& obj, UErrorCode& status)
+    : ncnf((NumberFormat*) &nf), num(&obj), setCurr(FALSE) {
+
+    const UObject* o = obj.getObject(); // most commonly o==NULL
+    const CurrencyAmount* amt;
+    if (o != NULL && (amt = dynamic_cast<const CurrencyAmount*>(o)) != NULL) {
+        // getISOCurrency() returns a pointer to internal storage, so we
+        // copy it to retain it across the call to setCurrency().
+        const UChar* curr = amt->getISOCurrency();
+        u_strcpy(save, nf.getCurrency());
+        setCurr = (u_strcmp(curr, save) != 0);
+        if (setCurr) {
+            ncnf->setCurrency(curr, status);
+        }
+        num = &amt->getNumber();
+    }
+}
+
+ArgExtractor::~ArgExtractor() {
+    if (setCurr) {
+        UErrorCode ok = U_ZERO_ERROR;
+        ncnf->setCurrency(save, ok); // always restore currency
+    }
+}
+
+UnicodeString& NumberFormat::format(const DigitList &number,
+                      UnicodeString& appendTo,
+                      FieldPositionIterator* posIter,
+                      UErrorCode& status) const {
+    // DecimalFormat overrides this function, and handles DigitList based big decimals.
+    // Other subclasses (ChoiceFormat, RuleBasedNumberFormat) do not (yet) handle DigitLists,
+    // so this default implementation falls back to formatting decimal numbers as doubles.
+    if (U_FAILURE(status)) {
+        return appendTo;
+    }
+    double dnum = number.getDouble();
+    format(dnum, appendTo, posIter, status);
+    return appendTo;
+}
+
+
+
+UnicodeString&
+NumberFormat::format(const DigitList &number,
+                     UnicodeString& appendTo,
+                     FieldPosition& pos,
+                     UErrorCode &status) const { 
+    // DecimalFormat overrides this function, and handles DigitList based big decimals.
+    // Other subclasses (ChoiceFormat, RuleBasedNumberFormat) do not (yet) handle DigitLists,
+    // so this default implementation falls back to formatting decimal numbers as doubles.
+    if (U_FAILURE(status)) {
+        return appendTo;
+    }
+    double dnum = number.getDouble();
+    format(dnum, appendTo, pos, status);
+    return appendTo;
+}
+
 UnicodeString&
 NumberFormat::format(const Formattable& obj,
                         UnicodeString& appendTo,
@@ -233,51 +479,80 @@ NumberFormat::format(const Formattable& obj,
 {
     if (U_FAILURE(status)) return appendTo;
 
-    NumberFormat* nonconst = (NumberFormat*) this;
-    const Formattable* n = &obj;
+    ArgExtractor arg(*this, obj, status);
+    const Formattable *n = arg.number();
 
-    UChar save[4];
-    UBool setCurr = FALSE;
-    const UObject* o = obj.getObject(); // most commonly o==NULL
-    if (o != NULL &&
-        o->getDynamicClassID() == CurrencyAmount::getStaticClassID()) {
-        // getISOCurrency() returns a pointer to internal storage, so we
-        // copy it to retain it across the call to setCurrency().
-        const CurrencyAmount* amt = (const CurrencyAmount*) o;
-        const UChar* curr = amt->getISOCurrency();
-        u_strcpy(save, getCurrency());
-        setCurr = (u_strcmp(curr, save) != 0);
-        if (setCurr) {
-            nonconst->setCurrency(curr, status);
+    if (n->isNumeric() && n->getDigitList() != NULL) {
+        // Decimal Number.  We will have a DigitList available if the value was
+        //   set to a decimal number, or if the value originated with a parse.
+        //
+        // The default implementation for formatting a DigitList converts it
+        // to a double, and formats that, allowing formatting classes that don't
+        // know about DigitList to continue to operate as they had.
+        //
+        // DecimalFormat overrides the DigitList formatting functions.
+        format(*n->getDigitList(), appendTo, pos, status);
+    } else {
+        switch (n->getType()) {
+        case Formattable::kDouble:
+            format(n->getDouble(), appendTo, pos);
+            break;
+        case Formattable::kLong:
+            format(n->getLong(), appendTo, pos);
+            break;
+        case Formattable::kInt64:
+            format(n->getInt64(), appendTo, pos);
+            break;
+        default:
+            status = U_INVALID_FORMAT_ERROR;
+            break;
         }
-        n = &amt->getNumber();
     }
 
-    switch (n->getType()) {
-    case Formattable::kDouble:
-        format(n->getDouble(), appendTo, pos);
-        break;
-    case Formattable::kLong:
-        format(n->getLong(), appendTo, pos);
-        break;
-    case Formattable::kInt64:
-        format(n->getInt64(), appendTo, pos);
-        break;
-    default:
-        status = U_INVALID_FORMAT_ERROR;
-        break;
+    return appendTo;
+}
+
+// -------------------------------------x
+// Formats the number object and save the format
+// result in the toAppendTo string buffer.
+
+UnicodeString&
+NumberFormat::format(const Formattable& obj,
+                        UnicodeString& appendTo,
+                        FieldPositionIterator* posIter,
+                        UErrorCode& status) const
+{
+    if (U_FAILURE(status)) return appendTo;
+
+    ArgExtractor arg(*this, obj, status);
+    const Formattable *n = arg.number();
+
+    if (n->isNumeric() && n->getDigitList() != NULL) {
+        // Decimal Number
+        format(*n->getDigitList(), appendTo, posIter, status);
+    } else {
+        switch (n->getType()) {
+        case Formattable::kDouble:
+            format(n->getDouble(), appendTo, posIter, status);
+            break;
+        case Formattable::kLong:
+            format(n->getLong(), appendTo, posIter, status);
+            break;
+        case Formattable::kInt64:
+            format(n->getInt64(), appendTo, posIter, status);
+            break;
+        default:
+            status = U_INVALID_FORMAT_ERROR;
+            break;
+        }
     }
 
-    if (setCurr) {
-        UErrorCode ok = U_ZERO_ERROR;
-        nonconst->setCurrency(save, ok); // always restore currency
-    }
     return appendTo;
 }
 
 // -------------------------------------
 
-UnicodeString& 
+UnicodeString&
 NumberFormat::format(int64_t number,
                      UnicodeString& appendTo,
                      FieldPosition& pos) const
@@ -360,9 +635,11 @@ Formattable& NumberFormat::parseCurrency(const UnicodeString& text,
         getEffectiveCurrency(curr, ec);
         if (U_SUCCESS(ec)) {
             Formattable n(result);
-            result.adoptObject(new CurrencyAmount(n, curr, ec));
-            if (U_FAILURE(ec)) {
+            CurrencyAmount *tempCurAmnt = new CurrencyAmount(n, curr, ec);  // Use for null testing.
+            if (U_FAILURE(ec) || tempCurAmnt == NULL) {
                 pos.setIndex(start); // indicate failure
+            } else {
+            	result.adoptObject(tempCurAmnt);
             }
         }
     }
@@ -378,13 +655,22 @@ NumberFormat::setParseIntegerOnly(UBool value)
     fParseIntegerOnly = value;
 }
 
+// -------------------------------------
+// Sets whether lenient parse is enabled.
+
+void
+NumberFormat::setLenient(UBool enable)
+{
+    fLenient = enable;
+}
+
 // -------------------------------------
 // Create a number style NumberFormat instance with the default locale.
 
 NumberFormat* U_EXPORT2
 NumberFormat::createInstance(UErrorCode& status)
 {
-    return createInstance(Locale::getDefault(), kNumberStyle, status);
+    return createInstance(Locale::getDefault(), UNUM_DECIMAL, status);
 }
 
 // -------------------------------------
@@ -393,7 +679,7 @@ NumberFormat::createInstance(UErrorCode& status)
 NumberFormat* U_EXPORT2
 NumberFormat::createInstance(const Locale& inLocale, UErrorCode& status)
 {
-    return createInstance(inLocale, kNumberStyle, status);
+    return createInstance(inLocale, UNUM_DECIMAL, status);
 }
 
 // -------------------------------------
@@ -410,8 +696,8 @@ NumberFormat::createCurrencyInstance(UErrorCode& status)
 
 NumberFormat* U_EXPORT2
 NumberFormat::createCurrencyInstance(const Locale& inLocale, UErrorCode& status)
-{  
-    return createInstance(inLocale, kCurrencyStyle, status);
+{
+    return createInstance(inLocale, UNUM_CURRENCY, status);
 }
 
 // -------------------------------------
@@ -420,7 +706,7 @@ NumberFormat::createCurrencyInstance(const Locale& inLocale, UErrorCode& status)
 NumberFormat* U_EXPORT2
 NumberFormat::createPercentInstance(UErrorCode& status)
 {
-    return createInstance(Locale::getDefault(), kPercentStyle, status);
+    return createInstance(Locale::getDefault(), UNUM_PERCENT, status);
 }
 
 // -------------------------------------
@@ -429,7 +715,7 @@ NumberFormat::createPercentInstance(UErrorCode& status)
 NumberFormat* U_EXPORT2
 NumberFormat::createPercentInstance(const Locale& inLocale, UErrorCode& status)
 {
-    return createInstance(inLocale, kPercentStyle, status);
+    return createInstance(inLocale, UNUM_PERCENT, status);
 }
 
 // -------------------------------------
@@ -438,7 +724,7 @@ NumberFormat::createPercentInstance(const Locale& inLocale, UErrorCode& status)
 NumberFormat* U_EXPORT2
 NumberFormat::createScientificInstance(UErrorCode& status)
 {
-    return createInstance(Locale::getDefault(), kScientificStyle, status);
+    return createInstance(Locale::getDefault(), UNUM_SCIENTIFIC, status);
 }
 
 // -------------------------------------
@@ -447,7 +733,7 @@ NumberFormat::createScientificInstance(UErrorCode& status)
 NumberFormat* U_EXPORT2
 NumberFormat::createScientificInstance(const Locale& inLocale, UErrorCode& status)
 {
-    return createInstance(inLocale, kScientificStyle, status);
+    return createInstance(inLocale, UNUM_SCIENTIFIC, status);
 }
 
 // -------------------------------------
@@ -465,28 +751,13 @@ NumberFormat::getAvailableLocales(int32_t& count)
 //-------------------------------------------
 
 #if !UCONFIG_NO_SERVICE
-static ICULocaleService* gService = NULL;
-
-/**
- * Release all static memory held by numberformat.  
- */
-U_CDECL_BEGIN
-static UBool U_CALLCONV numfmt_cleanup(void) {
-    if (gService) {
-        delete gService;
-        gService = NULL;
-    }
-    return TRUE;
-}
-U_CDECL_END
 
 // -------------------------------------
 
 class ICUNumberFormatFactory : public ICUResourceBundleFactory {
 protected:
     virtual UObject* handleCreate(const Locale& loc, int32_t kind, const ICUService* /* service */, UErrorCode& status) const {
-        // !!! kind is not an EStyles, need to determine how to handle this
-        return NumberFormat::makeInstance(loc, (NumberFormat::EStyles)kind, status);
+        return NumberFormat::makeInstance(loc, (UNumberFormatStyle)kind, status);
     }
 };
 
@@ -498,7 +769,7 @@ private:
     Hashtable* _ids;
 
 public:
-    NFFactory(NumberFormatFactory* delegate) 
+    NFFactory(NumberFormatFactory* delegate)
         : LocaleKeyFactory(delegate->visible() ? VISIBLE : INVISIBLE)
         , _delegate(delegate)
         , _ids(NULL)
@@ -519,7 +790,7 @@ public:
             lkey.canonicalLocale(loc);
             int32_t kind = lkey.kind();
 
-            UObject* result = _delegate->createFormat(loc, (UNumberFormatStyle)(kind+1));
+            UObject* result = _delegate->createFormat(loc, (UNumberFormatStyle)kind);
             if (result == NULL) {
                 result = service->getKey((ICUServiceKey&)key /* cast away const */, NULL, this, status);
             }
@@ -530,7 +801,7 @@ public:
 
 protected:
     /**
-     * Return the set of ids that this factory supports (visible or 
+     * Return the set of ids that this factory supports (visible or
      * otherwise).  This can be called often and might need to be
      * cached if it is expensive to create.
      */
@@ -571,7 +842,7 @@ public:
         int32_t kind = lkey.kind();
         Locale loc;
         lkey.currentLocale(loc);
-        return NumberFormat::makeInstance(loc, (NumberFormat::EStyles)kind, status);
+        return NumberFormat::makeInstance(loc, (UNumberFormatStyle)kind, status);
     }
 
     virtual UBool isDefault() const {
@@ -581,30 +852,26 @@ public:
 
 // -------------------------------------
 
-static ICULocaleService* 
+static ICULocaleService*
 getNumberFormatService(void)
 {
     UBool needInit;
-    {
-        Mutex mutex;
-        needInit = (UBool)(gService == NULL);
-    }
+    UMTX_CHECK(NULL, (UBool)(gService == NULL), needInit);
     if (needInit) {
         ICULocaleService * newservice = new ICUNumberFormatService();
         if (newservice) {
-            Mutex mutex;
+            umtx_lock(NULL);
             if (gService == NULL) {
                 gService = newservice;
                 newservice = NULL;
             }
+            umtx_unlock(NULL);
         }
         if (newservice) {
             delete newservice;
         } else {
             // we won the contention, this thread can register cleanup.
-#if !UCONFIG_NO_SERVICE
             ucln_i18n_registerCleanup(UCLN_I18N_NUMFMT, numfmt_cleanup);
-#endif
         }
     }
     return gService;
@@ -617,7 +884,10 @@ NumberFormat::registerFactory(NumberFormatFactory* toAdopt, UErrorCode& status)
 {
   ICULocaleService *service = getNumberFormatService();
   if (service) {
-    return service->registerFactory(new NFFactory(toAdopt), status);
+	  NFFactory *tempnnf = new NFFactory(toAdopt);
+	  if (tempnnf != NULL) {
+		  return service->registerFactory(tempnnf, status);
+	  }
   }
   status = U_MEMORY_ALLOCATION_ERROR;
   return NULL;
@@ -629,9 +899,8 @@ UBool U_EXPORT2
 NumberFormat::unregister(URegistryKey key, UErrorCode& status)
 {
     if (U_SUCCESS(status)) {
-        umtx_lock(NULL);
-        UBool haveService = gService != NULL;
-        umtx_unlock(NULL);
+        UBool haveService;
+        UMTX_CHECK(NULL, gService != NULL, haveService);
         if (haveService) {
             return gService->unregister(key, status);
         }
@@ -654,12 +923,11 @@ NumberFormat::getAvailableLocales(void)
 // -------------------------------------
 
 NumberFormat* U_EXPORT2
-NumberFormat::createInstance(const Locale& loc, EStyles kind, UErrorCode& status)
+NumberFormat::createInstance(const Locale& loc, UNumberFormatStyle kind, UErrorCode& status)
 {
 #if !UCONFIG_NO_SERVICE
-    umtx_lock(NULL);
-    UBool haveService = gService != NULL;
-    umtx_unlock(NULL);
+    UBool haveService;
+    UMTX_CHECK(NULL, gService != NULL, haveService);
     if (haveService) {
         return (NumberFormat*)gService->get(loc, kind, status);
     }
@@ -814,33 +1082,51 @@ void NumberFormat::getEffectiveCurrency(UChar* result, UErrorCode& ec) const {
 // Creates the NumberFormat instance of the specified style (number, currency,
 // or percent) for the desired locale.
 
+UBool
+NumberFormat::isStyleSupported(UNumberFormatStyle style) {
+    return gLastResortNumberPatterns[style] != NULL;
+}
+
 NumberFormat*
 NumberFormat::makeInstance(const Locale& desiredLocale,
-                           EStyles style,
+                           UNumberFormatStyle style,
                            UErrorCode& status)
 {
     if (U_FAILURE(status)) return NULL;
 
-    if (style < 0 || style >= kStyleCount) {
+    if (style < 0 || style >= UNUM_FORMAT_STYLE_COUNT) {
         status = U_ILLEGAL_ARGUMENT_ERROR;
         return NULL;
     }
 
+    // Some styles are not supported. This is a result of merging
+    // the @draft ICU 4.2 NumberFormat::EStyles into the long-existing UNumberFormatStyle.
+    // Ticket #8503 is for reviewing/fixing/merging the two relevant implementations:
+    // this one and unum_open().
+    // The UNUM_PATTERN_ styles are not supported here
+    // because this method does not take a pattern string.
+    if (!isStyleSupported(style)) {
+        status = U_UNSUPPORTED_ERROR;
+        return NULL;
+    }
+
 #ifdef U_WINDOWS
     char buffer[8];
     int32_t count = desiredLocale.getKeywordValue("compat", buffer, sizeof(buffer), status);
 
     // if the locale has "@compat=host", create a host-specific NumberFormat
-    if (count > 0 && uprv_strcmp(buffer, "host") == 0) {
+    if (U_SUCCESS(status) && count > 0 && uprv_strcmp(buffer, "host") == 0) {
         Win32NumberFormat *f = NULL;
         UBool curr = TRUE;
 
         switch (style) {
-        case kNumberStyle:
+        case UNUM_DECIMAL:
             curr = FALSE;
             // fall-through
 
-        case kCurrencyStyle:
+        case UNUM_CURRENCY:
+        case UNUM_CURRENCY_ISO: // do not support plural formatting here
+        case UNUM_CURRENCY_PLURAL:
             f = new Win32NumberFormat(desiredLocale, curr, status);
 
             if (U_SUCCESS(status)) {
@@ -849,76 +1135,172 @@ NumberFormat::makeInstance(const Locale& desiredLocale,
 
             delete f;
             break;
-            
+
         default:
             break;
         }
     }
 #endif
 
-    NumberFormat* f = NULL;
-    DecimalFormatSymbols* symbolsToAdopt = NULL;
+    LocalPointer<DecimalFormatSymbols> symbolsToAdopt;
     UnicodeString pattern;
-    UResourceBundle *resource = ures_open((char *)0, desiredLocale.getName(), &status);
-    UResourceBundle *numberPatterns = ures_getByKey(resource, DecimalFormat::fgNumberPatterns, NULL, &status);
-
+    LocalUResourceBundlePointer ownedResource(ures_open(NULL, desiredLocale.getName(), &status));
     if (U_FAILURE(status)) {
         // We don't appear to have resource data available -- use the last-resort data
         status = U_USING_FALLBACK_WARNING;
         // When the data is unavailable, and locale isn't passed in, last resort data is used.
-        symbolsToAdopt = new DecimalFormatSymbols(status);
+        symbolsToAdopt.adoptInstead(new DecimalFormatSymbols(status));
+        if (symbolsToAdopt.isNull()) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return NULL;
+        }
 
         // Creates a DecimalFormat instance with the last resort number patterns.
         pattern.setTo(TRUE, gLastResortNumberPatterns[style], -1);
     }
     else {
-        // If not all the styled patterns exists for the NumberFormat in this locale,
-        // sets the status code to failure and returns nil.
-        if (ures_getSize(numberPatterns) < (int32_t)(sizeof(gLastResortNumberPatterns)/sizeof(gLastResortNumberPatterns[0]))) {
-            status = U_INVALID_FORMAT_ERROR;
-            goto cleanup;
+        // Loads the decimal symbols of the desired locale.
+        symbolsToAdopt.adoptInstead(new DecimalFormatSymbols(desiredLocale, status));
+        if (symbolsToAdopt.isNull()) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return NULL;
         }
 
-        // Loads the decimal symbols of the desired locale.
-        symbolsToAdopt = new DecimalFormatSymbols(desiredLocale, status);
+        UResourceBundle *resource = ownedResource.orphan();
+        resource = ures_getByKeyWithFallback(resource, gNumberElements, resource, &status);
+        // TODO : Get patterns on a per numbering system basis, for right now assumes "latn" for patterns
+        resource = ures_getByKeyWithFallback(resource, gLatn, resource, &status);
+        resource = ures_getByKeyWithFallback(resource, gPatterns, resource, &status);
+        ownedResource.adoptInstead(resource);
 
         int32_t patLen = 0;
-        const UChar *patResStr = ures_getStringByIndex(numberPatterns, (int32_t)style, &patLen, &status);
+        const UChar *patResStr = ures_getStringByKeyWithFallback(resource, gFormatKeys[style], &patLen, &status);
+
         // Creates the specified decimal format style of the desired locale.
         pattern.setTo(TRUE, patResStr, patLen);
     }
-    if (U_FAILURE(status) || symbolsToAdopt == NULL) {
-        goto cleanup;
+    if (U_FAILURE(status)) {
+        return NULL;
     }
-    if(style==kCurrencyStyle){
+    if(style==UNUM_CURRENCY || style == UNUM_CURRENCY_ISO){
         const UChar* currPattern = symbolsToAdopt->getCurrencyPattern();
         if(currPattern!=NULL){
             pattern.setTo(currPattern, u_strlen(currPattern));
         }
     }
-    f = new DecimalFormat(pattern, symbolsToAdopt, status);
-    if (U_FAILURE(status) || f == NULL) {
-        goto cleanup;
+
+    // Use numbering system cache hashtable
+    UHashtable *cache;
+    UMTX_CHECK(&nscacheMutex, NumberingSystem_cache, cache);
+
+    // Check cache we got, create if non-existant
+    if (cache == NULL) {
+        cache = uhash_open(uhash_hashLong,
+                           uhash_compareLong,
+                           NULL,
+                           &status);
+
+        if (U_FAILURE(status)) {
+            // cache not created - out of memory
+            status = U_ZERO_ERROR;  // work without the cache
+            cache = NULL;
+        } else {
+            // cache created
+            uhash_setValueDeleter(cache, deleteNumberingSystem);
+
+            // set final NumberingSystem_cache value
+            Mutex lock(&nscacheMutex);
+            if (NumberingSystem_cache == NULL) {
+                NumberingSystem_cache = cache;
+                ucln_i18n_registerCleanup(UCLN_I18N_NUMFMT, numfmt_cleanup);
+            } else {
+                uhash_close(cache);
+                cache = NumberingSystem_cache;
+            }
+        }
     }
 
-    f->setLocaleIDs(ures_getLocaleByType(numberPatterns, ULOC_VALID_LOCALE, &status),
-                    ures_getLocaleByType(numberPatterns, ULOC_ACTUAL_LOCALE, &status));
-cleanup:
-    ures_close(numberPatterns);
-    ures_close(resource);
+    // Get cached numbering system
+    LocalPointer<NumberingSystem> ownedNs;
+    NumberingSystem *ns = NULL;
+    if (cache != NULL) {
+        // TODO: Bad hash key usage, see ticket #8504.
+        int32_t hashKey = desiredLocale.hashCode();
+
+        Mutex lock(&nscacheMutex);
+        ns = (NumberingSystem *)uhash_iget(cache, hashKey);
+        if (ns == NULL) {
+            ns = NumberingSystem::createInstance(desiredLocale,status);
+            uhash_iput(cache, hashKey, (void*)ns, &status);
+        }
+    } else {
+        ownedNs.adoptInstead(NumberingSystem::createInstance(desiredLocale,status));
+        ns = ownedNs.getAlias();
+    }
+
+    // check results of getting a numbering system
     if (U_FAILURE(status)) {
-        /* If f exists, then it will delete the symbols */
-        if (f==NULL) {
-            delete symbolsToAdopt;
-        }
-        else {
-            delete f;
-        }
         return NULL;
     }
-    if (f == NULL || symbolsToAdopt == NULL) {
-        status = U_MEMORY_ALLOCATION_ERROR;
-        f = NULL;
+
+    NumberFormat *f;
+    if (ns->isAlgorithmic()) {
+        UnicodeString nsDesc;
+        UnicodeString nsRuleSetGroup;
+        UnicodeString nsRuleSetName;
+        Locale nsLoc;
+        URBNFRuleSetTag desiredRulesType = URBNF_NUMBERING_SYSTEM;
+
+        nsDesc.setTo(ns->getDescription());
+        int32_t firstSlash = nsDesc.indexOf(gSlash);
+        int32_t lastSlash = nsDesc.lastIndexOf(gSlash);
+        if ( lastSlash > firstSlash ) {
+            CharString nsLocID;
+
+            nsLocID.appendInvariantChars(nsDesc.tempSubString(0, firstSlash), status);
+            nsRuleSetGroup.setTo(nsDesc,firstSlash+1,lastSlash-firstSlash-1);
+            nsRuleSetName.setTo(nsDesc,lastSlash+1);
+
+            nsLoc = Locale::createFromName(nsLocID.data());
+
+            UnicodeString SpelloutRules = UNICODE_STRING_SIMPLE("SpelloutRules");
+            if ( nsRuleSetGroup.compare(SpelloutRules) == 0 ) {
+                desiredRulesType = URBNF_SPELLOUT;
+            }
+        } else {
+            nsLoc = desiredLocale;
+            nsRuleSetName.setTo(nsDesc);
+        }
+
+        RuleBasedNumberFormat *r = new RuleBasedNumberFormat(desiredRulesType,nsLoc,status);
+        if (r == NULL) {
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return NULL;
+        }
+        r->setDefaultRuleSet(nsRuleSetName,status);
+        f = r;
+    } else {
+        // replace single currency sign in the pattern with double currency sign
+        // if the style is UNUM_CURRENCY_ISO
+        if (style == UNUM_CURRENCY_ISO) {
+            pattern.findAndReplace(gSingleCurrencySign, gDoubleCurrencySign);
+        }
+
+        // "new DecimalFormat()" does not adopt the symbols if its memory allocation fails.
+        DecimalFormatSymbols *syms = symbolsToAdopt.orphan();
+        f = new DecimalFormat(pattern, syms, style, status);
+        if (f == NULL) {
+            delete syms;
+            status = U_MEMORY_ALLOCATION_ERROR;
+            return NULL;
+        }
+    }
+
+    f->setLocaleIDs(ures_getLocaleByType(ownedResource.getAlias(), ULOC_VALID_LOCALE, &status),
+                    ures_getLocaleByType(ownedResource.getAlias(), ULOC_ACTUAL_LOCALE, &status));
+    if (U_FAILURE(status)) {
+        delete f;
+        return NULL;
     }
     return f;
 }
