@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (C) 1997-2010, International Business Machines Corporation and    *
+* Copyright (C) 1997-2011, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 *
@@ -35,8 +35,11 @@
 *                           available IDs code.  Misc. cleanup.
 *********************************************************************************/
 
+#include <typeinfo>  // for 'typeid' to work
+
 #include "unicode/utypes.h"
 #include "unicode/ustring.h"
+#include "ustr_imp.h"
 
 #ifdef U_DEBUG_TZ
 # include <stdio.h>
@@ -101,6 +104,7 @@ static char gStrBuf[256];
 static const UChar         WORLD[] = {0x30, 0x30, 0x31, 0x00}; /* "001" */
 
 static const UChar         GMT_ID[] = {0x47, 0x4D, 0x54, 0x00}; /* "GMT" */
+static const UChar         UNKNOWN_ZONE_ID[] = {0x45, 0x74, 0x63, 0x2F, 0x55, 0x6E, 0x6B, 0x6E, 0x6F, 0x77, 0x6E, 0x00}; /* "Etc/Unknown" */
 static const UChar         Z_STR[] = {0x7A, 0x00}; /* "z" */
 static const UChar         ZZZZ_STR[] = {0x7A, 0x7A, 0x7A, 0x7A, 0x00}; /* "zzzz" */
 static const UChar         Z_UC_STR[] = {0x5A, 0x00}; /* "Z" */
@@ -110,6 +114,7 @@ static const UChar         VVVV_STR[] = {0x76, 0x76, 0x76, 0x76, 0x00}; /* "vvvv
 static const UChar         V_UC_STR[] = {0x56, 0x00}; /* "V" */
 static const UChar         VVVV_UC_STR[] = {0x56, 0x56, 0x56, 0x56, 0x00}; /* "VVVV" */
 static const int32_t       GMT_ID_LENGTH = 3;
+static const int32_t       UNKNOWN_ZONE_ID_LENGTH = 11;
 
 static UMTX                             LOCK;
 static UMTX                             TZSET_LOCK;
@@ -119,6 +124,14 @@ static U_NAMESPACE_QUALIFIER TimeZone*  _GMT = NULL; // cf. TimeZone::GMT
 static char TZDATA_VERSION[16];
 static UBool TZDataVersionInitialized = FALSE;
 
+static int32_t* MAP_SYSTEM_ZONES = NULL;
+static int32_t* MAP_CANONICAL_SYSTEM_ZONES = NULL;
+static int32_t* MAP_CANONICAL_SYSTEM_LOCATION_ZONES = NULL;
+
+int32_t LEN_SYSTEM_ZONES = 0;
+int32_t LEN_CANONICAL_SYSTEM_ZONES = 0;
+int32_t LEN_CANONICAL_SYSTEM_LOCATION_ZONES = 0;
+
 U_CDECL_BEGIN
 static UBool U_CALLCONV timeZone_cleanup(void)
 {
@@ -131,6 +144,18 @@ static UBool U_CALLCONV timeZone_cleanup(void)
     uprv_memset(TZDATA_VERSION, 0, sizeof(TZDATA_VERSION));
     TZDataVersionInitialized = FALSE;
 
+    LEN_SYSTEM_ZONES = 0;
+    uprv_free(MAP_SYSTEM_ZONES);
+    MAP_SYSTEM_ZONES = 0;
+
+    LEN_CANONICAL_SYSTEM_ZONES = 0;
+    uprv_free(MAP_CANONICAL_SYSTEM_ZONES);
+    MAP_CANONICAL_SYSTEM_ZONES = 0;
+
+    LEN_CANONICAL_SYSTEM_LOCATION_ZONES = 0;
+    uprv_free(MAP_CANONICAL_SYSTEM_LOCATION_ZONES);
+    MAP_CANONICAL_SYSTEM_LOCATION_ZONES = 0;
+
     if (LOCK) {
         umtx_destroy(&LOCK);
         LOCK = NULL;
@@ -146,64 +171,21 @@ U_CDECL_END
 
 U_NAMESPACE_BEGIN
 
-/**
- * The Olson data is stored the "zoneinfo" resource bundle.
- * Sub-resources are organized into three ranges of data: Zones, final
- * rules, and country tables.  There is also a meta-data resource
- * which has 3 integers: The number of zones, rules, and countries,
- * respectively.  The country count includes the non-country 'Default'.
- */
-static int32_t OLSON_ZONE_COUNT = 0;  // count of zones
-
-/**
- * Given a pointer to an open "zoneinfo" resource, load up the Olson
- * meta-data. Return TRUE if successful.
- */
-static UBool getOlsonMeta(const UResourceBundle* top) {
-    if (OLSON_ZONE_COUNT == 0) {
-        UErrorCode ec = U_ZERO_ERROR;
-        UResourceBundle res;
-        ures_initStackObject(&res);
-        ures_getByKey(top, kZONES, &res, &ec);
-        if(U_SUCCESS(ec)) {
-            OLSON_ZONE_COUNT = ures_getSize(&res);
-            U_DEBUG_TZ_MSG(("OZC%d\n",OLSON_ZONE_COUNT));
-        }
-        ures_close(&res);
-    }
-    return (OLSON_ZONE_COUNT > 0);
-}
-
-/**
- * Load up the Olson meta-data. Return TRUE if successful.
- */
-static UBool getOlsonMeta() {
-    if (OLSON_ZONE_COUNT == 0) {
-        UErrorCode ec = U_ZERO_ERROR;
-        UResourceBundle *top = ures_openDirect(0, kZONEINFO, &ec);
-        if (U_SUCCESS(ec)) {
-            getOlsonMeta(top);
-        }
-        ures_close(top);
-    }
-    return (OLSON_ZONE_COUNT > 0);
-}
-
 static int32_t findInStringArray(UResourceBundle* array, const UnicodeString& id, UErrorCode &status)
 {
     UnicodeString copy;
     const UChar *u;
     int32_t len;
-    
+
     int32_t start = 0;
     int32_t limit = ures_getSize(array);
     int32_t mid;
     int32_t lastMid = INT32_MAX;
-    if(U_FAILURE(status) || (limit < 1)) { 
+    if(U_FAILURE(status) || (limit < 1)) {
         return -1;
     }
     U_DEBUG_TZ_MSG(("fisa: Looking for %s, between %d and %d\n", U_DEBUG_TZ_STR(UnicodeString(id).getTerminatedBuffer()), start, limit));
-    
+
     for (;;) {
         mid = (int32_t)((start + limit) / 2);
         if (lastMid == mid) {   /* Have we moved? */
@@ -231,7 +213,7 @@ static int32_t findInStringArray(UResourceBundle* array, const UnicodeString& id
 }
 
 /**
- * Fetch a specific zone by name.  Replaces the getByKey call. 
+ * Fetch a specific zone by name.  Replaces the getByKey call.
  * @param top Top timezone resource
  * @param id Time zone ID
  * @param oldbundle Bundle for reuse (or NULL).   see 'ures_open()'
@@ -240,12 +222,12 @@ static int32_t findInStringArray(UResourceBundle* array, const UnicodeString& id
 static UResourceBundle* getZoneByName(const UResourceBundle* top, const UnicodeString& id, UResourceBundle *oldbundle, UErrorCode& status) {
     // load the Rules object
     UResourceBundle *tmp = ures_getByKey(top, kNAMES, NULL, &status);
-    
+
     // search for the string
     int32_t idx = findInStringArray(tmp, id, status);
-    
+
     if((idx == -1) && U_SUCCESS(status)) {
-        // not found 
+        // not found
         status = U_MISSING_RESOURCE_ERROR;
         //ures_close(oldbundle);
         //oldbundle = NULL;
@@ -257,7 +239,7 @@ static UResourceBundle* getZoneByName(const UResourceBundle* top, const UnicodeS
         U_DEBUG_TZ_MSG(("gzbn: loaded z#%d, size %d, type %d, path %s, %s\n", idx, ures_getSize(oldbundle), ures_getType(oldbundle), ures_getPath(oldbundle),  u_errorName(status)));
     }
     ures_close(tmp);
-    if(U_FAILURE(status)) { 
+    if(U_FAILURE(status)) {
         //ures_close(oldbundle);
         return NULL;
     } else {
@@ -299,7 +281,7 @@ static UResourceBundle* openOlsonResource(const UnicodeString& id,
     // Dereference if this is an alias.  Docs say result should be 1
     // but it is 0 in 2.8 (?).
     U_DEBUG_TZ_MSG(("Loading zone '%s' (%s, size %d) - %s\n", buf, ures_getKey((UResourceBundle*)&res), ures_getSize(&res), u_errorName(ec)));
-    if (ures_getType(&res) == URES_INT && getOlsonMeta(top)) {
+    if (ures_getType(&res) == URES_INT) {
         int32_t deref = ures_getInt(&res, &ec) + 0;
         U_DEBUG_TZ_MSG(("getInt: %s - type is %d\n", u_errorName(ec), ures_getType(&res)));
         UResourceBundle *ares = ures_getByKey(top, kZONES, NULL, &ec); // dereference Zones section
@@ -382,7 +364,7 @@ TimeZone::operator=(const TimeZone &right)
 UBool
 TimeZone::operator==(const TimeZone& that) const
 {
-    return getDynamicClassID() == that.getDynamicClassID() &&
+    return typeid(*this) == typeid(that) &&
         fID == that.fID;
 }
 
@@ -406,13 +388,8 @@ TimeZone::createTimeZone(const UnicodeString& ID)
         result = createCustomTimeZone(ID);
     }
     if (result == 0) {
-        U_DEBUG_TZ_MSG(("failed to load time zone with id - falling to GMT"));
-        const TimeZone* temptz = getGMT();
-        if (temptz == NULL) {
-            result = NULL;
-        } else {
-            result = temptz->clone();
-        }
+        U_DEBUG_TZ_MSG(("failed to load time zone with id - falling to Etc/Unknown(GMT)"));
+        result = new SimpleTimeZone(0, UNKNOWN_ZONE_ID);
     }
     return result;
 }
@@ -424,18 +401,24 @@ TimeZone::createTimeZone(const UnicodeString& ID)
  */
 TimeZone*
 TimeZone::createSystemTimeZone(const UnicodeString& id) {
-    TimeZone* z = 0;
     UErrorCode ec = U_ZERO_ERROR;
+    return createSystemTimeZone(id, ec);
+}
+
+TimeZone*
+TimeZone::createSystemTimeZone(const UnicodeString& id, UErrorCode& ec) {
+    if (U_FAILURE(ec)) {
+        return NULL;
+    }
+    TimeZone* z = 0;
     UResourceBundle res;
     ures_initStackObject(&res);
     U_DEBUG_TZ_MSG(("pre-err=%s\n", u_errorName(ec)));
     UResourceBundle *top = openOlsonResource(id, res, ec);
     U_DEBUG_TZ_MSG(("post-err=%s\n", u_errorName(ec)));
     if (U_SUCCESS(ec)) {
-        z = new OlsonTimeZone(top, &res, ec);
-        if (z) {
-          z->setID(id);
-        } else {
+        z = new OlsonTimeZone(top, &res, id, ec);
+        if (z == NULL) {
           U_DEBUG_TZ_MSG(("cstz: olson time zone failed to initialize - err %s\n", u_errorName(ec)));
         }
     }
@@ -461,7 +444,7 @@ TimeZone::createSystemTimeZone(const UnicodeString& id) {
  */
 void
 TimeZone::initDefault()
-{ 
+{
     // We access system timezone data through TPlatformUtilities,
     // including tzset(), timezone, and tzname[].
     int32_t rawOffset = 0;
@@ -484,13 +467,13 @@ TimeZone::initDefault()
 
         ucln_i18n_registerCleanup(UCLN_I18N_TIMEZONE, timeZone_cleanup);
         uprv_tzset(); // Initialize tz... system data
-        
+
         // Get the timezone ID from the host.  This function should do
         // any required host-specific remapping; e.g., on Windows this
         // function maps the Date and Time control panel setting to an
         // ICU timezone ID.
         hostID = uprv_tzname(0);
-        
+
         // Invert sign because UNIX semantics are backwards
         rawOffset = uprv_timezone() * -U_MILLIS_PER_SECOND;
     }
@@ -549,7 +532,7 @@ TimeZone::initDefault()
         ucln_i18n_registerCleanup(UCLN_I18N_TIMEZONE, timeZone_cleanup);
     }
     umtx_unlock(&LOCK);
-    
+
     delete default_zone;
 }
 
@@ -657,11 +640,18 @@ private:
     // Map into to zones.  Our results are zone[map[i]] for
     // i=0..len-1, where zone[i] is the i-th Olson zone.  If map==NULL
     // then our results are zone[i] for i=0..len-1.  Len will be zero
-    // iff the zone data could not be loaded.
+    // if the zone data could not be loaded.
     int32_t* map;
+    int32_t* localMap;
     int32_t  len;
     int32_t  pos;
 
+    TZEnumeration(int32_t* mapData, int32_t mapLen, UBool adoptMapData) : pos(0) {
+        map = mapData;
+        localMap = adoptMapData ? mapData : NULL;
+        len = mapLen;
+    }
+
     UBool getID(int32_t i) {
         UErrorCode ec = U_ZERO_ERROR;
         int32_t idLen = 0;
@@ -679,118 +669,272 @@ private:
         return U_SUCCESS(ec);
     }
 
-public:
-    TZEnumeration() : map(NULL), len(0), pos(0) {
-        if (getOlsonMeta()) {
-            len = OLSON_ZONE_COUNT;
+    static int32_t* getMap(USystemTimeZoneType type, int32_t& len, UErrorCode& ec) {
+        len = 0;
+        if (U_FAILURE(ec)) {
+            return NULL;
         }
+        int32_t* m = NULL;
+        switch (type) {
+        case UCAL_ZONE_TYPE_ANY:
+            m = MAP_SYSTEM_ZONES;
+            len = LEN_SYSTEM_ZONES;
+            break;
+        case UCAL_ZONE_TYPE_CANONICAL:
+            m = MAP_CANONICAL_SYSTEM_ZONES;
+            len = LEN_CANONICAL_SYSTEM_ZONES;
+            break;
+        case UCAL_ZONE_TYPE_CANONICAL_LOCATION:
+            m = MAP_CANONICAL_SYSTEM_LOCATION_ZONES;
+            len = LEN_CANONICAL_SYSTEM_LOCATION_ZONES;
+            break;
+        }
+        UBool needsInit = FALSE;
+        UMTX_CHECK(&LOCK, (len == 0), needsInit);
+        if (needsInit) {
+            m = initMap(type, len, ec);
+        }
+        return m;
     }
 
-    TZEnumeration(int32_t rawOffset) : map(NULL), len(0), pos(0) {
-        if (!getOlsonMeta()) {
-            return;
+    static int32_t* initMap(USystemTimeZoneType type, int32_t& len, UErrorCode& ec) {
+        len = 0;
+        if (U_FAILURE(ec)) {
+            return NULL;
         }
 
-        // Allocate more space than we'll need.  The end of the array will
-        // be blank.
-        map = (int32_t*)uprv_malloc(OLSON_ZONE_COUNT * sizeof(int32_t));
-        if (map == 0) {
-            return;
-        }
+        int32_t *result = NULL;
 
-        uprv_memset(map, 0, sizeof(int32_t) * OLSON_ZONE_COUNT);
-
-        UnicodeString s;
-        for (int32_t i=0; i<OLSON_ZONE_COUNT; ++i) {
-            if (getID(i)) {
-                // This is VERY inefficient.
-                TimeZone* z = TimeZone::createTimeZone(unistr);
-                // Make sure we get back the ID we wanted (if the ID is
-                // invalid we get back GMT).
-                if (z != 0 && z->getID(s) == unistr &&
-                    z->getRawOffset() == rawOffset) {
-                    map[len++] = i;
-                }
-                delete z;
-            }
-        }
-    }
-
-    TZEnumeration(const char* country) : map(NULL), len(0), pos(0) {
-        if (!getOlsonMeta()) {
-            return;
-        }
-
-        UErrorCode ec = U_ZERO_ERROR;
         UResourceBundle *res = ures_openDirect(0, kZONEINFO, &ec);
-        ures_getByKey(res, kREGIONS, res, &ec);
-        if (U_SUCCESS(ec) && ures_getType(res) == URES_ARRAY) {
-            UChar uCountry[] = {0, 0, 0, 0};
-            if (country) {
-                u_charsToUChars(country, uCountry, 2);
+        res = ures_getByKey(res, kNAMES, res, &ec); // dereference Zones section
+        if (U_SUCCESS(ec)) {
+            int32_t size = ures_getSize(res);
+            int32_t *m = (int32_t *)uprv_malloc(size * sizeof(int32_t));
+            if (m == NULL) {
+                ec = U_MEMORY_ALLOCATION_ERROR;
             } else {
-                u_strcpy(uCountry, WORLD);
-            }
-
-            // count matches
-            int32_t count = 0;
-            int32_t i;
-            const UChar *region;
-            for (i = 0; i < ures_getSize(res); i++) {
-                region = ures_getStringByIndex(res, i, NULL, &ec);
-                if (U_FAILURE(ec)) {
-                    break;
-                }
-                if (u_strcmp(uCountry, region) == 0) {
-                    count++;
-                }
-            }
-
-            if (count > 0) {
-                map = (int32_t*)uprv_malloc(sizeof(int32_t) * count);
-                if (map != NULL) {
-                    int32_t idx = 0;
-                    for (i = 0; i < ures_getSize(res); i++) {
-                        region = ures_getStringByIndex(res, i, NULL, &ec);
+                int32_t numEntries = 0;
+                for (int32_t i = 0; i < size; i++) {
+                    const UChar *id = ures_getStringByIndex(res, i, NULL, &ec);
+                    if (U_FAILURE(ec)) {
+                        break;
+                    }
+                    if (u_strcmp(id, UNKNOWN_ZONE_ID) == 0) {
+                        // exclude Etc/Unknown
+                        continue;
+                    }
+                    if (type == UCAL_ZONE_TYPE_CANONICAL || type == UCAL_ZONE_TYPE_CANONICAL_LOCATION) {
+                        UnicodeString canonicalID;
+                        ZoneMeta::getCanonicalCLDRID(id, canonicalID, ec);
                         if (U_FAILURE(ec)) {
                             break;
                         }
-                        if (u_strcmp(uCountry, region) == 0) {
-                            map[idx++] = i;
+                        if (canonicalID.compare(id, -1) != 0) {
+                            // exclude aliases
+                            continue;
                         }
                     }
-                    if (U_SUCCESS(ec)) {
-                        len = count;
-                    } else {
-                        uprv_free(map);
-                        map = NULL;
+                    if (type == UCAL_ZONE_TYPE_CANONICAL_LOCATION) {
+                        const UChar *region = TimeZone::getRegion(id, ec);
+                        if (U_FAILURE(ec)) {
+                            break;
+                        }
+                        if (u_strcmp(region, WORLD) == 0) {
+                           // exclude non-location ("001")
+                            continue;
+                        }
                     }
-                } else {
-                    U_DEBUG_TZ_MSG(("Failed to load tz for region %s: %s\n", country, u_errorName(ec)));
+                    m[numEntries++] = i;
                 }
+                if (U_SUCCESS(ec)) {
+                    int32_t *tmp = m;
+                    m = (int32_t *)uprv_realloc(tmp, numEntries * sizeof(int32_t));
+                    if (m == NULL) {
+                        // realloc failed.. use the original one even it has unused
+                        // area at the end
+                        m = tmp;
+                    }
+
+                    umtx_lock(&LOCK);
+                    {
+                        switch(type) {
+                        case UCAL_ZONE_TYPE_ANY:
+                            if (MAP_SYSTEM_ZONES == NULL) {
+                                MAP_SYSTEM_ZONES = m;
+                                LEN_SYSTEM_ZONES = numEntries;
+                                m = NULL;
+                                ucln_i18n_registerCleanup(UCLN_I18N_TIMEZONE, timeZone_cleanup);
+                            }
+                            result = MAP_SYSTEM_ZONES;
+                            len = LEN_SYSTEM_ZONES;
+                            break;
+                        case UCAL_ZONE_TYPE_CANONICAL:
+                            if (MAP_CANONICAL_SYSTEM_ZONES == NULL) {
+                                MAP_CANONICAL_SYSTEM_ZONES = m;
+                                LEN_CANONICAL_SYSTEM_ZONES = numEntries;
+                                m = NULL;
+                                ucln_i18n_registerCleanup(UCLN_I18N_TIMEZONE, timeZone_cleanup);
+                            }
+                            result = MAP_CANONICAL_SYSTEM_ZONES;
+                            len = LEN_CANONICAL_SYSTEM_ZONES;
+                            break;
+                        case UCAL_ZONE_TYPE_CANONICAL_LOCATION:
+                            if (MAP_CANONICAL_SYSTEM_LOCATION_ZONES == NULL) {
+                                MAP_CANONICAL_SYSTEM_LOCATION_ZONES = m;
+                                LEN_CANONICAL_SYSTEM_LOCATION_ZONES = numEntries;
+                                m = NULL;
+                                ucln_i18n_registerCleanup(UCLN_I18N_TIMEZONE, timeZone_cleanup);
+                            }
+                            result = MAP_CANONICAL_SYSTEM_LOCATION_ZONES;
+                            len = LEN_CANONICAL_SYSTEM_LOCATION_ZONES;
+                            break;
+                        }
+                    }
+                    umtx_unlock(&LOCK);
+                }
+                uprv_free(m);
             }
         }
+
         ures_close(res);
+        return result;
     }
 
-  TZEnumeration(const TZEnumeration &other) : StringEnumeration(), map(NULL), len(0), pos(0) {
-        if(other.len > 0) {
-            if(other.map != NULL) {
-                map = (int32_t *)uprv_malloc(other.len * sizeof(int32_t));
-                if(map != NULL) {
-                    len = other.len;
-                    uprv_memcpy(map, other.map, len * sizeof(int32_t));
-                    pos = other.pos;
-                }
-            } else {
-                len = other.len;
-                pos = other.pos;
+public:
+
+#define DEFAULT_FILTERED_MAP_SIZE 8
+#define MAP_INCREMENT_SIZE 8
+
+    static TZEnumeration* create(USystemTimeZoneType type, const char* region, const int32_t* rawOffset, UErrorCode& ec) {
+        if (U_FAILURE(ec)) {
+            return NULL;
+        }
+
+        int32_t baseLen;
+        int32_t *baseMap = getMap(type, baseLen, ec);
+
+        if (U_FAILURE(ec)) {
+            return NULL;
+        }
+
+        // If any additional conditions are available,
+        // create instance local map filtered by the conditions.
+
+        int32_t *filteredMap = NULL;
+        int32_t numEntries = 0;
+
+        if (region != NULL || rawOffset != NULL) {
+            int32_t filteredMapSize = DEFAULT_FILTERED_MAP_SIZE;
+            filteredMap = (int32_t *)uprv_malloc(filteredMapSize * sizeof(int32_t));
+            if (filteredMap == NULL) {
+                ec = U_MEMORY_ALLOCATION_ERROR;
+                return NULL;
             }
+
+            // Walk through the base map
+            UResourceBundle *res = ures_openDirect(0, kZONEINFO, &ec);
+            res = ures_getByKey(res, kNAMES, res, &ec); // dereference Zones section
+            for (int32_t i = 0; i < baseLen; i++) {
+                int32_t zidx = baseMap[i];
+                const UChar *id = ures_getStringByIndex(res, zidx, NULL, &ec);
+                if (U_FAILURE(ec)) {
+                    break;
+                }
+                if (region != NULL) {
+                    // Filter by region
+                    char tzregion[4]; // max 3 letters + null term
+                    TimeZone::getRegion(id, tzregion, sizeof(tzregion), ec);
+                    if (U_FAILURE(ec)) {
+                        break;
+                    }
+                    if (uprv_stricmp(tzregion, region) != 0) {
+                        // region does not match
+                        continue;
+                    }
+                }
+                if (rawOffset != NULL) {
+                    // Filter by raw offset
+                    // Note: This is VERY inefficient
+                    TimeZone *z = TimeZone::createSystemTimeZone(id, ec);
+                    if (U_FAILURE(ec)) {
+                        break;
+                    }
+                    int32_t tzoffset = z->getRawOffset();
+                    delete z;
+
+                    if (tzoffset != *rawOffset) {
+                        continue;
+                    }
+                }
+
+                if (filteredMapSize <= numEntries) {
+                    filteredMapSize += MAP_INCREMENT_SIZE;
+                    int32_t *tmp = (int32_t *)uprv_realloc(filteredMap, filteredMapSize * sizeof(int32_t));
+                    if (tmp == NULL) {
+                        ec = U_MEMORY_ALLOCATION_ERROR;
+                        break;
+                    } else {
+                        filteredMap = tmp;
+                    }
+                }
+
+                filteredMap[numEntries++] = zidx;
+            }
+
+            if (U_FAILURE(ec)) {
+                uprv_free(filteredMap);
+                filteredMap = NULL;
+            }
+
+            ures_close(res);
+        }
+
+        TZEnumeration *result = NULL;
+        if (U_SUCCESS(ec)) {
+            // Finally, create a new enumeration instance
+            if (filteredMap == NULL) {
+                result = new TZEnumeration(baseMap, baseLen, FALSE);
+            } else {
+                result = new TZEnumeration(filteredMap, numEntries, TRUE);
+                filteredMap = NULL;
+            }
+            if (result == NULL) {
+                ec = U_MEMORY_ALLOCATION_ERROR;
+            }
+        }
+
+        if (filteredMap != NULL) {
+            uprv_free(filteredMap);
+        }
+
+        return result;
+    }
+
+    TZEnumeration(const TZEnumeration &other) : StringEnumeration(), map(NULL), localMap(NULL), len(0), pos(0) {
+        if (other.localMap != NULL) {
+            localMap = (int32_t *)uprv_malloc(other.len * sizeof(int32_t));
+            if (localMap != NULL) {
+                len = other.len;
+                uprv_memcpy(localMap, other.localMap, len * sizeof(int32_t));
+                pos = other.pos;
+                map = localMap;
+            } else {
+                len = 0;
+                pos = 0;
+                map = NULL;
+            }
+        } else {
+            map = other.map;
+            localMap = NULL;
+            len = other.len;
+            pos = other.pos;
         }
     }
 
     virtual ~TZEnumeration() {
-        uprv_free(map);
+        if (localMap != NULL) {
+            uprv_free(localMap);
+        }
     }
 
     virtual StringEnumeration *clone() const {
@@ -802,8 +946,8 @@ public:
     }
 
     virtual const UnicodeString* snext(UErrorCode& status) {
-        if (U_SUCCESS(status) && pos < len) {
-            getID((map == 0) ? pos : map[pos]);
+        if (U_SUCCESS(status) && map != NULL && pos < len) {
+            getID(map[pos]);
             ++pos;
             return &unistr;
         }
@@ -821,19 +965,31 @@ public:
 
 UOBJECT_DEFINE_RTTI_IMPLEMENTATION(TZEnumeration)
 
+StringEnumeration* U_EXPORT2
+TimeZone::createTimeZoneIDEnumeration(
+            USystemTimeZoneType zoneType,
+            const char* region,
+            const int32_t* rawOffset,
+            UErrorCode& ec) {
+    return TZEnumeration::create(zoneType, region, rawOffset, ec);
+}
+
 StringEnumeration* U_EXPORT2
 TimeZone::createEnumeration() {
-    return new TZEnumeration();
+    UErrorCode ec = U_ZERO_ERROR;
+    return TZEnumeration::create(UCAL_ZONE_TYPE_ANY, NULL, NULL, ec);
 }
 
 StringEnumeration* U_EXPORT2
 TimeZone::createEnumeration(int32_t rawOffset) {
-    return new TZEnumeration(rawOffset);
+    UErrorCode ec = U_ZERO_ERROR;
+    return TZEnumeration::create(UCAL_ZONE_TYPE_ANY, NULL, &rawOffset, ec);
 }
 
 StringEnumeration* U_EXPORT2
 TimeZone::createEnumeration(const char* country) {
-    return new TZEnumeration(country);
+    UErrorCode ec = U_ZERO_ERROR;
+    return TZEnumeration::create(UCAL_ZONE_TYPE_ANY, country, NULL, ec);
 }
 
 // ---------------------------------------
@@ -876,7 +1032,7 @@ TimeZone::getEquivalentID(const UnicodeString& id, int32_t index) {
         ures_getByKey(&res, kLINKS, &r, &ec);
         const int32_t* v = ures_getIntVector(&r, &size, &ec);
         if (U_SUCCESS(ec)) {
-            if (index >= 0 && index < size && getOlsonMeta()) {
+            if (index >= 0 && index < size) {
                 zone = v[index];
             }
         }
@@ -904,7 +1060,26 @@ TimeZone::getEquivalentID(const UnicodeString& id, int32_t index) {
 
 // ---------------------------------------
 
-// These two methods are used by ZoneMeta class only.
+// These methods are used by ZoneMeta class only.
+
+const UChar*
+TimeZone::findID(const UnicodeString& id) {
+    const UChar *result = NULL;
+    UErrorCode ec = U_ZERO_ERROR;
+    UResourceBundle *rb = ures_openDirect(NULL, kZONEINFO, &ec);
+
+    // resolve zone index by name
+    UResourceBundle *names = ures_getByKey(rb, kNAMES, NULL, &ec);
+    int32_t idx = findInStringArray(names, id, ec);
+    result = ures_getStringByIndex(names, idx, NULL, &ec);
+    if (U_FAILURE(ec)) {
+        result = NULL;
+    }
+    ures_close(names);
+    ures_close(rb);
+    return result;
+}
+
 
 const UChar*
 TimeZone::dereferOlsonLink(const UnicodeString& id) {
@@ -940,18 +1115,26 @@ TimeZone::dereferOlsonLink(const UnicodeString& id) {
 
 const UChar*
 TimeZone::getRegion(const UnicodeString& id) {
-    const UChar *result = WORLD;
-    UErrorCode ec = U_ZERO_ERROR;
-    UResourceBundle *rb = ures_openDirect(NULL, kZONEINFO, &ec);
+    UErrorCode status = U_ZERO_ERROR;
+    return getRegion(id, status);
+}
+
+const UChar*
+TimeZone::getRegion(const UnicodeString& id, UErrorCode& status) {
+    if (U_FAILURE(status)) {
+        return NULL;
+    }
+    const UChar *result = NULL;
+    UResourceBundle *rb = ures_openDirect(NULL, kZONEINFO, &status);
 
     // resolve zone index by name
-    UResourceBundle *res = ures_getByKey(rb, kNAMES, NULL, &ec);
-    int32_t idx = findInStringArray(res, id, ec);
+    UResourceBundle *res = ures_getByKey(rb, kNAMES, NULL, &status);
+    int32_t idx = findInStringArray(res, id, status);
 
     // get region mapping
-    ures_getByKey(rb, kREGIONS, res, &ec);
-    const UChar *tmp = ures_getStringByIndex(res, idx, NULL, &ec);
-    if (U_SUCCESS(ec)) {
+    ures_getByKey(rb, kREGIONS, res, &status);
+    const UChar *tmp = ures_getStringByIndex(res, idx, NULL, &status);
+    if (U_SUCCESS(status)) {
         result = tmp;
     }
 
@@ -961,6 +1144,39 @@ TimeZone::getRegion(const UnicodeString& id) {
     return result;
 }
 
+
+// ---------------------------------------
+int32_t
+TimeZone::getRegion(const UnicodeString& id, char *region, int32_t capacity, UErrorCode& status)
+{
+    int32_t resultLen = 0;
+    *region = 0;
+    if (U_FAILURE(status)) {
+        return 0;
+    }
+
+    const UChar *uregion = NULL;
+    // "Etc/Unknown" is not a system zone ID,
+    // but in the zone data
+    if (id.compare(UNKNOWN_ZONE_ID, UNKNOWN_ZONE_ID_LENGTH) != 0) {
+        uregion = getRegion(id);
+    }
+    if (uregion == NULL) {
+        status = U_ILLEGAL_ARGUMENT_ERROR;
+        return 0;
+    }
+    resultLen = u_strlen(uregion);
+    // A region code is represented by invariant characters
+    u_UCharsToChars(uregion, region, uprv_min(resultLen, capacity));
+
+    if (capacity < resultLen) {
+        status = U_BUFFER_OVERFLOW_ERROR;
+        return resultLen;
+    }
+
+    return u_terminateChars(region, capacity, resultLen, &status);
+}
+
 // ---------------------------------------
 
 
@@ -982,7 +1198,7 @@ TimeZone::getDisplayName(UBool daylight, EDisplayType style, UnicodeString& resu
     return getDisplayName(daylight,style, Locale::getDefault(), result);
 }
 //--------------------------------------
-int32_t 
+int32_t
 TimeZone::getDSTSavings()const {
     if (useDaylightTime()) {
         return 3600000;
@@ -1019,13 +1235,15 @@ TimeZone::getDisplayName(UBool daylight, EDisplayType style, const Locale& local
         pat = ZZZZ_UC_STR;
         break;
     case SHORT_COMMONLY_USED:
-        pat = V_UC_STR;
+        //pat = V_UC_STR;
+        pat = Z_STR;
         break;
     case GENERIC_LOCATION:
         pat = VVVV_UC_STR;
         break;
     default: // SHORT
-        pat = Z_STR;
+        //pat = Z_STR;
+        pat = V_UC_STR;
         break;
     }
 
@@ -1048,7 +1266,7 @@ TimeZone::getDisplayName(UBool daylight, EDisplayType style, const Locale& local
     if (U_FAILURE(status)) {
         return result.remove();
     }
-    
+
     if ((daylight && dstOffset != 0) || 
         (!daylight && dstOffset == 0) ||
         (style == SHORT_GENERIC) ||
@@ -1058,7 +1276,7 @@ TimeZone::getDisplayName(UBool daylight, EDisplayType style, const Locale& local
         format.setTimeZone(*this);
         return format.format(d, result);
     }
-    
+
     // Create a new SimpleTimeZone as a stand-in for this zone; the
     // stand-in will have no DST, or DST during July, but the same ID and offset,
     // and hence the same display name.
@@ -1075,10 +1293,10 @@ TimeZone::getDisplayName(UBool daylight, EDisplayType style, const Locale& local
         }
         cal.set(UCAL_MONTH, UCAL_JULY);
         cal.set(UCAL_DATE, 1);
-        
+
         // Get July 1 date
         d = cal.getTime(status);
-        
+
         // Check if it is in DST
         if (cal.get(UCAL_DST_OFFSET, status) == 0) {
             // We need to create a fake time zone
@@ -1178,6 +1396,7 @@ TimeZone::parseCustomID(const UnicodeString& id, int32_t& sign,
             return FALSE;
         }
         numberFormat->setParseIntegerOnly(TRUE);
+        //numberFormat->setLenient(TRUE); // TODO: May need to set this, depends on latest timezone parsing
 
         // Look for either hh:mm, hhmm, or hh
         int32_t start = pos.getIndex();
@@ -1308,10 +1527,10 @@ TimeZone::formatCustomID(int32_t hour, int32_t min, int32_t sec,
 }
 
 
-UBool 
+UBool
 TimeZone::hasSameRules(const TimeZone& other) const
 {
-    return (getRawOffset() == other.getRawOffset() && 
+    return (getRawOffset() == other.getRawOffset() &&
             useDaylightTime() == other.useDaylightTime());
 }
 
@@ -1365,13 +1584,19 @@ TimeZone::getCanonicalID(const UnicodeString& id, UnicodeString& canonicalID, UB
     if (U_FAILURE(status)) {
         return canonicalID;
     }
-    ZoneMeta::getCanonicalSystemID(id, canonicalID, status);
-    if (U_SUCCESS(status)) {
-        isSystemID = TRUE;
+    if (id.compare(UNKNOWN_ZONE_ID, UNKNOWN_ZONE_ID_LENGTH) == 0) {
+        // special case - Etc/Unknown is a canonical ID, but not system ID
+        canonicalID.fastCopyFrom(id);
+        isSystemID = FALSE;
     } else {
-        // Not a system ID
-        status = U_ZERO_ERROR;
-        getCustomID(id, canonicalID, status);
+        ZoneMeta::getCanonicalCLDRID(id, canonicalID, status);
+        if (U_SUCCESS(status)) {
+            isSystemID = TRUE;
+        } else {
+            // Not a system ID
+            status = U_ZERO_ERROR;
+            getCustomID(id, canonicalID, status);
+        }
     }
     return canonicalID;
 }
