@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-*   Copyright (C) 2004-2008, International Business Machines
+*   Copyright (C) 2004-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 *   file name:  ucol_sit.cpp
@@ -19,7 +19,6 @@
 #include "utracimp.h"
 #include "ucol_imp.h"
 #include "ucol_tok.h"
-#include "unormimp.h"
 #include "cmemory.h"
 #include "cstring.h"
 #include "uresimp.h"
@@ -114,7 +113,7 @@ static const AttributeConversion conversions[12] = {
 };
 
 
-static char 
+static char
 ucol_sit_attributeValueToLetter(UColAttributeValue value, UErrorCode *status) {
     uint32_t i = 0;
     for(i = 0; i < sizeof(conversions)/sizeof(conversions[0]); i++) {
@@ -126,7 +125,7 @@ ucol_sit_attributeValueToLetter(UColAttributeValue value, UErrorCode *status) {
     return 0;
 }
 
-static UColAttributeValue 
+static UColAttributeValue
 ucol_sit_letterToAttributeValue(char letter, UErrorCode *status) {
     uint32_t i = 0;
     for(i = 0; i < sizeof(conversions)/sizeof(conversions[0]); i++) {
@@ -147,8 +146,8 @@ U_CDECL_END
 
 U_CDECL_BEGIN
 static const char* U_CALLCONV
-_processLocaleElement(CollatorSpec *spec, uint32_t value, const char* string, 
-                      UErrorCode *status) 
+_processLocaleElement(CollatorSpec *spec, uint32_t value, const char* string,
+                      UErrorCode *status)
 {
     int32_t len = 0;
     do {
@@ -169,8 +168,8 @@ U_CDECL_END
 
 U_CDECL_BEGIN
 static const char* U_CALLCONV
-_processRFC3066Locale(CollatorSpec *spec, uint32_t, const char* string, 
-                      UErrorCode *status) 
+_processRFC3066Locale(CollatorSpec *spec, uint32_t, const char* string,
+                      UErrorCode *status)
 {
     char terminator = *string;
     string++;
@@ -188,8 +187,8 @@ U_CDECL_END
 
 U_CDECL_BEGIN
 static const char* U_CALLCONV
-_processCollatorOption(CollatorSpec *spec, uint32_t option, const char* string, 
-                       UErrorCode *status) 
+_processCollatorOption(CollatorSpec *spec, uint32_t option, const char* string,
+                       UErrorCode *status)
 {
     spec->options[option] = ucol_sit_letterToAttributeValue(*string, status);
     if((*(++string) != '_' && *string) || U_FAILURE(*status)) {
@@ -200,8 +199,8 @@ _processCollatorOption(CollatorSpec *spec, uint32_t option, const char* string,
 U_CDECL_END
 
 
-static UChar 
-readHexCodeUnit(const char **string, UErrorCode *status) 
+static UChar
+readHexCodeUnit(const char **string, UErrorCode *status)
 {
     UChar result = 0;
     int32_t value = 0;
@@ -231,7 +230,7 @@ readHexCodeUnit(const char **string, UErrorCode *status)
 
 U_CDECL_BEGIN
 static const char* U_CALLCONV
-_processVariableTop(CollatorSpec *spec, uint32_t value1, const char* string, UErrorCode *status) 
+_processVariableTop(CollatorSpec *spec, uint32_t value1, const char* string, UErrorCode *status)
 {
     // get four digits
     int32_t i = 0;
@@ -240,7 +239,7 @@ _processVariableTop(CollatorSpec *spec, uint32_t value1, const char* string, UEr
             spec->variableTopString[i++] = readHexCodeUnit(&string, status);
         }
         spec->variableTopStringLen = i;
-        if(i == locElementCapacity && (*string != 0 || *string != '_')) {
+        if(i == locElementCapacity && *string != 0 && *string != '_') {
             *status = U_BUFFER_OVERFLOW_ERROR;
         }
     } else {
@@ -248,7 +247,7 @@ _processVariableTop(CollatorSpec *spec, uint32_t value1, const char* string, UEr
     }
     if(U_SUCCESS(*status)) {
         spec->variableTopSet = TRUE;
-    } 
+    }
     return string;
 }
 U_CDECL_END
@@ -263,7 +262,7 @@ struct ShortStringOptions {
 
 static const ShortStringOptions options[UCOL_SIT_ITEMS_COUNT] =
 {
-/* 10 ALTERNATE_HANDLING */   {alternateHArg,     _processCollatorOption, UCOL_ALTERNATE_HANDLING }, // alternate  N, S, D 
+/* 10 ALTERNATE_HANDLING */   {alternateHArg,     _processCollatorOption, UCOL_ALTERNATE_HANDLING }, // alternate  N, S, D
 /* 15 VARIABLE_TOP_VALUE */   {variableTopValArg, _processVariableTop,    1 },
 /* 08 CASE_FIRST */           {caseFirstArg,      _processCollatorOption, UCOL_CASE_FIRST }, // case first L, U, X, D
 /* 09 NUMERIC_COLLATION */    {numericCollArg,    _processCollatorOption, UCOL_NUMERIC_COLLATION }, // codan      O, X, D
@@ -283,8 +282,8 @@ static const ShortStringOptions options[UCOL_SIT_ITEMS_COUNT] =
 
 
 static
-const char* ucol_sit_readOption(const char *start, CollatorSpec *spec, 
-                            UErrorCode *status) 
+const char* ucol_sit_readOption(const char *start, CollatorSpec *spec,
+                            UErrorCode *status)
 {
   int32_t i = 0;
 
@@ -292,7 +291,7 @@ const char* ucol_sit_readOption(const char *start, CollatorSpec *spec,
       if(*start == options[i].optionStart) {
           spec->entries[i].start = start;
           const char* end = options[i].action(spec, options[i].attr, start+1, status);
-          spec->entries[i].len = end - start;
+          spec->entries[i].len = (int32_t)(end - start);
           return end;
       }
   }
@@ -301,7 +300,7 @@ const char* ucol_sit_readOption(const char *start, CollatorSpec *spec,
 }
 
 static
-void ucol_sit_initCollatorSpecs(CollatorSpec *spec) 
+void ucol_sit_initCollatorSpecs(CollatorSpec *spec)
 {
     // reset everything
     uprv_memset(spec, 0, sizeof(CollatorSpec));
@@ -312,12 +311,12 @@ void ucol_sit_initCollatorSpecs(CollatorSpec *spec)
     }
 }
 
-static const char* 
-ucol_sit_readSpecs(CollatorSpec *s, const char *string, 
+static const char*
+ucol_sit_readSpecs(CollatorSpec *s, const char *string,
                         UParseError *parseError, UErrorCode *status)
 {
     const char *definition = string;
-    while(U_SUCCESS(*status) && *string) { 
+    while(U_SUCCESS(*status) && *string) {
         string = ucol_sit_readOption(string, s, status);
         // advance over '_'
         while(*string && *string == '_') {
@@ -325,7 +324,7 @@ ucol_sit_readSpecs(CollatorSpec *s, const char *string,
         }
     }
     if(U_FAILURE(*status)) {
-        parseError->offset = string - definition;
+        parseError->offset = (int32_t)(string - definition);
     }
     return string;
 }
@@ -335,7 +334,7 @@ int32_t ucol_sit_dumpSpecs(CollatorSpec *s, char *destination, int32_t capacity,
 {
     int32_t i = 0, j = 0;
     int32_t len = 0;
-    char optName; 
+    char optName;
     if(U_SUCCESS(*status)) {
         for(i = 0; i < UCOL_SIT_ITEMS_COUNT; i++) {
             if(s->entries[i].start) {
@@ -344,7 +343,7 @@ int32_t ucol_sit_dumpSpecs(CollatorSpec *s, char *destination, int32_t capacity,
                         uprv_strcat(destination, "_");
                     }
                     len++;
-                } 
+                }
                 optName = *(s->entries[i].start);
                 if(optName == languageArg || optName == regionArg || optName == variantArg || optName == keywordArg) {
                     for(j = 0; j < s->entries[i].len; j++) {
@@ -429,7 +428,7 @@ ucol_prepareShortStringOpen( const char *definition,
     ucol_sit_initCollatorSpecs(&s);
     ucol_sit_readSpecs(&s, definition, parseError, status);
     ucol_sit_calculateWholeLocale(&s);
-    
+
     char buffer[internalBufferSize];
     uprv_memset(buffer, 0, internalBufferSize);
     uloc_canonicalize(s.locale, buffer, internalBufferSize, status);
@@ -493,7 +492,7 @@ ucol_openFromShortString( const char *definition,
     ucol_sit_initCollatorSpecs(&s);
     string = ucol_sit_readSpecs(&s, definition, parseError, status);
     ucol_sit_calculateWholeLocale(&s);
-    
+
     char buffer[internalBufferSize];
     uprv_memset(buffer, 0, internalBufferSize);
     uloc_canonicalize(s.locale, buffer, internalBufferSize, status);
@@ -508,7 +507,7 @@ ucol_openFromShortString( const char *definition,
             }
 
             if(U_FAILURE(*status)) {
-                parseError->offset = string - definition;
+                parseError->offset = (int32_t)(string - definition);
                 ucol_close(result);
                 return NULL;
             }
@@ -571,7 +570,7 @@ ucol_getShortDefinitionString(const UCollator *coll,
     ucol_sit_initCollatorSpecs(&s);
 
     if(!locale) {
-        locale = ucol_getLocale(coll, ULOC_VALID_LOCALE, status);
+        locale = ucol_getLocaleByType(coll, ULOC_VALID_LOCALE, status);
     }
     elementSize = ucol_getFunctionalEquivalent(locBuff, internalBufferSize, "collation", locale, &isAvailable, status);
 
@@ -587,7 +586,7 @@ ucol_getShortDefinitionString(const UCollator *coll,
         appendShortStringElement(tempbuff, elementSize, buffer, &resultSize, /*capacity*/internalBufferSize, variantArg);
         elementSize = uloc_getKeywordValue(locBuff, "collation", tempbuff, internalBufferSize, status);
         appendShortStringElement(tempbuff, elementSize, buffer, &resultSize, /*capacity*/internalBufferSize, keywordArg);
-    } 
+    }
 
     int32_t i = 0;
     UColAttributeValue attribute = UCOL_DEFAULT;
@@ -596,7 +595,7 @@ ucol_getShortDefinitionString(const UCollator *coll,
             attribute = ucol_getAttributeOrDefault(coll, (UColAttribute)options[i].attr, status);
             if(attribute != UCOL_DEFAULT) {
                 char letter = ucol_sit_attributeValueToLetter(attribute, status);
-                appendShortStringElement(&letter, 1, 
+                appendShortStringElement(&letter, 1,
                     buffer, &resultSize, /*capacity*/internalBufferSize, options[i].optionStart);
             }
         }
@@ -640,7 +639,7 @@ ucol_normalizeShortDefinitionString(const char *definition,
 }
 
 U_CAPI UColAttributeValue  U_EXPORT2
-ucol_getAttributeOrDefault(const UCollator *coll, UColAttribute attr, UErrorCode *status) 
+ucol_getAttributeOrDefault(const UCollator *coll, UColAttribute attr, UErrorCode *status)
 {
     if(U_FAILURE(*status) || coll == NULL) {
       return UCOL_DEFAULT;
@@ -683,8 +682,8 @@ struct contContext {
 
 
 static void
-addSpecial(contContext *context, UChar *buffer, int32_t bufLen, 
-               uint32_t CE, int32_t leftIndex, int32_t rightIndex, UErrorCode *status) 
+addSpecial(contContext *context, UChar *buffer, int32_t bufLen,
+               uint32_t CE, int32_t leftIndex, int32_t rightIndex, UErrorCode *status)
 {
   const UCollator *coll = context->coll;
   USet *contractions = context->conts;
@@ -704,7 +703,7 @@ addSpecial(contContext *context, UChar *buffer, int32_t bufLen,
               uset_addString(expansions, buffer+leftIndex, rightIndex-leftIndex);
             }
       }
-    }    
+    }
 
     UCharOffset++;
     // check whether we're doing contraction or prefix
@@ -755,7 +754,7 @@ addSpecial(contContext *context, UChar *buffer, int32_t bufLen,
 
 U_CDECL_BEGIN
 static UBool U_CALLCONV
-_processSpecials(const void *context, UChar32 start, UChar32 limit, uint32_t CE) 
+_processSpecials(const void *context, UChar32 start, UChar32 limit, uint32_t CE)
 {
     UErrorCode *status = ((contContext *)context)->status;
     USet *expansions = ((contContext *)context)->expansions;
@@ -765,7 +764,7 @@ _processSpecials(const void *context, UChar32 start, UChar32 limit, uint32_t CE)
     if(isSpecial(CE)) {
       if(((getCETag(CE) == SPEC_PROC_TAG && addPrefixes) || getCETag(CE) == CONTRACTION_TAG)) {
         while(start < limit && U_SUCCESS(*status)) {
-            // if there are suppressed contractions, we don't 
+            // if there are suppressed contractions, we don't
             // want to add them.
             if(removed && uset_contains(removed, start)) {
                 start++;
@@ -801,8 +800,6 @@ U_CDECL_END
  * @param conts the set to hold the result
  * @param status to hold the error code
  * @return the size of the contraction set
- *
- * @draft ICU 3.0
  */
 U_CAPI int32_t U_EXPORT2
 ucol_getContractions( const UCollator *coll,
@@ -847,14 +844,15 @@ ucol_getContractionsAndExpansions( const UCollator *coll,
     int32_t rulesLen = 0;
     const UChar* rules = ucol_getRules(coll, &rulesLen);
     UColTokenParser src;
-    ucol_tok_initTokenList(&src, rules, rulesLen, coll->UCA, status);
+    ucol_tok_initTokenList(&src, rules, rulesLen, coll->UCA,
+                           ucol_tok_getRulesFromBundle, NULL, status);
 
     contContext c = { NULL, contractions, expansions, src.removeSet, addPrefixes, status };
 
     // Add the UCA contractions
     c.coll = coll->UCA;
     utrie_enum(&coll->UCA->mapping, NULL, _processSpecials, &c);
-    
+
     // This is collator specific. Add contractions from a collator
     c.coll = coll;
     c.removedContractions =  NULL;
@@ -891,7 +889,7 @@ ucol_getUnsafeSet( const UCollator *coll,
     int32_t contsSize = ucol_getContractions(coll, contractions, status);
     UChar32 c = 0;
     // Contraction set consists only of strings
-    // to get unsafe code points, we need to 
+    // to get unsafe code points, we need to
     // break the strings apart and add them to the unsafe set
     for(i = 0; i < contsSize; i++) {
         len = uset_getItem(contractions, i, NULL, NULL, buffer, internalBufferSize, status);
