@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2001, International Business Machines
+*   Copyright (C) 1999-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -19,12 +19,39 @@
 * 
 */
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/umsg.h"
 #include "unicode/ustring.h"
 #include "unicode/fmtable.h"
-#include "cpputils.h"
 #include "unicode/msgfmt.h"
 #include "unicode/unistr.h"
+#include "cpputils.h"
+#include "uassert.h"
+#include "ustr_imp.h"
+
+U_NAMESPACE_BEGIN
+/**
+ * This class isolates our access to private internal methods of
+ * MessageFormat.  It is never instantiated; it exists only for C++
+ * access management.
+ */
+class MessageFormatAdapter {
+public:
+    static const Formattable::Type* getArgTypeList(const MessageFormat& m,
+                                                   int32_t& count);
+    static UBool hasArgTypeConflicts(const MessageFormat& m) {
+        return m.hasArgTypeConflicts;
+    }
+};
+const Formattable::Type*
+MessageFormatAdapter::getArgTypeList(const MessageFormat& m,
+                                     int32_t& count) {
+    return m.getArgTypeList(count);
+}
+U_NAMESPACE_END
 
 U_NAMESPACE_USE
 
@@ -38,15 +65,12 @@ u_formatMessage(const char  *locale,
                 ...)
 {
     va_list    ap;
-    int32_t actLen;
-    
+    int32_t actLen;        
     //argument checking defered to subsequent method calls
-
     // start vararg processing
     va_start(ap, status);
 
     actLen = u_vformatMessage(locale,pattern,patternLength,result,resultLength,ap,status);
-
     // end vararg processing
     va_end(ap);
 
@@ -64,7 +88,6 @@ u_vformatMessage(   const char  *locale,
 
 {
     //argument checking defered to subsequent method calls
-
     UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,NULL,status);
     int32_t retVal = umsg_vformat(fmt,result,resultLength,ap,status);
     umsg_close(fmt);
@@ -84,7 +107,6 @@ u_formatMessageWithError(const char *locale,
     va_list    ap;
     int32_t actLen;
     //argument checking defered to subsequent method calls
-
     // start vararg processing
     va_start(ap, status);
 
@@ -92,7 +114,6 @@ u_formatMessageWithError(const char *locale,
 
     // end vararg processing
     va_end(ap);
-
     return actLen;
 }
 
@@ -108,7 +129,6 @@ u_vformatMessageWithError(  const char  *locale,
 
 {
     //argument checking defered to subsequent method calls
-
     UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,parseError,status);
     int32_t retVal = umsg_vformat(fmt,result,resultLength,ap,status);
     umsg_close(fmt);
@@ -130,14 +150,12 @@ u_parseMessage( const char   *locale,
                 ...)
 {
     va_list    ap;
-
     //argument checking defered to subsequent method calls
 
     // start vararg processing
     va_start(ap, status);
 
     u_vparseMessage(locale,pattern,patternLength,source,sourceLength,ap,status);
-
     // end vararg processing
     va_end(ap);
 }
@@ -152,7 +170,6 @@ u_vparseMessage(const char  *locale,
                 UErrorCode  *status)
 {
     //argument checking defered to subsequent method calls
-
     UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,NULL,status);
     int32_t count = 0;
     umsg_vparse(fmt,source,sourceLength,&count,ap,status);
@@ -177,7 +194,6 @@ u_parseMessageWithError(const char  *locale,
     va_start(ap, status);
 
     u_vparseMessageWithError(locale,pattern,patternLength,source,sourceLength,ap,error,status);
-
     // end vararg processing
     va_end(ap);
 }
@@ -192,7 +208,6 @@ u_vparseMessageWithError(const char  *locale,
                          UErrorCode* status)
 {
     //argument checking defered to subsequent method calls
-
     UMessageFormat *fmt = umsg_open(pattern,patternLength,locale,error,status);
     int32_t count = 0;
     umsg_vparse(fmt,source,sourceLength,&count,ap,status);
@@ -223,25 +238,23 @@ umsg_open(  const UChar     *pattern,
     }
 
     UParseError tErr;
-   
     if(parseError==NULL)
     {
         parseError = &tErr;
     }
-        
-    UMessageFormat* retVal = 0;
 
     int32_t len = (patternLength == -1 ? u_strlen(pattern) : patternLength);
-    
-    UnicodeString patString((patternLength == -1 ? TRUE:FALSE), pattern,len);
+    UnicodeString patString(patternLength == -1, pattern, len);
 
-    retVal = (UMessageFormat*) new MessageFormat(pattern,Locale(locale),*parseError,*status);
-    
-    if(retVal == 0) {
+    MessageFormat* retVal = new MessageFormat(patString,Locale(locale),*parseError,*status);
+    if(retVal == NULL) {
         *status = U_MEMORY_ALLOCATION_ERROR;
-        return 0;
+        return NULL;
     }
-    return retVal;
+    if (U_SUCCESS(*status) && MessageFormatAdapter::hasArgTypeConflicts(*retVal)) {
+        *status = U_ARGUMENT_TYPE_MISMATCH;
+    }
+    return (UMessageFormat*)retVal;
 }
 
 U_CAPI void U_EXPORT2
@@ -285,13 +298,13 @@ umsg_setLocale(UMessageFormat *fmt, const char* locale)
 }
 
 U_CAPI const char*  U_EXPORT2
-umsg_getLocale(UMessageFormat *fmt)
+umsg_getLocale(const UMessageFormat *fmt)
 {
     //check arguments
     if(fmt==NULL){
         return "";
     }
-    return ((MessageFormat*)fmt)->getLocale().getName();
+    return ((const MessageFormat*)fmt)->getLocale().getName();
 }
 
 U_CAPI void  U_EXPORT2
@@ -322,7 +335,7 @@ umsg_applyPattern(UMessageFormat *fmt,
 }
 
 U_CAPI int32_t  U_EXPORT2
-umsg_toPattern(UMessageFormat *fmt,
+umsg_toPattern(const UMessageFormat *fmt,
                UChar* result, 
                int32_t resultLength,
                UErrorCode* status)
@@ -337,21 +350,25 @@ umsg_toPattern(UMessageFormat *fmt,
     }
 
 
-    UnicodeString res(result, 0, resultLength);
-    ((MessageFormat*)fmt)->toPattern(res);
+    UnicodeString res;
+    if(!(result==NULL && resultLength==0)) {
+        // NULL destination for pure preflighting: empty dummy string
+        // otherwise, alias the destination buffer
+        res.setTo(result, 0, resultLength);
+    }
+    ((const MessageFormat*)fmt)->toPattern(res);
     return res.extract(result, resultLength, *status);
 }
 
 U_CAPI int32_t
-umsg_format(    UMessageFormat *fmt,
+umsg_format(    const UMessageFormat *fmt,
                 UChar          *result,
                 int32_t        resultLength,
                 UErrorCode     *status,
                 ...)
 {
     va_list    ap;
-    int32_t actLen;
-    
+    int32_t actLen;  
     //argument checking defered to last method call umsg_vformat which
     //saves time when arguments are valid and we dont care when arguments are not
     //since we return an error anyway
@@ -369,7 +386,7 @@ umsg_format(    UMessageFormat *fmt,
 }
 
 U_CAPI int32_t U_EXPORT2
-umsg_vformat(   UMessageFormat *fmt,
+umsg_vformat(   const UMessageFormat *fmt,
                 UChar          *result,
                 int32_t        resultLength,
                 va_list        ap,
@@ -386,8 +403,11 @@ umsg_vformat(   UMessageFormat *fmt,
     }
 
     int32_t count =0;
-    const Formattable::Type* argTypes = ((MessageFormat*)fmt)->getFormatTypeList(count);
-    Formattable args[MessageFormat::kMaxFormat];
+    const Formattable::Type* argTypes =
+        MessageFormatAdapter::getArgTypeList(*(const MessageFormat*)fmt, count);
+    // Allocate at least one element.  Allocating an array of length
+    // zero causes problems on some platforms (e.g. Win32).
+    Formattable* args = new Formattable[count ? count : 1];
 
     // iterate through the vararg list, and get the arguments out
     for(int32_t i = 0; i < count; ++i) {
@@ -395,6 +415,7 @@ umsg_vformat(   UMessageFormat *fmt,
         UChar *stringVal;
         double tDouble=0;
         int32_t tInt =0;
+        int64_t tInt64 = 0;
         UDate tempDate = 0;
         switch(argTypes[i]) {
         case Formattable::kDate:
@@ -411,6 +432,11 @@ umsg_vformat(   UMessageFormat *fmt,
             tInt = va_arg(ap, int32_t);
             args[i].setLong(tInt);
             break;
+
+        case Formattable::kInt64:
+            tInt64 = va_arg(ap, int64_t);
+            args[i].setInt64(tInt64);
+            break;
             
         case Formattable::kString:
             // For some reason, a temporary is needed
@@ -430,13 +456,24 @@ umsg_vformat(   UMessageFormat *fmt,
             va_arg(ap, int);
             break;
 
+        case Formattable::kObject:
+        default:
+            // This will never happen because MessageFormat doesn't
+            // support kObject.  When MessageFormat is changed to
+            // understand MeasureFormats, modify this code to do the
+            // right thing. [alan]
+            U_ASSERT(FALSE);
+            *status=U_ILLEGAL_ARGUMENT_ERROR;
+            break;
         }
     }
     UnicodeString resultStr;
     FieldPosition fieldPosition(0);
     
     /* format the message */
-    ((MessageFormat*)fmt)->format(args,count,resultStr,fieldPosition,*status);
+    ((const MessageFormat*)fmt)->format(args,count,resultStr,fieldPosition,*status);
+
+    delete[] args;
 
     if(U_FAILURE(*status)){
         return -1;
@@ -446,7 +483,7 @@ umsg_vformat(   UMessageFormat *fmt,
 }
 
 U_CAPI void
-umsg_parse( UMessageFormat *fmt,
+umsg_parse( const UMessageFormat *fmt,
             const UChar    *source,
             int32_t        sourceLength,
             int32_t        *count,
@@ -454,7 +491,6 @@ umsg_parse( UMessageFormat *fmt,
             ...)
 {
     va_list    ap;
-
     //argument checking defered to last method call umsg_vparse which
     //saves time when arguments are valid and we dont care when arguments are not
     //since we return an error anyway
@@ -469,7 +505,7 @@ umsg_parse( UMessageFormat *fmt,
 }
 
 U_CAPI void U_EXPORT2
-umsg_vparse(UMessageFormat *fmt,
+umsg_vparse(const UMessageFormat *fmt,
             const UChar    *source,
             int32_t        sourceLength,
             int32_t        *count,
@@ -490,11 +526,12 @@ umsg_vparse(UMessageFormat *fmt,
     }
 
     UnicodeString srcString(source,sourceLength);
-    Formattable *args = ((MessageFormat*)fmt)->parse(source,*count,*status);
+    Formattable *args = ((const MessageFormat*)fmt)->parse(source,*count,*status);
     UDate *aDate;
     double *aDouble;
     UChar *aString;
     int32_t* aInt;
+    int64_t* aInt64;
     UnicodeString temp;
     int len =0;
     // assign formattables to varargs
@@ -520,7 +557,6 @@ umsg_vparse(UMessageFormat *fmt,
             break;
 
         case Formattable::kLong:
-            
             aInt = va_arg(ap, int32_t*);
             if(aInt){
                 *aInt = (int32_t) args[i].getLong();
@@ -529,6 +565,15 @@ umsg_vparse(UMessageFormat *fmt,
             }
             break;
 
+        case Formattable::kInt64:
+            aInt64 = va_arg(ap, int64_t*);
+            if(aInt64){
+                *aInt64 = args[i].getInt64();
+            }else{
+                *status=U_ILLEGAL_ARGUMENT_ERROR;
+            }
+            break;
+
         case Formattable::kString:
             aString = va_arg(ap, UChar*);
             if(aString){
@@ -541,9 +586,17 @@ umsg_vparse(UMessageFormat *fmt,
             }
             break;
 
+        case Formattable::kObject:
+            // This will never happen because MessageFormat doesn't
+            // support kObject.  When MessageFormat is changed to
+            // understand MeasureFormats, modify this code to do the
+            // right thing. [alan]
+            U_ASSERT(FALSE);
+            break;
+
         // better not happen!
         case Formattable::kArray:
-            // DIE
+            U_ASSERT(FALSE);
             break;
         }
     }
@@ -552,4 +605,105 @@ umsg_vparse(UMessageFormat *fmt,
     delete [] args;
 }
 
+#define SINGLE_QUOTE      ((UChar)0x0027)
+#define CURLY_BRACE_LEFT  ((UChar)0x007B)
+#define CURLY_BRACE_RIGHT ((UChar)0x007D)
 
+#define STATE_INITIAL 0
+#define STATE_SINGLE_QUOTE 1
+#define STATE_IN_QUOTE 2
+#define STATE_MSG_ELEMENT 3
+
+#define MAppend(c) if (len < destCapacity) dest[len++] = c; else len++
+
+int32_t umsg_autoQuoteApostrophe(const UChar* pattern, 
+                 int32_t patternLength,
+                 UChar* dest,
+                 int32_t destCapacity,
+                 UErrorCode* ec)
+{
+    int32_t state = STATE_INITIAL;
+    int32_t braceCount = 0;
+    int32_t len = 0;
+
+    if (ec == NULL || U_FAILURE(*ec)) {
+        return -1;
+    }
+
+    if (pattern == NULL || patternLength < -1 || (dest == NULL && destCapacity > 0)) {
+        *ec = U_ILLEGAL_ARGUMENT_ERROR;
+        return -1;
+    }
+
+    if (patternLength == -1) {
+        patternLength = u_strlen(pattern);
+    }
+
+    for (int i = 0; i < patternLength; ++i) {
+        UChar c = pattern[i];
+        switch (state) {
+        case STATE_INITIAL:
+            switch (c) {
+            case SINGLE_QUOTE:
+                state = STATE_SINGLE_QUOTE;
+                break;
+            case CURLY_BRACE_LEFT:
+                state = STATE_MSG_ELEMENT;
+                ++braceCount;
+                break;
+            }
+            break;
+
+        case STATE_SINGLE_QUOTE:
+            switch (c) {
+            case SINGLE_QUOTE:
+                state = STATE_INITIAL;
+                break;
+            case CURLY_BRACE_LEFT:
+            case CURLY_BRACE_RIGHT:
+                state = STATE_IN_QUOTE;
+                break;
+            default:
+                MAppend(SINGLE_QUOTE);
+                state = STATE_INITIAL;
+                break;
+            }
+        break;
+
+        case STATE_IN_QUOTE:
+            switch (c) {
+            case SINGLE_QUOTE:
+                state = STATE_INITIAL;
+                break;
+            }
+            break;
+
+        case STATE_MSG_ELEMENT:
+            switch (c) {
+            case CURLY_BRACE_LEFT:
+                ++braceCount;
+                break;
+            case CURLY_BRACE_RIGHT:
+                if (--braceCount == 0) {
+                    state = STATE_INITIAL;
+                }
+                break;
+            }
+            break;
+
+        default: // Never happens.
+            break;
+        }
+
+        MAppend(c);
+    }
+
+    // End of scan
+    if (state == STATE_SINGLE_QUOTE || state == STATE_IN_QUOTE) {
+        MAppend(SINGLE_QUOTE);
+    }
+
+    return u_terminateUChars(dest, destCapacity, len, ec);
+}
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
