@@ -1,10 +1,12 @@
 /*
 *******************************************************************************
-* Copyright (C) 2007, International Business Machines Corporation and         *
-* others. All Rights Reserved.                                                *
+* Copyright (C) 2007-2010, International Business Machines Corporation and
+* others. All Rights Reserved.
 *******************************************************************************
 */
 
+#include <typeinfo>  // for 'typeid' to work
+
 #include "unicode/utypes.h"
 
 #if !UCONFIG_NO_FORMATTING
@@ -20,6 +22,14 @@
 
 U_NAMESPACE_BEGIN
 
+// This is the deleter that will be use to remove TimeZoneRule
+U_CDECL_BEGIN
+static void U_CALLCONV
+deleteTimeZoneRule(void* obj) {
+    delete (TimeZoneRule*) obj;
+}
+U_CDECL_END
+
 // Smybol characters used by RFC2445 VTIMEZONE
 static const UChar COLON = 0x3A; /* : */
 static const UChar SEMICOLON = 0x3B; /* ; */
@@ -346,12 +356,12 @@ static void millisToOffset(int32_t millis, UnicodeString& str) {
 /*
  * Create a default TZNAME from TZID
  */
-static void getDefaultTZName(const UnicodeString tzid, UBool isDST, UnicodeString& tzname) {
-    tzname = tzid;
+static void getDefaultTZName(const UnicodeString tzid, UBool isDST, UnicodeString& zonename) {
+    zonename = tzid;
     if (isDST) {
-        tzname += UNICODE_STRING_SIMPLE("(DST)");
+        zonename += UNICODE_STRING_SIMPLE("(DST)");
     } else {
-        tzname += UNICODE_STRING_SIMPLE("(STD)");
+        zonename += UNICODE_STRING_SIMPLE("(STD)");
     }
 }
 
@@ -511,7 +521,7 @@ rruleParseError:
     }
 }
 
-static TimeZoneRule* createRuleByRRULE(const UnicodeString& tzname, int rawOffset, int dstSavings, UDate start,
+static TimeZoneRule* createRuleByRRULE(const UnicodeString& zonename, int rawOffset, int dstSavings, UDate start,
                                        UVector* dates, int fromOffset, UErrorCode& status) {
     if (U_FAILURE(status)) {
         return NULL;
@@ -704,7 +714,7 @@ static TimeZoneRule* createRuleByRRULE(const UnicodeString& tzname, int rawOffse
     if (adtr == NULL) {
         goto unsupportedRRule;
     }
-    return new AnnualTimeZoneRule(tzname, rawOffset, dstSavings, adtr, startYear, endYear);
+    return new AnnualTimeZoneRule(zonename, rawOffset, dstSavings, adtr, startYear, endYear);
 
 unsupportedRRule:
     status = U_INVALID_STATE_ERROR;
@@ -714,7 +724,7 @@ unsupportedRRule:
 /*
  * Create a TimeZoneRule by the RDATE definition
  */
-static TimeZoneRule* createRuleByRDATE(const UnicodeString& tzname, int32_t rawOffset, int32_t dstSavings,
+static TimeZoneRule* createRuleByRDATE(const UnicodeString& zonename, int32_t rawOffset, int32_t dstSavings,
                                        UDate start, UVector* dates, int32_t fromOffset, UErrorCode& status) {
     if (U_FAILURE(status)) {
         return NULL;
@@ -723,7 +733,7 @@ static TimeZoneRule* createRuleByRDATE(const UnicodeString& tzname, int32_t rawO
     if (dates == NULL || dates->size() == 0) {
         // When no RDATE line is provided, use start (DTSTART)
         // as the transition time
-        retVal = new TimeArrayTimeZoneRule(tzname, rawOffset, dstSavings,
+        retVal = new TimeArrayTimeZoneRule(zonename, rawOffset, dstSavings,
             &start, 1, DateTimeRule::UTC_TIME);
     } else {
         // Create an array of transition times
@@ -741,7 +751,7 @@ static TimeZoneRule* createRuleByRDATE(const UnicodeString& tzname, int32_t rawO
                 return NULL;
             }
         }
-        retVal = new TimeArrayTimeZoneRule(tzname, rawOffset, dstSavings,
+        retVal = new TimeArrayTimeZoneRule(zonename, rawOffset, dstSavings,
             times, size, DateTimeRule::UTC_TIME);
         uprv_free(times);
     }
@@ -1025,8 +1035,7 @@ VTimeZone::operator==(const TimeZone& that) const {
     if (this == &that) {
         return TRUE;
     }
-    if (getDynamicClassID() != that.getDynamicClassID()
-        || !BasicTimeZone::operator==(that)) {
+    if (typeid(*this) != typeid(that) || !BasicTimeZone::operator==(that)) {
         return FALSE;
     }
     VTimeZone *vtz = (VTimeZone*)&that;
@@ -1055,8 +1064,39 @@ VTimeZone::createVTimeZoneByID(const UnicodeString& ID) {
     UErrorCode status = U_ZERO_ERROR;
     UResourceBundle *bundle = NULL;
     const UChar* versionStr = NULL;
-    int32_t len;
-    bundle = ures_openDirect(NULL, "zoneinfo", &status);
+    int32_t len = 0;
+    bundle = ures_openDirect(NULL, "zoneinfo64", &status);
+    versionStr = ures_getStringByKey(bundle, "TZVersion", &len, &status);
+    if (U_SUCCESS(status)) {
+        vtz->icutzver.setTo(versionStr, len);
+    }
+    ures_close(bundle);
+    return vtz;
+}
+
+VTimeZone*
+VTimeZone::createVTimeZoneFromBasicTimeZone(const BasicTimeZone& basic_time_zone, UErrorCode &status) {
+    if (U_FAILURE(status)) {
+        return NULL;
+    }
+    VTimeZone *vtz = new VTimeZone();
+    if (vtz == NULL) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
+    vtz->tz = (BasicTimeZone *)basic_time_zone.clone();
+    if (vtz->tz == NULL) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        delete vtz;
+        return NULL;
+    }
+    vtz->tz->getID(vtz->olsonzid);
+
+    // Set ICU tzdata version
+    UResourceBundle *bundle = NULL;
+    const UChar* versionStr = NULL;
+    int32_t len = 0;
+    bundle = ures_openDirect(NULL, "zoneinfo64", &status);
     versionStr = ures_getStringByKey(bundle, "TZVersion", &len, &status);
     if (U_SUCCESS(status)) {
         vtz->icutzver.setTo(versionStr, len);
@@ -1316,7 +1356,7 @@ VTimeZone::parse(UErrorCode& status) {
     UBool dst = FALSE;      // current zone type
     UnicodeString from;     // current zone from offset
     UnicodeString to;       // current zone offset
-    UnicodeString tzname;   // current zone name
+    UnicodeString zonename;   // current zone name
     UnicodeString dtstart;  // current zone starts
     UBool isRRULE = FALSE;  // true if the rule is described by RRULE
     int32_t initialRawOffset = 0;   // initial offset
@@ -1328,14 +1368,24 @@ VTimeZone::parse(UErrorCode& status) {
     UVector *dates = NULL;  // list of RDATE or RRULE strings
     UVector *rules = NULL;  // list of TimeZoneRule instances
 
+    int32_t finalRuleIdx = -1;
+    int32_t finalRuleCount = 0;
+
     rules = new UVector(status);
     if (U_FAILURE(status)) {
         goto cleanupParse;
     }
+     // Set the deleter to remove TimeZoneRule vectors to avoid memory leaks due to unowned TimeZoneRules.
+    rules->setDeleter(deleteTimeZoneRule);
+    
     dates = new UVector(uhash_deleteUnicodeString, uhash_compareUnicodeString, status);
     if (U_FAILURE(status)) {
         goto cleanupParse;
     }
+    if (rules == NULL || dates == NULL) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        goto cleanupParse;
+    }
 
     for (n = 0; n < vtzlines->size(); n++) {
         UnicodeString *line = (UnicodeString*)vtzlines->elementAt(n);
@@ -1380,7 +1430,7 @@ VTimeZone::parse(UErrorCode& status) {
                     isRRULE = FALSE;
                     from.remove();
                     to.remove();
-                    tzname.remove();
+                    zonename.remove();
                     dst = isDST;
                     state = TZI;
                 } else {
@@ -1396,7 +1446,7 @@ VTimeZone::parse(UErrorCode& status) {
             if (name.compare(ICAL_DTSTART) == 0) {
                 dtstart = value;
             } else if (name.compare(ICAL_TZNAME) == 0) {
-                tzname = value;
+                zonename = value;
             } else if (name.compare(ICAL_TZOFFSETFROM) == 0) {
                 from = value;
             } else if (name.compare(ICAL_TZOFFSETTO) == 0) {
@@ -1440,9 +1490,9 @@ VTimeZone::parse(UErrorCode& status) {
                 if (dtstart.length() == 0 || from.length() == 0 || to.length() == 0) {
                     goto cleanupParse;
                 }
-                // if tzname is not available, create one from tzid
-                if (tzname.length() == 0) {
-                    getDefaultTZName(tzid, dst, tzname);
+                // if zonename is not available, create one from tzid
+                if (zonename.length() == 0) {
+                    getDefaultTZName(tzid, dst, zonename);
                 }
 
                 // create a time zone rule
@@ -1484,9 +1534,9 @@ VTimeZone::parse(UErrorCode& status) {
                 // Create the rule
                 UDate actualStart = MAX_MILLIS;
                 if (isRRULE) {
-                    rule = createRuleByRRULE(tzname, rawOffset, dstSavings, start, dates, fromOffset, status);
+                    rule = createRuleByRRULE(zonename, rawOffset, dstSavings, start, dates, fromOffset, status);
                 } else {
-                    rule = createRuleByRDATE(tzname, rawOffset, dstSavings, start, dates, fromOffset, status);
+                    rule = createRuleByRDATE(zonename, rawOffset, dstSavings, start, dates, fromOffset, status);
                 }
                 if (U_FAILURE(status) || rule == NULL) {
                     goto cleanupParse;
@@ -1527,12 +1577,106 @@ VTimeZone::parse(UErrorCode& status) {
     }
 
     // Create a initial rule
-    getDefaultTZName(tzid, FALSE, tzname);
-    initialRule = new InitialTimeZoneRule(tzname,
+    getDefaultTZName(tzid, FALSE, zonename);
+    initialRule = new InitialTimeZoneRule(zonename,
         initialRawOffset, initialDSTSavings);
+    if (initialRule == NULL) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        goto cleanupParse;
+    }
 
     // Finally, create the RuleBasedTimeZone
     rbtz = new RuleBasedTimeZone(tzid, initialRule);
+    if (rbtz == NULL) {
+        status = U_MEMORY_ALLOCATION_ERROR;
+        goto cleanupParse;
+    }
+    initialRule = NULL; // already adopted by RBTZ, no need to delete
+
+    for (n = 0; n < rules->size(); n++) {
+        TimeZoneRule *r = (TimeZoneRule*)rules->elementAt(n);
+        AnnualTimeZoneRule *atzrule = dynamic_cast<AnnualTimeZoneRule *>(r);
+        if (atzrule != NULL) {
+            if (atzrule->getEndYear() == AnnualTimeZoneRule::MAX_YEAR) {
+                finalRuleCount++;
+                finalRuleIdx = n;
+            }
+        }
+    }
+    if (finalRuleCount > 2) {
+        // Too many final rules
+        status = U_ILLEGAL_ARGUMENT_ERROR;
+        goto cleanupParse;
+    }
+
+    if (finalRuleCount == 1) {
+        if (rules->size() == 1) {
+            // Only one final rule, only governs the initial rule,
+            // which is already initialized, thus, we do not need to
+            // add this transition rule
+            rules->removeAllElements();
+        } else {
+            // Normalize the final rule
+            AnnualTimeZoneRule *finalRule = (AnnualTimeZoneRule*)rules->elementAt(finalRuleIdx);
+            int32_t tmpRaw = finalRule->getRawOffset();
+            int32_t tmpDST = finalRule->getDSTSavings();
+
+            // Find the last non-final rule
+            UDate finalStart, start;
+            finalRule->getFirstStart(initialRawOffset, initialDSTSavings, finalStart);
+            start = finalStart;
+            for (n = 0; n < rules->size(); n++) {
+                if (finalRuleIdx == n) {
+                    continue;
+                }
+                TimeZoneRule *r = (TimeZoneRule*)rules->elementAt(n);
+                UDate lastStart;
+                r->getFinalStart(tmpRaw, tmpDST, lastStart);
+                if (lastStart > start) {
+                    finalRule->getNextStart(lastStart,
+                        r->getRawOffset(),
+                        r->getDSTSavings(),
+                        FALSE,
+                        start);
+                }
+            }
+
+            TimeZoneRule *newRule;
+            UnicodeString tznam;
+            if (start == finalStart) {
+                // Transform this into a single transition
+                newRule = new TimeArrayTimeZoneRule(
+                        finalRule->getName(tznam),
+                        finalRule->getRawOffset(),
+                        finalRule->getDSTSavings(),
+                        &finalStart,
+                        1,
+                        DateTimeRule::UTC_TIME);
+            } else {
+                // Update the end year
+                int32_t y, m, d, dow, doy, mid;
+                Grego::timeToFields(start, y, m, d, dow, doy, mid);
+                newRule = new AnnualTimeZoneRule(
+                        finalRule->getName(tznam),
+                        finalRule->getRawOffset(),
+                        finalRule->getDSTSavings(),
+                        *(finalRule->getRule()),
+                        finalRule->getStartYear(),
+                        y);
+            }
+            if (newRule == NULL) {
+                status = U_MEMORY_ALLOCATION_ERROR;
+                goto cleanupParse;
+            }
+            rules->removeElementAt(finalRuleIdx);
+            rules->addElement(newRule, status);
+            if (U_FAILURE(status)) {
+                delete newRule;
+                goto cleanupParse;
+            }
+        }
+    }
+
     while (!rules->isEmpty()) {
         TimeZoneRule *tzr = (TimeZoneRule*)rules->orphanElementAt(0);
         rbtz->addTransitionRule(tzr, status);
@@ -1809,12 +1953,13 @@ VTimeZone::writeZone(VTZWriter& w, BasicTimeZone& basictz,
         Grego::timeToFields(tzt.getTime() + fromOffset, year, month, dom, dow, doy, mid);
         int32_t weekInMonth = Grego::dayOfWeekInMonth(year, month, dom);
         UBool sameRule = FALSE;
+        const AnnualTimeZoneRule *atzrule;
         if (isDst) {
             if (finalDstRule == NULL
-                && tzt.getTo()->getDynamicClassID() == AnnualTimeZoneRule::getStaticClassID()) {
-                if (((AnnualTimeZoneRule*)tzt.getTo())->getEndYear() == AnnualTimeZoneRule::MAX_YEAR) {
-                    finalDstRule = (AnnualTimeZoneRule*)tzt.getTo()->clone();
-                }
+                && (atzrule = dynamic_cast<const AnnualTimeZoneRule *>(tzt.getTo())) != NULL
+                && atzrule->getEndYear() == AnnualTimeZoneRule::MAX_YEAR
+            ) {
+                finalDstRule = (AnnualTimeZoneRule*)tzt.getTo()->clone();
             }
             if (dstCount > 0) {
                 if (year == dstStartYear + dstCount
@@ -1862,10 +2007,10 @@ VTimeZone::writeZone(VTZWriter& w, BasicTimeZone& basictz,
             }
         } else {
             if (finalStdRule == NULL
-                && tzt.getTo()->getDynamicClassID() == AnnualTimeZoneRule::getStaticClassID()) {
-                    if (((AnnualTimeZoneRule*)tzt.getTo())->getEndYear() == AnnualTimeZoneRule::MAX_YEAR) {
-                    finalStdRule = (AnnualTimeZoneRule*)tzt.getTo()->clone();
-                }
+                && (atzrule = dynamic_cast<const AnnualTimeZoneRule *>(tzt.getTo())) != NULL
+                && atzrule->getEndYear() == AnnualTimeZoneRule::MAX_YEAR
+            ) {
+                finalStdRule = (AnnualTimeZoneRule*)tzt.getTo()->clone();
             }
             if (stdCount > 0) {
                 if (year == stdStartYear + stdCount
@@ -2067,13 +2212,13 @@ VTimeZone::writeFooter(VTZWriter& writer, UErrorCode& status) const {
  * Write a single start time
  */
 void
-VTimeZone::writeZonePropsByTime(VTZWriter& writer, UBool isDst, const UnicodeString& tzname,
+VTimeZone::writeZonePropsByTime(VTZWriter& writer, UBool isDst, const UnicodeString& zonename,
                                 int32_t fromOffset, int32_t toOffset, UDate time, UBool withRDATE,
                                 UErrorCode& status) const {
     if (U_FAILURE(status)) {
         return;
     }
-    beginZoneProps(writer, isDst, tzname, fromOffset, toOffset, time, status);
+    beginZoneProps(writer, isDst, zonename, fromOffset, toOffset, time, status);
     if (U_FAILURE(status)) {
         return;
     }
@@ -2094,14 +2239,14 @@ VTimeZone::writeZonePropsByTime(VTZWriter& writer, UBool isDst, const UnicodeStr
  * Write start times defined by a DOM rule using VTIMEZONE RRULE
  */
 void
-VTimeZone::writeZonePropsByDOM(VTZWriter& writer, UBool isDst, const UnicodeString& tzname,
+VTimeZone::writeZonePropsByDOM(VTZWriter& writer, UBool isDst, const UnicodeString& zonename,
                                int32_t fromOffset, int32_t toOffset,
                                int32_t month, int32_t dayOfMonth, UDate startTime, UDate untilTime,
                                UErrorCode& status) const {
     if (U_FAILURE(status)) {
         return;
     }
-    beginZoneProps(writer, isDst, tzname, fromOffset, toOffset, startTime, status);
+    beginZoneProps(writer, isDst, zonename, fromOffset, toOffset, startTime, status);
     if (U_FAILURE(status)) {
         return;
     }
@@ -2128,14 +2273,14 @@ VTimeZone::writeZonePropsByDOM(VTZWriter& writer, UBool isDst, const UnicodeStri
  * Write start times defined by a DOW rule using VTIMEZONE RRULE
  */
 void
-VTimeZone::writeZonePropsByDOW(VTZWriter& writer, UBool isDst, const UnicodeString& tzname,
+VTimeZone::writeZonePropsByDOW(VTZWriter& writer, UBool isDst, const UnicodeString& zonename,
                                int32_t fromOffset, int32_t toOffset,
                                int32_t month, int32_t weekInMonth, int32_t dayOfWeek,
                                UDate startTime, UDate untilTime, UErrorCode& status) const {
     if (U_FAILURE(status)) {
         return;
     }
-    beginZoneProps(writer, isDst, tzname, fromOffset, toOffset, startTime, status);
+    beginZoneProps(writer, isDst, zonename, fromOffset, toOffset, startTime, status);
     if (U_FAILURE(status)) {
         return;
     }
@@ -2164,7 +2309,7 @@ VTimeZone::writeZonePropsByDOW(VTZWriter& writer, UBool isDst, const UnicodeStri
  * Write start times defined by a DOW_GEQ_DOM rule using VTIMEZONE RRULE
  */
 void
-VTimeZone::writeZonePropsByDOW_GEQ_DOM(VTZWriter& writer, UBool isDst, const UnicodeString& tzname,
+VTimeZone::writeZonePropsByDOW_GEQ_DOM(VTZWriter& writer, UBool isDst, const UnicodeString& zonename,
                                        int32_t fromOffset, int32_t toOffset,
                                        int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
                                        UDate startTime, UDate untilTime, UErrorCode& status) const {
@@ -2174,21 +2319,21 @@ VTimeZone::writeZonePropsByDOW_GEQ_DOM(VTZWriter& writer, UBool isDst, const Uni
     // Check if this rule can be converted to DOW rule
     if (dayOfMonth%7 == 1) {
         // Can be represented by DOW rule
-        writeZonePropsByDOW(writer, isDst, tzname, fromOffset, toOffset,
+        writeZonePropsByDOW(writer, isDst, zonename, fromOffset, toOffset,
                 month, (dayOfMonth + 6)/7, dayOfWeek, startTime, untilTime, status);
         if (U_FAILURE(status)) {
             return;
         }
     } else if (month != UCAL_FEBRUARY && (MONTHLENGTH[month] - dayOfMonth)%7 == 6) {
         // Can be represented by DOW rule with negative week number
-        writeZonePropsByDOW(writer, isDst, tzname, fromOffset, toOffset,
+        writeZonePropsByDOW(writer, isDst, zonename, fromOffset, toOffset,
                 month, -1*((MONTHLENGTH[month] - dayOfMonth + 1)/7), dayOfWeek, startTime, untilTime, status);
         if (U_FAILURE(status)) {
             return;
         }
     } else {
         // Otherwise, use BYMONTHDAY to include all possible dates
-        beginZoneProps(writer, isDst, tzname, fromOffset, toOffset, startTime, status);
+        beginZoneProps(writer, isDst, zonename, fromOffset, toOffset, startTime, status);
         if (U_FAILURE(status)) {
             return;
         }
@@ -2288,7 +2433,7 @@ VTimeZone::writeZonePropsByDOW_GEQ_DOM_sub(VTZWriter& writer, int32_t month, int
  * Write start times defined by a DOW_LEQ_DOM rule using VTIMEZONE RRULE
  */
 void
-VTimeZone::writeZonePropsByDOW_LEQ_DOM(VTZWriter& writer, UBool isDst, const UnicodeString& tzname,
+VTimeZone::writeZonePropsByDOW_LEQ_DOM(VTZWriter& writer, UBool isDst, const UnicodeString& zonename,
                                        int32_t fromOffset, int32_t toOffset,
                                        int32_t month, int32_t dayOfMonth, int32_t dayOfWeek,
                                        UDate startTime, UDate untilTime, UErrorCode& status) const {
@@ -2298,19 +2443,19 @@ VTimeZone::writeZonePropsByDOW_LEQ_DOM(VTZWriter& writer, UBool isDst, const Uni
     // Check if this rule can be converted to DOW rule
     if (dayOfMonth%7 == 0) {
         // Can be represented by DOW rule
-        writeZonePropsByDOW(writer, isDst, tzname, fromOffset, toOffset,
+        writeZonePropsByDOW(writer, isDst, zonename, fromOffset, toOffset,
                 month, dayOfMonth/7, dayOfWeek, startTime, untilTime, status);
     } else if (month != UCAL_FEBRUARY && (MONTHLENGTH[month] - dayOfMonth)%7 == 0){
         // Can be represented by DOW rule with negative week number
-        writeZonePropsByDOW(writer, isDst, tzname, fromOffset, toOffset,
+        writeZonePropsByDOW(writer, isDst, zonename, fromOffset, toOffset,
                 month, -1*((MONTHLENGTH[month] - dayOfMonth)/7 + 1), dayOfWeek, startTime, untilTime, status);
     } else if (month == UCAL_FEBRUARY && dayOfMonth == 29) {
         // Specical case for February
-        writeZonePropsByDOW(writer, isDst, tzname, fromOffset, toOffset,
+        writeZonePropsByDOW(writer, isDst, zonename, fromOffset, toOffset,
                 UCAL_FEBRUARY, -1, dayOfWeek, startTime, untilTime, status);
     } else {
         // Otherwise, convert this to DOW_GEQ_DOM rule
-        writeZonePropsByDOW_GEQ_DOM(writer, isDst, tzname, fromOffset, toOffset,
+        writeZonePropsByDOW_GEQ_DOM(writer, isDst, zonename, fromOffset, toOffset,
                 month, dayOfMonth - 6, dayOfWeek, startTime, untilTime, status);
     }
 }
@@ -2331,6 +2476,18 @@ VTimeZone::writeFinalRule(VTZWriter& writer, UBool isDst, const AnnualTimeZoneRu
         modifiedRule = FALSE;
         dtrule = rule->getRule();
     }
+
+    // If the rule's mills in a day is out of range, adjust start time.
+    // Olson tzdata supports 24:00 of a day, but VTIMEZONE does not.
+    // See ticket#7008/#7518
+
+    int32_t timeInDay = dtrule->getRuleMillisInDay();
+    if (timeInDay < 0) {
+        startTime = startTime + (0 - timeInDay);
+    } else if (timeInDay >= U_MILLIS_PER_DAY) {
+        startTime = startTime - (timeInDay - (U_MILLIS_PER_DAY - 1));
+    }
+
     int32_t toOffset = rule->getRawOffset() + rule->getDSTSavings();
     UnicodeString name;
     rule->getName(name);
@@ -2361,7 +2518,7 @@ VTimeZone::writeFinalRule(VTZWriter& writer, UBool isDst, const AnnualTimeZoneRu
  * Write the opening section of zone properties
  */
 void
-VTimeZone::beginZoneProps(VTZWriter& writer, UBool isDst, const UnicodeString& tzname,
+VTimeZone::beginZoneProps(VTZWriter& writer, UBool isDst, const UnicodeString& zonename,
                           int32_t fromOffset, int32_t toOffset, UDate startTime, UErrorCode& status) const {
     if (U_FAILURE(status)) {
         return;
@@ -2394,7 +2551,7 @@ VTimeZone::beginZoneProps(VTZWriter& writer, UBool isDst, const UnicodeString& t
     // TZNAME
     writer.write(ICAL_TZNAME);
     writer.write(COLON);
-    writer.write(tzname);
+    writer.write(zonename);
     writer.write(ICAL_NEWLINE);
     
     // DTSTART
@@ -2466,4 +2623,3 @@ U_NAMESPACE_END
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
 //eof
-
