@@ -1,13 +1,17 @@
 /*
- * @(#)SingleSubstitutionSubtables.h	1.6 00/03/15
  *
- * (C) Copyright IBM Corp. 1998, 1999, 2000, 2001 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2004 - All Rights Reserved
  *
  */
 
 #ifndef __SINGLESUBSTITUTIONSUBTABLES_H
 #define __SINGLESUBSTITUTIONSUBTABLES_H
 
+/**
+ * \file
+ * \internal
+ */
+
 #include "LETypes.h"
 #include "LEGlyphFilter.h"
 #include "OpenTypeTables.h"
@@ -31,7 +35,7 @@ struct SingleSubstitutionFormat1Subtable : SingleSubstitutionSubtable
 struct SingleSubstitutionFormat2Subtable : SingleSubstitutionSubtable
 {
     le_uint16  glyphCount;
-    LEGlyphID  substituteArray[ANY_NUMBER];
+    TTGlyphID  substituteArray[ANY_NUMBER];
 
     le_uint32  process(GlyphIterator *glyphIterator, const LEGlyphFilter *filter = NULL) const;
 };
