@@ -1,5 +1,5 @@
 /********************************************************************
- * Copyright (c) 1997-2007, International Business Machines
+ * Copyright (c) 1997-2011, International Business Machines
  * Corporation and others. All Rights Reserved.
  ********************************************************************/
 
@@ -10,7 +10,7 @@
 #include "cintltst.h"
 #include "cucdapi.h"
 
-#define ARRAY_SIZE(array) (int32_t)(sizeof array  / sizeof array[0])
+#define LENGTHOF(array) (int32_t)(sizeof(array)/sizeof(array[0]))
 
 void TestUScriptCodeAPI(){
     int i =0;
@@ -72,7 +72,7 @@ void TestUScriptCodeAPI(){
             UScriptCode script[10]={USCRIPT_INVALID_CODE};
             uscript_getCode(testNames[i],script,capacity, &err);
             if( script[0] != expected[i]){
-                   log_err("Error getting script code Got: %i  Expected: %i for name %s\n",
+                   log_data_err("Error getting script code Got: %i  Expected: %i for name %s (Error code does not propagate if data is not present. Are you missing data?)\n",
                        script[0],expected[i],testNames[i]);
                    numErrors++;
             }
@@ -186,7 +186,6 @@ void TestUScriptCodeAPI(){
     }
     /* now test uscript_getScript() API */
     {
-#define MAX_ARRAY_SIZE 23
         uint32_t codepoints[] = {
                 0x0000FF9D, /* USCRIPT_KATAKANA*/
                 0x0000FFBE, /* USCRIPT_HANGUL*/
@@ -211,7 +210,6 @@ void TestUScriptCodeAPI(){
                 0x0001D182, /* USCRIPT_INHERITED*/
                 0x0001D18B, /* USCRIPT_INHERITED*/
                 0x0001D1AD, /* USCRIPT_INHERITED*/
-                0x00110000, /* USCRIPT_INVALID_CODE */
         };
 
         UScriptCode expected[] = {
@@ -238,14 +236,12 @@ void TestUScriptCodeAPI(){
                 USCRIPT_INHERITED ,
                 USCRIPT_INHERITED ,
                 USCRIPT_INHERITED ,
-                USCRIPT_INVALID_CODE,
         };
         UScriptCode code = USCRIPT_INVALID_CODE;
         UErrorCode status = U_ZERO_ERROR;
         UBool passed = TRUE;
 
-        i =0;
-        while(i< MAX_ARRAY_SIZE){
+        for(i=0; i<LENGTHOF(codepoints); ++i){
             code = uscript_getScript(codepoints[i],&status);
             if(U_SUCCESS(status)){
                 if( code != expected[i] ||
@@ -259,7 +255,6 @@ void TestUScriptCodeAPI(){
                          codepoints[i],u_errorName(status));
                 break;
             }
-            i++;
         }
         
         if(passed==FALSE){
@@ -298,7 +293,7 @@ void TestUScriptCodeAPI(){
             }
         }
     }
-                
+
     {
         /*
          * These script codes were originally added to ICU pre-3.6, so that ICU would
@@ -309,13 +304,24 @@ void TestUScriptCodeAPI(){
          * Whenever this happens, the long script names here need to be updated.
          */
         static const char* expectedLong[] = {
-            "Balinese", "Batk", "Blis", "Brah", "Cham", "Cirt", "Cyrs", "Egyd", "Egyh", "Egyp", 
-            "Geok", "Hans", "Hant", "Hmng", "Hung", "Inds", "Java", "Kali", "Latf", "Latg", 
-            "Lepc", "Lina", "Mand", "Maya", "Mero", "Nko", "Orkh", "Perm", "Phags_Pa", "Phoenician", 
-            "Plrd", "Roro", "Sara", "Syre", "Syrj", "Syrn", "Teng", "Vaii", "Visp", "Cuneiform", 
+            "Balinese", "Batak", "Blis", "Brahmi", "Cham", "Cirt", "Cyrs", "Egyd", "Egyh", "Egyptian_Hieroglyphs", 
+            "Geok", "Hans", "Hant", "Hmng", "Hung", "Inds", "Javanese", "Kayah_Li", "Latf", "Latg", 
+            "Lepcha", "Lina", "Mandaic", "Maya", "Mero", "Nko", "Old_Turkic", "Perm", "Phags_Pa", "Phoenician", 
+            "Plrd", "Roro", "Sara", "Syre", "Syrj", "Syrn", "Teng", "Vai", "Visp", "Cuneiform", 
             "Zxxx", "Unknown",
-            "Cari", "Jpan", "Lana", "Lyci", "Lydi", "Olck", "Rjng", "Saur", "Sgnw", "Sund",
-            "Moon", "Mtei",
+            "Carian", "Jpan", "Tai_Tham", "Lycian", "Lydian", "Ol_Chiki", "Rejang", "Saurashtra", "Sgnw", "Sundanese",
+            "Moon", "Meetei_Mayek",
+            /* new in ICU 4.0 */
+            "Imperial_Aramaic", "Avestan", "Cakm", "Kore",
+            "Kaithi", "Mani", "Inscriptional_Pahlavi", "Phlp", "Phlv", "Inscriptional_Parthian", "Samaritan", "Tai_Viet",
+            "Zmth", "Zsym",
+            /* new in ICU 4.4 */
+            "Bamum", "Lisu", "Nkgb", "Old_South_Arabian",
+            /* new in ICU 4.6 */
+            "Bass", "Dupl", "Elba", "Gran", "Kpel", "Loma", "Mend", "Merc",
+            "Narb", "Nbat", "Palm", "Sind", "Wara",
+            /* new in ICU 4.8 */
+            "Afak", "Jurc", "Mroo", "Nshu", "Shrd", "Sora", "Takr", "Tang", "Wole",
         };
         static const char* expectedShort[] = {
             "Bali", "Batk", "Blis", "Brah", "Cham", "Cirt", "Cyrs", "Egyd", "Egyh", "Egyp", 
@@ -325,8 +331,23 @@ void TestUScriptCodeAPI(){
             "Zxxx", "Zzzz",
             "Cari", "Jpan", "Lana", "Lyci", "Lydi", "Olck", "Rjng", "Saur", "Sgnw", "Sund",
             "Moon", "Mtei",
+            /* new in ICU 4.0 */
+            "Armi", "Avst", "Cakm", "Kore",
+            "Kthi", "Mani", "Phli", "Phlp", "Phlv", "Prti", "Samr", "Tavt",
+            "Zmth", "Zsym",
+            /* new in ICU 4.4 */
+            "Bamu", "Lisu", "Nkgb", "Sarb",
+            /* new in ICU 4.6 */
+            "Bass", "Dupl", "Elba", "Gran", "Kpel", "Loma", "Mend", "Merc",
+            "Narb", "Nbat", "Palm", "Sind", "Wara",
+            /* new in ICU 4.8 */
+            "Afak", "Jurc", "Mroo", "Nshu", "Shrd", "Sora", "Takr", "Tang", "Wole",
         };
         int32_t j = 0;
+        if(LENGTHOF(expectedLong)!=(USCRIPT_CODE_LIMIT-USCRIPT_BALINESE)) {
+            log_err("need to add new script codes in cucdapi.c!\n");
+            return;
+        }
         for(i=USCRIPT_BALINESE; (UScriptCode)i<USCRIPT_CODE_LIMIT; i++, j++){
             const char* name = uscript_getName((UScriptCode)i);
             if(name==NULL || strcmp(name,expectedLong[j])!=0){
@@ -337,11 +358,11 @@ void TestUScriptCodeAPI(){
                 log_err("uscript_getShortName failed for code %i: %s!=%s\n", i, name, expectedShort[j]);
             }
         }
-        for(i=0; i<ARRAY_SIZE(expectedLong); i++){
+        for(i=0; i<LENGTHOF(expectedLong); i++){
             UScriptCode fillIn[5] = {USCRIPT_INVALID_CODE};
             UErrorCode status = U_ZERO_ERROR;
             int32_t len = 0;
-            len = uscript_getCode(expectedShort[i], fillIn, ARRAY_SIZE(fillIn), &status);
+            len = uscript_getCode(expectedShort[i], fillIn, LENGTHOF(fillIn), &status);
             if(U_FAILURE(status)){
                 log_err("uscript_getCode failed for script name %s. Error: %s\n",expectedShort[i], u_errorName(status));
             }
@@ -353,4 +374,141 @@ void TestUScriptCodeAPI(){
             }
         }
     }
- }
+
+    {
+        /* test characters which have Script_Extensions */
+        UErrorCode errorCode=U_ZERO_ERROR;
+        if(!(
+                USCRIPT_COMMON==uscript_getScript(0x0640, &errorCode) &&
+                USCRIPT_INHERITED==uscript_getScript(0x0650, &errorCode) &&
+                USCRIPT_ARABIC==uscript_getScript(0xfdf2, &errorCode)) ||
+            U_FAILURE(errorCode)
+        ) {
+            log_err("uscript_getScript(character with Script_Extensions) failed\n");
+        }
+    }
+}
+
+void TestHasScript() {
+    if(!(
+        !uscript_hasScript(0x063f, USCRIPT_COMMON) &&
+        uscript_hasScript(0x063f, USCRIPT_ARABIC) &&  /* main Script value */
+        !uscript_hasScript(0x063f, USCRIPT_SYRIAC) &&
+        !uscript_hasScript(0x063f, USCRIPT_THAANA))
+    ) {
+        log_err("uscript_hasScript(U+063F, ...) is wrong\n");
+    }
+    if(!(
+        uscript_hasScript(0x0640, USCRIPT_COMMON) &&  /* main Script value */
+        uscript_hasScript(0x0640, USCRIPT_ARABIC) &&
+        uscript_hasScript(0x0640, USCRIPT_SYRIAC) &&
+        !uscript_hasScript(0x0640, USCRIPT_THAANA))
+    ) {
+        log_err("uscript_hasScript(U+0640, ...) is wrong\n");
+    }
+    if(!(
+        uscript_hasScript(0x0650, USCRIPT_INHERITED) &&  /* main Script value */
+        uscript_hasScript(0x0650, USCRIPT_ARABIC) &&
+        uscript_hasScript(0x0650, USCRIPT_SYRIAC) &&
+        !uscript_hasScript(0x0650, USCRIPT_THAANA))
+    ) {
+        log_err("uscript_hasScript(U+0650, ...) is wrong\n");
+    }
+    if(!(
+        uscript_hasScript(0x0660, USCRIPT_COMMON) &&  /* main Script value */
+        uscript_hasScript(0x0660, USCRIPT_ARABIC) &&
+        !uscript_hasScript(0x0660, USCRIPT_SYRIAC) &&
+        uscript_hasScript(0x0660, USCRIPT_THAANA))
+    ) {
+        log_err("uscript_hasScript(U+0660, ...) is wrong\n");
+    }
+    if(!(
+        !uscript_hasScript(0xfdf2, USCRIPT_COMMON) &&
+        uscript_hasScript(0xfdf2, USCRIPT_ARABIC) &&  /* main Script value */
+        !uscript_hasScript(0xfdf2, USCRIPT_SYRIAC) &&
+        uscript_hasScript(0xfdf2, USCRIPT_THAANA))
+    ) {
+        log_err("uscript_hasScript(U+FDF2, ...) is wrong\n");
+    }
+}
+
+void TestGetScriptExtensions() {
+    UScriptCode scripts[20];
+    int32_t length;
+    UErrorCode errorCode;
+
+    /* errors and overflows */
+    errorCode=U_PARSE_ERROR;
+    length=uscript_getScriptExtensions(0x0640, scripts, LENGTHOF(scripts), &errorCode);
+    if(errorCode!=U_PARSE_ERROR) {
+        log_err("uscript_getScriptExtensions(U_PARSE_ERROR) did not preserve the UErrorCode - %s\n",
+              u_errorName(errorCode));
+    }
+    errorCode=U_ZERO_ERROR;
+    length=uscript_getScriptExtensions(0x0640, NULL, LENGTHOF(scripts), &errorCode);
+    if(errorCode!=U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("uscript_getScriptExtensions(NULL) did not set U_ILLEGAL_ARGUMENT_ERROR - %s\n",
+              u_errorName(errorCode));
+    }
+    errorCode=U_ZERO_ERROR;
+    length=uscript_getScriptExtensions(0x0640, scripts, -1, &errorCode);
+    if(errorCode!=U_ILLEGAL_ARGUMENT_ERROR) {
+        log_err("uscript_getScriptExtensions(capacity<0) did not set U_ILLEGAL_ARGUMENT_ERROR - %s\n",
+              u_errorName(errorCode));
+    }
+    errorCode=U_ZERO_ERROR;
+    length=uscript_getScriptExtensions(0x0640, scripts, 0, &errorCode);
+    if(errorCode!=U_BUFFER_OVERFLOW_ERROR || length!=2) {
+        log_err("uscript_getScriptExtensions(capacity=0: pure preflighting)=%d != 2 - %s\n",
+              (int)length, u_errorName(errorCode));
+    }
+    errorCode=U_ZERO_ERROR;
+    length=uscript_getScriptExtensions(0x0640, scripts, 1, &errorCode);
+    if(errorCode!=U_BUFFER_OVERFLOW_ERROR || length!=2) {
+        log_err("uscript_getScriptExtensions(capacity=1: preflighting)=%d != 2 - %s\n",
+              (int)length, u_errorName(errorCode));
+    }
+
+    /* normal usage */
+    errorCode=U_ZERO_ERROR;
+    length=uscript_getScriptExtensions(0x063f, scripts, 0, &errorCode);
+    if(U_FAILURE(errorCode) || length!=0) {
+        log_err("uscript_getScriptExtensions(U+063F, capacity=0)=%d != 0 - %s\n",
+              (int)length, u_errorName(errorCode));
+    }
+    length=uscript_getScriptExtensions(0x0640, scripts, LENGTHOF(scripts), &errorCode);
+    if(U_FAILURE(errorCode) || length!=2 || scripts[0]!=USCRIPT_ARABIC || scripts[1]!=USCRIPT_SYRIAC) {
+        log_err("uscript_getScriptExtensions(U+0640)=%d failed - %s\n",
+              (int)length, u_errorName(errorCode));
+    }
+    length=uscript_getScriptExtensions(0xfdf2, scripts, LENGTHOF(scripts), &errorCode);
+    if(U_FAILURE(errorCode) || length!=2 || scripts[0]!=USCRIPT_ARABIC || scripts[1]!=USCRIPT_THAANA) {
+        log_err("uscript_getScriptExtensions(U+FDF2)=%d failed - %s\n",
+              (int)length, u_errorName(errorCode));
+    }
+    length=uscript_getScriptExtensions(0xff65, scripts, LENGTHOF(scripts), &errorCode);
+    if(U_FAILURE(errorCode) || length!=6 || scripts[0]!=USCRIPT_BOPOMOFO || scripts[5]!=USCRIPT_YI) {
+        log_err("uscript_getScriptExtensions(U+FF65)=%d failed - %s\n",
+              (int)length, u_errorName(errorCode));
+    }
+}
+
+void TestBinaryValues() {
+    /*
+     * Unicode 5.1 explicitly defines binary property value aliases.
+     * Verify that they are all recognized.
+     */
+    static const char *const falseValues[]={ "N", "No", "F", "False" };
+    static const char *const trueValues[]={ "Y", "Yes", "T", "True" };
+    int32_t i;
+    for(i=0; i<LENGTHOF(falseValues); ++i) {
+        if(FALSE!=u_getPropertyValueEnum(UCHAR_ALPHABETIC, falseValues[i])) {
+            log_data_err("u_getPropertyValueEnum(UCHAR_ALPHABETIC, \"%s\")!=FALSE (Are you missing data?)\n", falseValues[i]);
+        }
+    }
+    for(i=0; i<LENGTHOF(trueValues); ++i) {
+        if(TRUE!=u_getPropertyValueEnum(UCHAR_ALPHABETIC, trueValues[i])) {
+            log_data_err("u_getPropertyValueEnum(UCHAR_ALPHABETIC, \"%s\")!=TRUE (Are you missing data?)\n", trueValues[i]);
+        }
+    }
+}
