@@ -1,12 +1,16 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2011, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
 #ifndef _CALENDARREGRESSIONTEST_
 #define _CALENDARREGRESSIONTEST_
  
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/calendar.h"
 #include "unicode/gregocal.h"
 #include "intltest.h"
@@ -60,7 +64,14 @@ public:
     void Test4197699(void);
     void TestJ81(void);
     void TestJ438(void);
+    void TestT5555(void);
+    void TestT6745(void);
+	void TestT8057(void);
     void TestLeapFieldDifference(void);
+    void TestMalaysianInstance(void);
+    void TestWeekShift(void);
+    void TestTimeZoneTransitionAdd(void);
+    void TestDeprecates(void);
 
     void printdate(GregorianCalendar *cal, const char *string);
     void dowTest(UBool lenient) ;
@@ -77,6 +88,7 @@ protected:
     UBool failure(UErrorCode status, const char* msg);
 };
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
  
 #endif // _CALENDARREGRESSIONTEST_
 //eof
