@@ -1,5 +1,5 @@
 /***********************************************************************
- * Copyright (c) 1997-2007, International Business Machines Corporation
+ * Copyright (c) 1997-2011, International Business Machines Corporation
  * and others. All Rights Reserved.
  ***********************************************************************/
  
@@ -43,7 +43,7 @@ UBool
 DateFormatMiscTests::failure(UErrorCode status, const char* msg)
 {
     if(U_FAILURE(status)) {
-        errln(UnicodeString("FAIL: ") + msg + " failed, error " + u_errorName(status));
+        errcheckln(status, UnicodeString("FAIL: ") + msg + " failed, error " + u_errorName(status));
         return TRUE;
     }
 
@@ -113,7 +113,10 @@ DateFormatMiscTests::test4097450()
     UErrorCode status = U_ZERO_ERROR;
     SimpleDateFormat *formatter;
     SimpleDateFormat *resultFormatter = new SimpleDateFormat((UnicodeString)"yyyy", status);
-    failure(status, "new SimpleDateFormat");
+    if (U_FAILURE(status)) {
+        dataerrln("Fail new SimpleDateFormat: %s", u_errorName(status));
+        return;
+    }
 
     logln("Format\tSource\tResult");
     logln("-------\t-------\t-------");
@@ -311,23 +314,23 @@ DateFormatMiscTests::test4117335()
 
     UnicodeString jstLong(jstLongC, 5, 5);
 
-    UnicodeString jstShort = "JST";
+//    UnicodeString jstShort = "JST";
     
-    UnicodeString tzID = "meta/Japan";
+    UnicodeString tzID = "Asia/Tokyo";
 
     UnicodeString jdtLong(jdtLongC, 5, 5);
  
-    UnicodeString jdtShort = "JDT";
+//    UnicodeString jdtShort = "JDT";
     UErrorCode status = U_ZERO_ERROR;
     DateFormatSymbols *symbols = new DateFormatSymbols(Locale::getJapan(), status);
     if(U_FAILURE(status)) {
-      errln("Failure creating DateFormatSymbols, %s", u_errorName(status));
+      errcheckln(status, "Failure creating DateFormatSymbols, %s", u_errorName(status));
       delete symbols;
       return;
     }
     failure(status, "new DateFormatSymbols");
     int32_t eraCount = 0;
-    const UnicodeString *eras = symbols->getEras(eraCount);
+    const UnicodeString *eras = symbols->getEraNames(eraCount);
     
     logln(UnicodeString("BC = ") + eras[0]);
     if (eras[0] != bc) {
@@ -356,21 +359,21 @@ DateFormatMiscTests::test4117335()
         errln("*** Should have been " + prettify(jstLong)+ " but it is: " + prettify(zones[index][1]));
         //throw new Exception("Error in long TZ name");
     }
-    logln(UnicodeString("Short zone name = ") + zones[index][2]);
-    if (zones[index][2] != jstShort) {
-        errln("*** Should have been " + prettify(jstShort) + " but it is: " + prettify(zones[index][2]));
-        //throw new Exception("Error in short TZ name");
-    }
+//    logln(UnicodeString("Short zone name = ") + zones[index][2]);
+//    if (zones[index][2] != jstShort) {
+//        errln("*** Should have been " + prettify(jstShort) + " but it is: " + prettify(zones[index][2]));
+//        //throw new Exception("Error in short TZ name");
+//    }
     logln(UnicodeString("Long zone name = ") + zones[index][3]);
     if (zones[index][3] != jdtLong) {
         errln("*** Should have been " + prettify(jstLong) + " but it is: " + prettify(zones[index][3]));
         //throw new Exception("Error in long TZ name");
     }
-    logln(UnicodeString("SHORT zone name = ") + zones[index][4]);
-    if (zones[index][4] != jdtShort) {
-        errln("*** Should have been " + prettify(jstShort)+ " but it is: " + prettify(zones[index][4]));
-        //throw new Exception("Error in short TZ name");
-    }
+//    logln(UnicodeString("SHORT zone name = ") + zones[index][4]);
+//    if (zones[index][4] != jdtShort) {
+//        errln("*** Should have been " + prettify(jstShort)+ " but it is: " + prettify(zones[index][4]));
+//        //throw new Exception("Error in short TZ name");
+//    }
     delete symbols;
 
 }
