@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2000-2006, International Business Machines
+*   Copyright (C) 2000-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -79,12 +79,6 @@ TransliteratorRoundTripTest::runIndexedTest(int32_t index, UBool exec,
     }
 }
 
-//--------------------------------------------------------------------
-// Time bomb - allows temporary behavior that expires at a given
-//             release
-//--------------------------------------------------------------------
-static const UVersionInfo ICU_37 = {3,7,0,0};
-
 
 //--------------------------------------------------------------------
 // TransliteratorPointer
@@ -480,7 +474,7 @@ void RTTest::test(const UnicodeString& sourceRangeVal,
     UnicodeSet okAnyway("[^[:Letter:]]", status);
 
     if (U_FAILURE(status)) {
-        parent->errln("FAIL: Initializing UnicodeSet with [:Other:] or [^[:Letter:]]");
+        parent->dataerrln("FAIL: Initializing UnicodeSet with [:Other:] or [^[:Letter:]] - Error: %s", u_errorName(status));
         return;
     }
 
@@ -522,7 +516,7 @@ void RTTest::test(const UnicodeString& sourceRangeVal,
 
     this->roundtripExclusionsSet.clear();
     if (roundtripExclusions != NULL && strlen(roundtripExclusions) > 0) {
-        this->roundtripExclusionsSet.applyPattern(roundtripExclusions, status);
+        this->roundtripExclusionsSet.applyPattern(UnicodeString(roundtripExclusions, -1, US_INV), status);
         if (U_FAILURE(status)) {
             parent->errln("FAIL: UnicodeSet::applyPattern(%s)", roundtripExclusions);
             return;
@@ -572,7 +566,7 @@ void RTTest::test2(UBool quickRt, int32_t density) {
         Transliterator::createInstance(transliteratorID, UTRANS_FORWARD, parseError,
                                        status));
     if ((Transliterator *)sourceToTarget == NULL) {
-        parent->errln("FAIL: createInstance(" + transliteratorID +
+        parent->dataerrln("FAIL: createInstance(" + transliteratorID +
                    ") returned NULL. Error: " + u_errorName(status)
                    + "\n\tpreContext : " + prettify(parseError.preContext) 
                    + "\n\tpostContext : " + prettify(parseError.postContext));
@@ -961,12 +955,15 @@ void RTTest::logRoundTripFailure(const UnicodeString& from,
 30FF          ; 3.2 #       KATAKANA DIGRAPH KOTO
 31F0..31FF    ; 3.2 #  [16] KATAKANA LETTER SMALL KU..KATAKANA LETTER SMALL RO
 
+    Unicode 5.2 added another Hiragana character:
+1F200         ; 5.2 #       SQUARE HIRAGANA HOKA
+
     We will not add them to the rules until they are more supported (e.g. in fonts on Windows)
     A bug has been filed to remind us to do this: #1979.
     */
-    
-static const char KATAKANA[] = "[[[:katakana:][\\u30A1-\\u30FA\\u30FC]]-[\\u30FF\\u31F0-\\u31FF]]";
-static const char HIRAGANA[] = "[[[:hiragana:][\\u3040-\\u3094]]-[\\u3095-\\u3096\\u309F-\\u30A0]]";
+
+static const char KATAKANA[] = "[[[:katakana:][\\u30A1-\\u30FA\\u30FC]]-[\\u30FF\\u31F0-\\u31FF]-[:^age=5.2:]]";
+static const char HIRAGANA[] = "[[[:hiragana:][\\u3040-\\u3094]]-[\\u3095-\\u3096\\u309F-\\u30A0\\U0001F200-\\U0001F2FF]-[:^age=5.2:]]";
 static const char LENGTH[] = "[\\u30FC]";
 static const char HALFWIDTH_KATAKANA[] = "[\\uFF65-\\uFF9D]";
 static const char KATAKANA_ITERATION[] = "[\\u30FD\\u30FE]";
@@ -991,7 +988,7 @@ void TransliteratorRoundTripTest::TestHiragana() {
     RTTest test("Latin-Hiragana");
     Legal *legal = new Legal();
     test.test(UnicodeString("[a-zA-Z]", ""), 
-              HIRAGANA, 
+              UnicodeString(HIRAGANA, -1, US_INV), 
               HIRAGANA_ITERATION, this, quick, legal);
     delete legal;
 }
@@ -1005,7 +1002,7 @@ void TransliteratorRoundTripTest::TestKatakana() {
     strcat(temp, HALFWIDTH_KATAKANA);
     strcat(temp, "]");
     test.test(UnicodeString("[a-zA-Z]", ""), 
-              KATAKANA,
+              UnicodeString(KATAKANA, -1, US_INV),
               temp, 
               this, quick, legal);
     delete legal;
@@ -1033,7 +1030,7 @@ void TransliteratorRoundTripTest::TestHangul() {
 
 
 #define ASSERT_SUCCESS(status) {if (U_FAILURE(status)) { \
-     errln("error at file %s, line %d, status = %s", __FILE__, __LINE__, \
+     errcheckln(status, "error at file %s, line %d, status = %s", __FILE__, __LINE__, \
          u_errorName(status)); \
          return;}}
     
@@ -1055,26 +1052,16 @@ static void writeStringInU8(FILE *out, const UnicodeString &s) {
 
 void TransliteratorRoundTripTest::TestHan() {
     UErrorCode  status = U_ZERO_ERROR;
-
-    // TODO:  getExemplars() exists only as a C API, taking a USet.
-    //        The set API we need to use exists only on UnicodeSet, not on USet.
-    //        Do a hacky cast, knowing that a USet is really a UnicodeSet in
-    //        the implementation.  Once USet gets the missing API, switch back
-    //        to using that.
-    USet       *USetExemplars = NULL;
-    ULocaleData *uld = ulocdata_open("zh",&status);
-    USetExemplars = uset_open(0, 0);
-    USetExemplars = ulocdata_getExemplarSet(uld, USetExemplars, 0, ULOCDATA_ES_STANDARD, &status);
+    LocalULocaleDataPointer uld(ulocdata_open("zh",&status));
+    LocalUSetPointer USetExemplars(ulocdata_getExemplarSet(uld.getAlias(), uset_openEmpty(), 0, ULOCDATA_ES_STANDARD, &status));
     ASSERT_SUCCESS(status);
-    UnicodeSet *exemplars = (UnicodeSet *)USetExemplars;
-    ulocdata_close(uld);
 
     UnicodeString source;
     UChar32       c;
     int           i;
     for (i=0; ;i++) {
         // Add all of the Chinese exemplar chars to the string "source".
-        c = exemplars->charAt(i);
+        c = uset_charAt(USetExemplars.getAlias(), i);
         if (c == (UChar32)-1) {
             break;
         }
@@ -1106,7 +1093,7 @@ void TransliteratorRoundTripTest::TestHan() {
 
     UnicodeString nfded = target2;
     nfd->transliterate(nfded);
-    UnicodeSet allMarks("[\\u0304\\u0301\\u030C\\u0300\\u0306]", status); // look only for Pinyin tone marks, not all marks (there are some others in there)
+    UnicodeSet allMarks(UNICODE_STRING_SIMPLE("[\\u0304\\u0301\\u030C\\u0300\\u0306]"), status); // look only for Pinyin tone marks, not all marks (there are some others in there)
     ASSERT_SUCCESS(status);
     assertFalse("NumericPinyin must contain no marks", allMarks.containsSome(nfded));
 
@@ -1135,20 +1122,22 @@ void TransliteratorRoundTripTest::TestHan() {
     delete pn;
     delete nfd;
     delete np;
-    uset_close(USetExemplars);
 }
 
 
 void TransliteratorRoundTripTest::TestGreek() {
 
-    if (isICUVersionAtLeast(ICU_37)) {
-        // We temporarily filter against Unicode 4.1, but we only do this
-        // before version 3.4.
-        errln("FAIL: TestGreek needs to be updated to remove delete the [:Age=4.0:] filter ");
-        return;
-    } else {
-        logln("Warning: TestGreek needs to be updated to remove delete the section marked [:Age=4.0:] filter");
-    }
+    // CLDR bug #1911: This test should be moved into CLDR.
+    // It is left in its current state as a regression test.
+    
+//    if (isICUVersionAtLeast(ICU_39)) {
+//        // We temporarily filter against Unicode 4.1, but we only do this
+//        // before version 3.4.
+//        errln("FAIL: TestGreek needs to be updated to remove delete the [:Age=4.0:] filter ");
+//        return;
+//    } else {
+//        logln("Warning: TestGreek needs to be updated to remove delete the section marked [:Age=4.0:] filter");
+//    }
     
     RTTest test("Latin-Greek");
     LegalGreek *legal = new LegalGreek(TRUE);
@@ -1173,14 +1162,17 @@ void TransliteratorRoundTripTest::TestGreek() {
 
 void TransliteratorRoundTripTest::TestGreekUNGEGN() {
 
-    if (isICUVersionAtLeast(ICU_37)) {
-        // We temporarily filter against Unicode 4.1, but we only do this
-        // before version 3.4.
-        errln("FAIL: TestGreek needs to be updated to remove delete the [:Age=4.0:] filter ");
-        return;
-    } else {
-        logln("Warning: TestGreek needs to be updated to remove delete the section marked [:Age=4.0:] filter");
-    }
+    // CLDR bug #1911: This test should be moved into CLDR.
+    // It is left in its current state as a regression test.
+
+//    if (isICUVersionAtLeast(ICU_39)) {
+//        // We temporarily filter against Unicode 4.1, but we only do this
+//        // before version 3.4.
+//        errln("FAIL: TestGreek needs to be updated to remove delete the [:Age=4.0:] filter ");
+//        return;
+//    } else {
+//        logln("Warning: TestGreek needs to be updated to remove delete the section marked [:Age=4.0:] filter");
+//    }
 
     RTTest test("Latin-Greek/UNGEGN");
     LegalGreek *legal = new LegalGreek(FALSE);
@@ -1202,14 +1194,17 @@ void TransliteratorRoundTripTest::TestGreekUNGEGN() {
 
 void TransliteratorRoundTripTest::Testel() {
     
-    if (isICUVersionAtLeast(ICU_37)) {
-        // We temporarily filter against Unicode 4.1, but we only do this
-        // before version 3.4.
-        errln("FAIL: TestGreek needs to be updated to remove delete the [:Age=4.0:] filter ");
-        return;
-    } else {
-        logln("Warning: TestGreek needs to be updated to remove delete the section marked [:Age=4.0:] filter");
-    }
+    // CLDR bug #1911: This test should be moved into CLDR.
+    // It is left in its current state as a regression test.
+
+//    if (isICUVersionAtLeast(ICU_39)) {
+//        // We temporarily filter against Unicode 4.1, but we only do this
+//        // before version 3.4.
+//        errln("FAIL: TestGreek needs to be updated to remove delete the [:Age=4.0:] filter ");
+//        return;
+//    } else {
+//        logln("Warning: TestGreek needs to be updated to remove delete the section marked [:Age=4.0:] filter");
+//    }
 
     RTTest test("Latin-el");
     LegalGreek *legal = new LegalGreek(FALSE);
@@ -1232,10 +1227,10 @@ void TransliteratorRoundTripTest::Testel() {
 
 
 void TransliteratorRoundTripTest::TestArabic() {
-    UnicodeString ARABIC("[\\u060C\\u061B\\u061F\\u0621\\u0627-\\u063A\\u0641-\\u0655\\u0660-\\u066C\\u067E\\u0686\\u0698\\u06A4\\u06AD\\u06AF\\u06CB-\\u06CC\\u06F0-\\u06F9]");
+    UnicodeString ARABIC("[\\u060C\\u061B\\u061F\\u0621\\u0627-\\u063A\\u0641-\\u0655\\u0660-\\u066C\\u067E\\u0686\\u0698\\u06A4\\u06AD\\u06AF\\u06CB-\\u06CC\\u06F0-\\u06F9]", -1, US_INV);
     Legal *legal = new Legal();
     RTTest test("Latin-Arabic");
-        test.test("[a-zA-Z\\u02BE\\u02BF\\u207F]", ARABIC, "[a-zA-Z\\u02BE\\u02BF\\u207F]",this, quick, legal); //
+        test.test(UNICODE_STRING_SIMPLE("[a-zA-Z\\u02BE\\u02BF\\u207F]"), ARABIC, "[a-zA-Z\\u02BE\\u02BF\\u207F]",this, quick, legal); //
    delete legal;
 }
 class LegalHebrew : public Legal {
@@ -1250,8 +1245,8 @@ public:
 };
 
 LegalHebrew::LegalHebrew(UErrorCode& error){
-    FINAL.applyPattern("[\\u05DA\\u05DD\\u05DF\\u05E3\\u05E5]", error);
-    NON_FINAL.applyPattern("[\\u05DB\\u05DE\\u05E0\\u05E4\\u05E6]", error);
+    FINAL.applyPattern(UNICODE_STRING_SIMPLE("[\\u05DA\\u05DD\\u05DF\\u05E3\\u05E5]"), error);
+    NON_FINAL.applyPattern(UNICODE_STRING_SIMPLE("[\\u05DB\\u05DE\\u05E0\\u05E4\\u05E6]"), error);
     LETTER.applyPattern("[:letter:]", error);
 }
 UBool LegalHebrew::is(const UnicodeString& sourceString)const{
@@ -1270,23 +1265,25 @@ UBool LegalHebrew::is(const UnicodeString& sourceString)const{
     return TRUE;
 }
 void TransliteratorRoundTripTest::TestHebrew() {
-    if (isICUVersionAtLeast(ICU_37)) {
-        // We temporarily filter against Unicode 4.1, but we only do this
-        // before version 3.4.
-        errln("FAIL: TestHebrew needs to be updated to remove delete the [:Age=4.0:] filter ");
-        return;
-    } else {
-        logln("Warning: TestHebrew needs to be updated to remove delete the section marked [:Age=4.0:] filter");
-    }
+    // CLDR bug #1911: This test should be moved into CLDR.
+    // It is left in its current state as a regression test.
+//    if (isICUVersionAtLeast(ICU_39)) {
+//        // We temporarily filter against Unicode 4.1, but we only do this
+//        // before version 3.4.
+//        errln("FAIL: TestHebrew needs to be updated to remove delete the [:Age=4.0:] filter ");
+//        return;
+//    } else {
+//        logln("Warning: TestHebrew needs to be updated to remove delete the section marked [:Age=4.0:] filter");
+//    }
     //long start = System.currentTimeMillis();
     UErrorCode error = U_ZERO_ERROR;
     LegalHebrew* legal = new LegalHebrew(error);
     if(U_FAILURE(error)){
-        errln("Could not construct LegalHebrew object. Error: %s", u_errorName(error));
+        dataerrln("Could not construct LegalHebrew object. Error: %s", u_errorName(error));
         return;
     }
     RTTest test("Latin-Hebrew");
-    test.test("[a-zA-Z\\u02BC\\u02BB]", "[[[:hebrew:]-[\\u05BD\\uFB00-\\uFBFF]]&[:Age=4.0:]]", "[\\u05F0\\u05F1\\u05F2]", this, quick, legal);
+    test.test(UNICODE_STRING_SIMPLE("[a-zA-Z\\u02BC\\u02BB]"), UNICODE_STRING_SIMPLE("[[[:hebrew:]-[\\u05BD\\uFB00-\\uFBFF]]&[:Age=4.0:]]"), "[\\u05F0\\u05F1\\u05F2]", this, quick, legal);
    
     //showElapsed(start, "TestHebrew");
     delete legal;
@@ -1387,20 +1384,22 @@ void TransliteratorRoundTripTest::TestDevanagariLatin() {
                 errln("FAIL: could not create the Inverse:-( \n");
             }
         }else {
-            errln("FAIL: could not create the transliterator. Error: %s\n", u_errorName(status));
+            dataerrln("FAIL: could not create the transliterator. Error: %s\n", u_errorName(status));
         }
 
     }
     RTTest test("Latin-Devanagari");
     Legal *legal = new LegalIndic();
-    if (isICUVersionAtLeast(ICU_37)) {
-        // We temporarily filter against Unicode 4.1, but we only do this
-        // before version 3.4.
-        errln("FAIL: TestDevanagariLatin needs to be updated to remove delete the [:Age=4.1:] filter ");
-        return;
-    } else {
-        logln("Warning: TestDevanagariLatin needs to be updated to remove delete the section marked [:Age=4.1:] filter");
-    }
+    // CLDR bug #1911: This test should be moved into CLDR.
+    // It is left in its current state as a regression test.
+//    if (isICUVersionAtLeast(ICU_39)) {
+//        // We temporarily filter against Unicode 4.1, but we only do this
+//        // before version 3.4.
+//        errln("FAIL: TestDevanagariLatin needs to be updated to remove delete the [:Age=4.1:] filter ");
+//        return;
+//    } else {
+//        logln("Warning: TestDevanagariLatin needs to be updated to remove delete the section marked [:Age=4.1:] filter");
+//    }
     test.test(UnicodeString(latinForIndic, ""), 
         UnicodeString("[[[:Devanagari:][\\u094d][\\u0964\\u0965]]&[:Age=4.1:]]", ""), "[\\u0965\\u0904]", this, quick, 
             legal, 50);
@@ -1440,7 +1439,7 @@ static const char * const interIndicArray[] = {
     "[\\u0901\\u0904\\u093c\\u0943-\\u094a\\u0951-\\u0954\\u0962\\u0963\\u090B\\u090C\\u090D\\u0911\\u0916\\u0917\\u0918\\u091B\\u091D\\u0920\\u0921\\u0922\\u0925\\u0926\\u0927\\u092B\\u092C\\u092D\\u0936\\u093d\\u0950[\\u0958-\\u0961]\\u097d]", /*roundtrip exclusions*/
 
     "DEVANAGARI-Tamil", "[:Devanagari:]", "[:tamil:]", 
-    "[\\u0bd7]", /*roundtrip exclusions*/
+    "[\\u0bd7\\u0BF0\\u0BF1\\u0BF2]", /*roundtrip exclusions*/
 
     "Telugu-DEVANAGARI", "[:telugu:]", "[:Devanagari:]", 
     "[\\u0904\\u093c\\u0950\\u0945\\u0949\\u0951-\\u0954\\u0962\\u0963\\u090D\\u0911\\u093d\\u0929\\u0934[\\u0958-\\u095f]\\u097d]", /*roundtrip exclusions*/
@@ -1482,7 +1481,7 @@ static const char * const interIndicArray[] = {
     "[\\u0981\\u09bc\\u09c3\\u09c4\\u09e2\\u09e3\\u09f0\\u09f1\\u098B\\u098C\\u0996\\u0997\\u0998\\u099B\\u099D\\u09A0\\u09A1\\u09A2\\u09A5\\u09A6\\u09A7\\u09AB\\u09AC\\u09AD\\u09B6\\u09DC\\u09DD\\u09DF\\u09E0\\u09E1\\u09f2-\\u09fa\\u09ce]", /*roundtrip exclusions*/
 
     "BENGALI-Tamil", "[:BENGALI:]", "[:tamil:]",
-    "[\\u0bc6\\u0bc7\\u0bca\\u0B8E\\u0B92\\u0BA9\\u0BB1\\u0BB3\\u0BB4\\u0BB5]", /*roundtrip exclusions*/
+    "[\\u0bc6\\u0bc7\\u0bca\\u0B8E\\u0B92\\u0BA9\\u0BB1\\u0BB3\\u0BB4\\u0BB5\\u0BF0\\u0BF1\\u0BF2]", /*roundtrip exclusions*/
 
     "Telugu-BENGALI", "[:telugu:]", "[:BENGALI:]", 
     "[\\u09e2\\u09e3\\u09bc\\u09d7\\u09f0\\u09f1\\u09dc\\u09dd\\u09df\\u09f2-\\u09fa\\u09ce]", /*roundtrip exclusions*/
@@ -1501,7 +1500,7 @@ static const char * const interIndicArray[] = {
 
     "BENGALI-MALAYALAM", "[:BENGALI:]", "[:MALAYALAM:]",
     "[\\u0d46\\u0d4a\\u0d47\\u0d31-\\u0d35\\u0d0e\\u0d12]", /*roundtrip exclusions*/
-       
+
     "GUJARATI-GURMUKHI", "[:GUJARATI:]", "[:GURMUKHI:]", 
     "[\\u0A02\\u0ab3\\u0ab6\\u0A70\\u0a71\\u0a82\\u0a83\\u0ac3\\u0ac4\\u0ac5\\u0ac9\\u0a5c\\u0a72\\u0a73\\u0a74\\u0a8b\\u0a8d\\u0a91\\u0abd]", /*roundtrip exclusions*/
 
@@ -1518,7 +1517,7 @@ static const char * const interIndicArray[] = {
     "[\\u0A01\\u0A02\\u0a33\\u0a36\\u0a3c\\u0a70\\u0a71\\u0a47\\u0A16\\u0A17\\u0A18\\u0A1B\\u0A1D\\u0A20\\u0A21\\u0A22\\u0A25\\u0A26\\u0A27\\u0A2B\\u0A2C\\u0A2D\\u0A59\\u0A5A\\u0A5B\\u0A5C\\u0A5E\\u0A72\\u0A73\\u0A74]", /*roundtrip exclusions*/
 
     "GURMUKHI-TAMIL", "[:GURMUKHI:]", "[:TAMIL:]",
-    "[\\u0b82\\u0bc6\\u0bca\\u0bd7\\u0bb7\\u0bb3\\u0b83\\u0B8E\\u0B92\\u0BA9\\u0BB1\\u0BB4\\u0bb6]", /*roundtrip exclusions*/
+    "[\\u0b82\\u0bc6\\u0bca\\u0bd7\\u0bb7\\u0bb3\\u0b83\\u0B8E\\u0B92\\u0BA9\\u0BB1\\u0BB4\\u0bb6\\u0BF0\\u0BF1\\u0BF2]", /*roundtrip exclusions*/
 
     "TELUGU-GURMUKHI", "[:TELUGU:]", "[:GURMUKHI:]", 
     "[\\u0A02\\u0a33\\u0a36\\u0a3c\\u0a70\\u0a71\\u0A59\\u0A5A\\u0A5B\\u0A5C\\u0A5E\\u0A72\\u0A73\\u0A74]", /*roundtrip exclusions*/
@@ -1548,7 +1547,7 @@ static const char * const interIndicArray[] = {
     "[\\u0A81\\u0a8c\\u0abc\\u0ac3\\u0Ac4\\u0Ac5\\u0Ac9\\u0Ac7\\u0A8B\\u0A8D\\u0A91\\u0A96\\u0A97\\u0A98\\u0A9B\\u0A9D\\u0AA0\\u0AA1\\u0AA2\\u0AA5\\u0AA6\\u0AA7\\u0AAB\\u0AAC\\u0AAD\\u0AB6\\u0ABD\\u0AD0\\u0AE0\\u0AE1]", /*roundtrip exclusions*/
 
     "GUJARATI-TAMIL", "[:GUJARATI:]", "[:TAMIL:]",
-    "[\\u0Bc6\\u0Bca\\u0Bd7\\u0B8E\\u0B92\\u0BA9\\u0BB1\\u0BB4]", /*roundtrip exclusions*/
+    "[\\u0Bc6\\u0Bca\\u0Bd7\\u0B8E\\u0B92\\u0BA9\\u0BB1\\u0BB4\\u0BF0\\u0BF1\\u0BF2]", /*roundtrip exclusions*/
 
     "TELUGU-GUJARATI", "[:TELUGU:]", "[:GUJARATI:]", 
     "[\\u0abc\\u0Ac5\\u0Ac9\\u0A8D\\u0A91\\u0ABD\\u0Ad0]", /*roundtrip exclusions*/
@@ -1572,7 +1571,7 @@ static const char * const interIndicArray[] = {
     "[\\u0B01\\u0b3c\\u0b43\\u0b56\\u0B0B\\u0B0C\\u0B16\\u0B17\\u0B18\\u0B1B\\u0B1D\\u0B20\\u0B21\\u0B22\\u0B25\\u0B26\\u0B27\\u0B2B\\u0B2C\\u0B2D\\u0B36\\u0B3D\\u0B5C\\u0B5D\\u0B5F\\u0B60\\u0B61\\u0b70\\u0b71]", /*roundtrip exclusions*/
 
     "ORIYA-TAMIL", "[:ORIYA:]", "[:TAMIL:]",
-    "[\\u0bc6\\u0bca\\u0bc7\\u0B8E\\u0B92\\u0BA9\\u0BB1\\u0BB4\\u0BB5]", /*roundtrip exclusions*/
+    "[\\u0bc6\\u0bca\\u0bc7\\u0B8E\\u0B92\\u0BA9\\u0BB1\\u0BB4\\u0BB5\\u0BF0\\u0BF1\\u0BF2]", /*roundtrip exclusions*/
 
     "TELUGU-ORIYA", "[:TELUGU:]", "[:ORIYA:]", 
     "[\\u0b3c\\u0b57\\u0b56\\u0B3D\\u0B5C\\u0B5D\\u0B5F\\u0b70\\u0b71]", /*roundtrip exclusions*/
@@ -1593,19 +1592,19 @@ static const char * const interIndicArray[] = {
     "[\\u0D47\\u0D46\\u0D4a\\u0D0E\\u0D12\\u0D31\\u0D34\\u0D35]", /*roundtrip exclusions*/
 
     "TELUGU-TAMIL", "[:TELUGU:]", "[:TAMIL:]", 
-    "[\\u0bd7\\u0ba9\\u0bb4]", /*roundtrip exclusions*/
+    "[\\u0bd7\\u0ba9\\u0bb4\\u0BF0\\u0BF1\\u0BF2]", /*roundtrip exclusions*/
 
     "TAMIL-TELUGU", "[:TAMIL:]", "[:TELUGU:]",
     "[\\u0C01\\u0c43\\u0c44\\u0c46\\u0c47\\u0c55\\u0c56\\u0c66\\u0C0B\\u0C0C\\u0C16\\u0C17\\u0C18\\u0C1B\\u0C1D\\u0C20\\u0C21\\u0C22\\u0C25\\u0C26\\u0C27\\u0C2B\\u0C2C\\u0C2D\\u0C36\\u0C60\\u0C61]", /*roundtrip exclusions*/
 
     "KANNADA-TAMIL", "[:KANNADA:]", "[:TAMIL:]", 
-    "[\\u0bd7\\u0bc6\\u0ba9\\u0bb4]", /*roundtrip exclusions*/
+    "[\\u0bd7\\u0bc6\\u0ba9\\u0bb4\\u0BF0\\u0BF1\\u0BF2]", /*roundtrip exclusions*/
 
     "TAMIL-KANNADA", "[:TAMIL:]", "[:KANNADA:]",
     "[\\u0cc3\\u0cc4\\u0cc6\\u0cc7\\u0cd5\\u0cd6\\u0C8B\\u0C8C\\u0C96\\u0C97\\u0C98\\u0C9B\\u0C9D\\u0CA0\\u0CA1\\u0CA2\\u0CA5\\u0CA6\\u0CA7\\u0CAB\\u0CAC\\u0CAD\\u0CB6\\u0cbc\\u0cbd\\u0CDE\\u0CE0\\u0CE1]", /*roundtrip exclusions*/
 
     "MALAYALAM-TAMIL", "[:MALAYALAM:]", "[:TAMIL:]", 
-    "[\\u0ba9]", /*roundtrip exclusions*/
+    "[\\u0ba9\\u0BF0\\u0BF1\\u0BF2]", /*roundtrip exclusions*/
 
     "TAMIL-MALAYALAM", "[:TAMIL:]", "[:MALAYALAM:]",
     "[\\u0d43\\u0d12\\u0D0B\\u0D0C\\u0D16\\u0D17\\u0D18\\u0D1B\\u0D1D\\u0D20\\u0D21\\u0D22\\u0D25\\u0D26\\u0D27\\u0D2B\\u0D2C\\u0D2D\\u0D36\\u0D60\\u0D61]", /*roundtrip exclusions*/
@@ -1627,31 +1626,30 @@ static const char * const interIndicArray[] = {
 
     "KANNADA-MALAYALAM", "[:KANNADA:]", "[:MALAYALAM:]",
     "[\\u0d4c\\u0d57\\u0d46\\u0D34]", /*roundtrip exclusions*/
-    
+
     "Latin-Bengali",latinForIndic, "[[:Bengali:][\\u0964\\u0965]]", 
     "[\\u0965\\u09f0-\\u09fa\\u09ce]" /*roundtrip exclusions*/ ,
-    
+
     "Latin-Gurmukhi", latinForIndic, "[[:Gurmukhi:][\\u0964\\u0965]]", 
     "[\\u0a01\\u0965\\u0a02\\u0a72\\u0a73\\u0a74]" /*roundtrip exclusions*/,
-    
+
     "Latin-Gujarati",latinForIndic, "[[:Gujarati:][\\u0964\\u0965]]", 
     "[\\u0965]" /*roundtrip exclusions*/,
-    
+
     "Latin-Oriya",latinForIndic, "[[:Oriya:][\\u0964\\u0965]]", 
     "[\\u0965\\u0b70]" /*roundtrip exclusions*/,
-    
+
     "Latin-Tamil",latinForIndic, "[:Tamil:]", 
-    NULL /*roundtrip exclusions*/,
-    
+    "[\\u0BF0\\u0BF1\\u0BF2]" /*roundtrip exclusions*/,
+
     "Latin-Telugu",latinForIndic, "[:Telugu:]", 
     NULL /*roundtrip exclusions*/,
-    
+
     "Latin-Kannada",latinForIndic, "[:Kannada:]", 
     NULL /*roundtrip exclusions*/,
-    
+
     "Latin-Malayalam",latinForIndic, "[:Malayalam:]", 
     NULL /*roundtrip exclusions*/  
-
 };
 
 void TransliteratorRoundTripTest::TestDebug(const char* name,const char fromSet[],
@@ -1669,14 +1667,16 @@ void TransliteratorRoundTripTest::TestInterIndic() {
         logln("Testing only 5 of %i. Skipping rest (use -e for exhaustive)",num);
         num = 5;
     }
-    if (isICUVersionAtLeast(ICU_37)) {
-        // We temporarily filter against Unicode 4.1, but we only do this
-        // before version 3.4.
-        errln("FAIL: TestInterIndic needs to be updated to remove delete the [:Age=4.1:] filter ");
-        return;
-    } else {
-        logln("Warning: TestInterIndic needs to be updated to remove delete the section marked [:Age=4.1:] filter");
-    }
+    // CLDR bug #1911: This test should be moved into CLDR.
+    // It is left in its current state as a regression test.
+//    if (isICUVersionAtLeast(ICU_39)) {
+//        // We temporarily filter against Unicode 4.1, but we only do this
+//        // before version 3.4.
+//        errln("FAIL: TestInterIndic needs to be updated to remove delete the [:Age=4.1:] filter ");
+//        return;
+//    } else {
+//        logln("Warning: TestInterIndic needs to be updated to remove delete the section marked [:Age=4.1:] filter");
+//    }
     for(int i = 0; i < num;i++){
         RTTest test(interIndicArray[i*INTER_INDIC_ARRAY_WIDTH + 0]);
         Legal *legal = new LegalIndic();
