@@ -1,21 +1,28 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2002, International Business Machines Corporation and
+ * Copyright (c) 1997-2011, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
 #ifndef UNICODESTRINGTEST_H
 #define UNICODESTRINGTEST_H
 
+#include "unicode/unistr.h"
 #include "intltest.h"
 
+U_NAMESPACE_BEGIN
+
+class Appendable;
+
+U_NAMESPACE_END
+
 /**
  * Perform API and functionality tests for class UnicodeString
  **/
 class UnicodeStringTest: public IntlTest {
 public:
     UnicodeStringTest() {}
-    virtual ~UnicodeStringTest() {}
+    virtual ~UnicodeStringTest();
     
     void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
 
@@ -47,14 +54,11 @@ public:
      * Test methods startsWith and endsWith
      **/
     void TestPrefixAndSuffix(void);
+    void TestStartsWithAndEndsWithNulTerminated();
     /**
      * Test method findAndReplace
      **/
     void TestFindAndReplace(void);
-    /**
-     * Test method numDisplayCells
-     **/
-    void TestCellWidth(void);
     /**
      * Test method reverse
      **/
@@ -71,17 +75,35 @@ public:
      * Test the unescape() function.
      */
     void TestUnescape(void);
+
+    void _testUnicodeStringHasMoreChar32Than(const UnicodeString &s, int32_t start, int32_t length, int32_t number);
+    void TestCountChar32();
+    void TestBogus();
+    void TestStringEnumeration();
+    void TestNameSpace();
+    void TestUTF32();
+    void TestUTF8();
+    void TestReadOnlyAlias();
+    void doTestAppendable(UnicodeString &dest, Appendable &app);
+    void TestAppendable();
+    void TestUnicodeStringImplementsAppendable();
+    void TestSizeofUnicodeString();
 };
 
 class StringCaseTest: public IntlTest {
 public:
     StringCaseTest() {}
-    virtual ~StringCaseTest() {}
+    virtual ~StringCaseTest();
     
     void runIndexedTest(int32_t index, UBool exec, const char *&name, char *par=0);
 
     void TestCaseConversion();
-    void TestTitleCasing();
+
+    void TestCasingImpl(const UnicodeString &input,
+                        const UnicodeString &output,
+                        int32_t whichCase,
+                        void *iter, const char *localeID, uint32_t options);
+    void TestCasing();
 };
 
 #endif
