@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (C) 2001 IBM, Inc.   All Rights Reserved.
+ * Copyright (C) 2001-2011 IBM, Inc.   All Rights Reserved.
  *
  ********************************************************************/
 /********************************************************************************
@@ -37,6 +37,9 @@
 #include "cstring.h"
 #include "uoptions.h"
 #include "ucol_imp.h"
+#include <unicode/ures.h>
+#include <unicode/uniset.h>
+#include <unicode/usetiter.h>
 
 /**
 * Command line option variables. 
@@ -44,18 +47,19 @@
 * command line by the user.
 */
 static UOption options[]={
-    UOPTION_HELP_H,
-    UOPTION_HELP_QUESTION_MARK,
-    {"locale",        NULL, NULL, NULL, 'l', UOPT_REQUIRES_ARG, 0},
-    {"serialize",     NULL, NULL, NULL, 'z', UOPT_NO_ARG, 0},
-	UOPTION_DESTDIR,
-    UOPTION_SOURCEDIR,
-    {"attribute",     NULL, NULL, NULL, 'a', UOPT_REQUIRES_ARG, 0},
-    {"rule",          NULL, NULL, NULL, 'r', UOPT_REQUIRES_ARG, 0},
-    {"normalization", NULL, NULL, NULL, 'n', UOPT_REQUIRES_ARG, 0},
-    {"scripts",       NULL, NULL, NULL, 't', UOPT_NO_ARG, 0},
-    {"reducehan",     NULL, NULL, NULL, 'e', UOPT_NO_ARG, 0},
-	UOPTION_VERBOSE
+    /* 00 */ UOPTION_HELP_H, 
+    /* 01 */ UOPTION_HELP_QUESTION_MARK,
+    /* 02 */ {"locale",        NULL, NULL, NULL, 'l', UOPT_REQUIRES_ARG, 0},
+    /* 03 */ {"serialize",     NULL, NULL, NULL, 'z', UOPT_NO_ARG, 0},
+	/* 04 */ UOPTION_DESTDIR,
+    /* 05 */ UOPTION_SOURCEDIR,
+    /* 06 */ {"attribute",     NULL, NULL, NULL, 'a', UOPT_REQUIRES_ARG, 0},
+    /* 07 */ {"rule",          NULL, NULL, NULL, 'r', UOPT_REQUIRES_ARG, 0},
+    /* 08 */ {"normalization", NULL, NULL, NULL, 'n', UOPT_REQUIRES_ARG, 0},
+    /* 09 */ {"scripts",       NULL, NULL, NULL, 't', UOPT_NO_ARG, 0},
+    /* 10 */ {"reducehan",     NULL, NULL, NULL, 'e', UOPT_NO_ARG, 0},
+	/* 11 */ UOPTION_VERBOSE,
+    /* 12 */ {"wholescripts",      NULL, NULL, NULL, 'W', UOPT_NO_ARG, 0}
 };
 
 /**
@@ -69,11 +73,9 @@ static FILE *OUTPUT_;
 
 static UColAttributeValue ATTRIBUTE_[UCOL_ATTRIBUTE_COUNT] = {
     UCOL_DEFAULT, UCOL_DEFAULT, UCOL_DEFAULT, UCOL_DEFAULT, UCOL_DEFAULT, 
-    UCOL_DEFAULT, UCOL_DEFAULT
+    UCOL_DEFAULT, UCOL_DEFAULT, UCOL_DEFAULT,
 };
 
-static UNormalizationMode NORMALIZATION_ = UNORM_DEFAULT;
-
 typedef struct {
     int   value;
     char *name;
@@ -87,6 +89,8 @@ static const EnumNameValuePair ATTRIBUTE_NAME_[] = {
     {UCOL_NORMALIZATION_MODE, 
         "UCOL_NORMALIZATION_MODE|UCOL_DECOMPOSITION_MODE"},
     {UCOL_STRENGTH, "UCOL_STRENGTH"},
+	{UCOL_HIRAGANA_QUATERNARY_MODE, "UCOL_HIRAGANA_QUATERNARY_MODE"},
+    {UCOL_NUMERIC_COLLATION, "UCOL_NUMERIC_COLLATION"},
     NULL
 };
      
@@ -102,16 +106,6 @@ static const EnumNameValuePair ATTRIBUTE_VALUE_[] = {
     {UCOL_NON_IGNORABLE, "UCOL_NON_IGNORABLE"},
     {UCOL_LOWER_FIRST, "UCOL_LOWER_FIRST"},
     {UCOL_UPPER_FIRST, "UCOL_UPPER_FIRST"},
-    {UCOL_ON_WITHOUT_HANGUL, "UCOL_ON_WITHOUT_HANGUL"},
-    NULL
-};
-
-static const EnumNameValuePair NORMALIZATION_VALUE_[] = {
-    {UNORM_NONE, "UNORM_NONE"},
-    {UNORM_NFD, "UNORM_NFD"},
-    {UNORM_NFKD, "UNORM_NFKD"},
-    {UNORM_NFC, "UNORM_NFC|UNORM_DEFAULT"},
-    {UNORM_NFKC, "UNORM_NFKC"},
     NULL
 };
 
@@ -265,18 +259,17 @@ void serialize(FILE *f, UChar *rule, int rlen, UBool contractiononly,
     
     src.opts = &opts;
       
-    src.source       = src.current = rule;
+    src.source       = rule; 
+	src.current = rule;
     src.end          = rule + rlen;
     src.extraCurrent = src.end;
     src.extraEnd     = src.end + UCOL_TOK_EXTRA_RULE_SPACE_SIZE;
 
         
-    while ((current = ucol_tok_parseNextToken(&src, &strength, &chOffset, 
-                                              &chLen, &exOffset, &exLen,
-                                              &prefixOffset, &prefixLen,
-                                              &specs, rstart, &parseError,
-                                              &error)) 
-                                              != NULL) {
+    while ((current = ucol_tok_parseNextToken(&src, rstart, &parseError,
+                                              &error)) != NULL) {
+      chOffset = src.parsedToken.charsOffset;
+      chLen = src.parsedToken.charsLen;
         // contractions handled here
         if (!contractiononly || chLen > 1) {
             ucol_setText(iter, rule + chOffset, chLen, &error);
@@ -331,17 +324,18 @@ void outputAttribute(UCollator *collator, UErrorCode *error)
 */
 void outputNormalization(UCollator *collator) 
 {
-    int normmode = ucol_getNormalization(collator);
+	UErrorCode status = U_ZERO_ERROR;
+    int normmode = ucol_getAttribute(collator, UCOL_NORMALIZATION_MODE, &status);
     int count = 0;
     while (TRUE) {
         // getting attribute name
-        if (NORMALIZATION_VALUE_[count].value == normmode) {
+        if (ATTRIBUTE_VALUE_[count].value == normmode) {
             break;
         }
         count ++;
     }
     fprintf(OUTPUT_, "NORMALIZATION MODE = %s\n", 
-        NORMALIZATION_VALUE_[count].name);
+            ATTRIBUTE_VALUE_[count].name);
 }
 
 /**
@@ -459,17 +453,6 @@ void setAttributes(UCollator *collator, UErrorCode *error)
     }
 }
 
-/**
-* Sets the collator with the normalization mode
-* @param collator
-*/
-void setNormalization(UCollator *collator) 
-{
-    if (NORMALIZATION_ != UCOL_DEFAULT) {
-        ucol_setNormalization(collator, NORMALIZATION_);
-    }
-}
-
 /**
 * Appends directory path with an ending seperator if necessary.
 * @param path with enough space to append one seperator
@@ -522,7 +505,6 @@ void serialize() {
                 return;
             }
             setAttributes(COLLATOR_, &error);
-            setNormalization(COLLATOR_);
             if (U_FAILURE(error)) {
                 fprintf(stdout, "Collator attribute setting failed:");
                 fprintf(stdout, u_errorName(error));
@@ -552,7 +534,6 @@ CLOSEUCA :
                 return;
             }
             setAttributes(COLLATOR_, &error);
-            setNormalization(COLLATOR_);
             if (U_FAILURE(error)) {
                 fprintf(stdout, "Collator attribute setting failed:");
                 fprintf(stdout, u_errorName(error));
@@ -591,7 +572,7 @@ CLOSETAILOR :
     }
 
     if (options[7].doesOccur) {
-        char inputfilename[128];
+        char inputfilename[128] = "";
         // rules are to be used
         if (options[5].doesOccur) {
             strcpy(inputfilename, options[5].value);
@@ -736,19 +717,6 @@ void parseAttributes() {
     }
 }
 
-/**
-* Parser for normalization mode
-*/
-void parseNormalization() {
-    const char *str = options[8].value;
-    int norm = parseEnums(NORMALIZATION_VALUE_, str);
-    if (norm == -1) {
-        fprintf(stdout, "Normalization mode not found: %s\n", str);
-        return;
-    }
-    NORMALIZATION_ = (UNormalizationMode)norm;
-}
-
 /**
 * Checks if the locale argument is a base language 
 * @param locale to be checked
@@ -979,6 +947,85 @@ inline UBool checkInScripts(UScriptCode script[], int scriptcount,
     return FALSE;
 }
 
+/**
+* Gets the script elements and contractions belonging to the script
+* @param elems output list
+* @param locale locale
+* @return number of script elements
+* Add by Richard
+*/
+int getScriptElementsFromExemplars(ScriptElement scriptelem[], const char* locale) {
+    UErrorCode error = U_ZERO_ERROR;
+    UChar32 codepoint = 0;
+
+    UResourceBundle* ures = ures_open(NULL, locale, &error);
+    if (U_FAILURE(error)) {
+        fprintf(stdout, "Can not find resource bundle for locale: %s\n", locale);
+        return -1;
+    }
+    int32_t length;
+    const UChar* exemplarChars = ures_getStringByKey(ures, "ExemplarCharacters", &length, &error);
+
+    if (U_FAILURE(error)) {
+        fprintf(stdout, "Can not find ExemplarCharacters in resource bundle\n");
+        return -1;
+    }
+
+    UChar* upperChars = new UChar[length * 2];
+    if (upperChars == 0) {
+        fprintf(stdout, "Memory error\n");
+        return -1;
+    }
+
+    int32_t destLength = u_strToUpper(upperChars, length * 2, exemplarChars, -1, locale, &error);
+    if (U_FAILURE(error)) {
+        fprintf(stdout, "Error when u_strToUpper() \n");
+        return -1;
+    }
+
+    UChar* pattern = new UChar[length + destLength + 10];
+    UChar left[2] = {0x005b, 0x0};
+    UChar right[2] = {0x005d, 0x0};
+    pattern = u_strcpy(pattern, left);
+    pattern = u_strcat(pattern, exemplarChars);
+    pattern = u_strcat(pattern, upperChars);
+    pattern = u_strcat(pattern, right);
+
+    UnicodeSet * uniset = new UnicodeSet(UnicodeString(pattern), error);
+    if (U_FAILURE(error)) {
+        fprintf(stdout, "Can not open USet \n");
+        return -1;
+    }
+
+    UnicodeSetIterator* usetiter = new UnicodeSetIterator(*uniset);
+
+    int32_t count = 0;
+
+    while (usetiter -> next()) {
+        if (usetiter -> isString()) {
+            UnicodeString strItem = usetiter -> getString();
+
+            scriptelem[count].count = 0;
+            for (int i = 0; i < strItem.length(); i++) {
+                codepoint = strItem.char32At(i);
+                UTF16_APPEND_CHAR_UNSAFE(scriptelem[count].ch, scriptelem[count].count, codepoint);
+                scriptelem[count].tailored = FALSE;
+            }
+        } else {
+            codepoint = usetiter -> getCodepoint();
+            scriptelem[count].count = 0;
+            UTF16_APPEND_CHAR_UNSAFE(scriptelem[count].ch, scriptelem[count].count, codepoint);
+            scriptelem[count].tailored = FALSE;
+        }
+        delete []pattern;
+
+        count++;
+    }
+    delete []pattern;
+
+    return count;
+}
+
 /**
 * Gets the script elements and contractions belonging to the script
 * @param script list
@@ -1007,20 +1054,6 @@ int getScriptElements(UScriptCode script[], int scriptcount,
         codepoint ++;
     }
 
-    UChar    ucarules[0x10000];
-    UChar   *rule;
-    int32_t  rulelength = 0;
-
-    rule      = ucarules;
-    rulelength = ucol_getRulesEx(COLLATOR_, UCOL_FULL_RULES, ucarules, 
-                                 0x10000);
-    if (rulelength + UCOL_TOK_EXTRA_RULE_SPACE_SIZE > 0x10000) {
-        rule = (UChar *)malloc(sizeof(UChar) * 
-                                (rulelength + UCOL_TOK_EXTRA_RULE_SPACE_SIZE));
-        rulelength = ucol_getRulesEx(COLLATOR_, UCOL_FULL_RULES, rule, 
-                                     rulelength);
-    }
-    
     const UChar           *current  = NULL;
           uint32_t         strength = 0;
           uint32_t         chOffset = 0; 
@@ -1034,22 +1067,30 @@ int getScriptElements(UScriptCode script[], int scriptcount,
           UColTokenParser  src;
           UColOptionSet    opts;
           UParseError      parseError;
-        
-    src.source       = src.current = rule;
-    src.end          = rule + rulelength;
+
+    int32_t  rulelength = ucol_getRulesEx(COLLATOR_, UCOL_FULL_RULES, NULL, 0);
+    src.source       = (UChar *)malloc(sizeof(UChar) * 
+                                (rulelength + UCOL_TOK_EXTRA_RULE_SPACE_SIZE));
+    rulelength = ucol_getRulesEx(COLLATOR_, UCOL_FULL_RULES, src.source, 
+                                 rulelength);
+    src.current      = src.source;
+    src.end          = src.source + rulelength;
     src.extraCurrent = src.end;
     src.extraEnd     = src.end + UCOL_TOK_EXTRA_RULE_SPACE_SIZE;
     src.opts         = &opts;
         
-    while ((current = ucol_tok_parseNextToken(&src, &strength, &chOffset, 
+	/*
+	ucol_tok_parseNextToken(&src, &strength, &chOffset, 
                                               &chLen, &exOffset, &exLen,
                                               &prefixOffset, &prefixLen,
                                               &specs, rstart, &parseError,
-                                              &error)) 
-                                              != NULL) {
+                                              &error)
+    */
+    while ((current = ucol_tok_parseNextToken(&src, rstart, &parseError,
+                                              &error)) != NULL) {
         // contractions handled here
         if (chLen > 1) {
-            u_strncpy(scriptelem[count].ch, rule + chOffset, chLen);
+            u_strncpy(scriptelem[count].ch, src.source + chOffset, chLen);
             scriptelem[count].count = chLen;
             if (checkInScripts(script, scriptcount, scriptelem[count])) {
                 scriptelem[count].tailored     = FALSE;
@@ -1061,9 +1102,8 @@ int getScriptElements(UScriptCode script[], int scriptcount,
     if (U_FAILURE(error)) {
         fprintf(stdout, "Error parsing rules: %s\n", u_errorName(error));
     }
-    if (rule != ucarules) {
-       free(rule);
-    }
+	// rule might have been reallocated, so delete this instead
+    free(src.source); 
     return count;
 }
 
@@ -1131,29 +1171,23 @@ void markTailored(UScriptCode script[], int scriptcount,
           UColOptionSet    opts;
           UParseError      parseError;
     
-    src.opts = &opts;
-    
-    UChar *copy = (UChar *)malloc(
+    src.opts         = &opts;
+    src.source       = (UChar *)malloc(
                (rulelength + UCOL_TOK_EXTRA_RULE_SPACE_SIZE) * sizeof(UChar));
-    memcpy(copy, rule, rulelength * sizeof(UChar));
-
-    src.source       = src.current = copy;
-    src.end          = (UChar *)copy + rulelength;
+    memcpy(src.source, rule, rulelength * sizeof(UChar));
+	src.current      = src.source;
+    src.end          = (UChar *)src.source + rulelength;
     src.extraCurrent = src.end;
     src.extraEnd     = src.end + UCOL_TOK_EXTRA_RULE_SPACE_SIZE;
 
     UErrorCode    error = U_ZERO_ERROR;
         
-    while ((current = ucol_tok_parseNextToken(&src, &strength, &chOffset, 
-                                              &chLen, &exOffset, &exLen,
-                                              &prefixOffset, &prefixLen,
-                                              &specs, rstart, &parseError,
-                                              &error)) 
-                                              != NULL) {
+    while ((current = ucol_tok_parseNextToken(&src, rstart, &parseError,
+                                              &error)) != NULL) {
         if (chLen >= 1 && strength != UCOL_TOK_RESET) {
             // skipping the reset characters and non useful stuff.
             ScriptElement se;
-            u_strncpy(se.ch, copy + chOffset, chLen);
+            u_strncpy(se.ch, src.source + chOffset, chLen);
             se.count = chLen;
 
             if (checkInScripts(script, scriptcount, se)) {
@@ -1173,7 +1207,7 @@ void markTailored(UScriptCode script[], int scriptcount,
         }
         rstart = FALSE;
     }
-    free(copy);
+    free(src.source);
     if (U_FAILURE(error)) {
         fprintf(stdout, "Error parsing rules\n");
     }
@@ -1225,7 +1259,9 @@ void outputHTMLFooter()
 * @param script code list
 * @param scriptcount number of scripts
 */
-void serializeScripts(UScriptCode script[], int scriptcount) 
+//void serializeScripts(UScriptCode script[], int scriptcount) 
+//Richard
+void serializeScripts(UScriptCode script[], int scriptcount, const char* locale = NULL) 
 {
     UErrorCode  error  = U_ZERO_ERROR;
     
@@ -1235,7 +1271,13 @@ void serializeScripts(UScriptCode script[], int scriptcount)
         fprintf(stdout, "Memory error\n");
         return;
     }
-    int count = getScriptElements(script, scriptcount, scriptelem);    
+    int count = 0;
+    if(locale) {
+      count = getScriptElementsFromExemplars(scriptelem, locale);
+    } else {
+      count = getScriptElements(script, scriptcount, scriptelem); 
+    }
+
     // Sort script elements using Quicksort algorithm:
     qsort(scriptelem, count, sizeof(ScriptElement), compareCodepoints);
     markTailored(script, scriptcount, scriptelem, count);
@@ -1281,11 +1323,11 @@ void outputHTMLHeader(const char *locale, UScriptCode script[],
     fprintf(OUTPUT_, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n");
     fprintf(OUTPUT_, "<meta http-equiv=\"Content-Language\" content=\"en-us\">\n");
     fprintf(OUTPUT_, "<link rel=\"stylesheet\" href=\"charts.css\" type=\"text/css\">\n");
-    fprintf(OUTPUT_, "<title>Collation charts</title>\n");
+    fprintf(OUTPUT_, "<title>ICU Collation charts</title>\n");
     fprintf(OUTPUT_, "<base target=\"main\">\n");
     fprintf(OUTPUT_, "</head>\n");
 
-    fprintf(OUTPUT_, "<body>\n");
+    fprintf(OUTPUT_, "<body bgcolor=#FFFFFF>\n");
     fprintf(OUTPUT_, "<!--\n");
     fprintf(OUTPUT_, "This file contains sorted characters in ascending order according to the locale stated\n");
     fprintf(OUTPUT_, "If the character is in red, it is tailored in the collation rules.\n");
@@ -1299,7 +1341,7 @@ void outputHTMLHeader(const char *locale, UScriptCode script[],
     fprintf(OUTPUT_, "<table border=0>\n");
     UChar      displayname[64];
     UErrorCode error = U_ZERO_ERROR;
-    int32_t size = uloc_getDisplayName(locale, NULL, displayname, 64, &error);
+    int32_t size = uloc_getDisplayName(locale, "en_US", displayname, 64, &error);
     char       utf8displayname[128];
     if (U_FAILURE(error)) {
         utf8displayname[0] = 0;
@@ -1320,7 +1362,7 @@ void outputHTMLHeader(const char *locale, UScriptCode script[],
     }
     fprintf(OUTPUT_, "</td></tr>\n");
     
-    fprintf(OUTPUT_, "<tr><th>Rules</th><td class='noborder'><a href=http://oss.software.ibm.com/cvs/icu/~checkout~/icu/data/%s.txt>%s.txt</a></td></tr>\n", locale, locale);
+    fprintf(OUTPUT_, "<tr><th>Rules</th><td class='noborder'><a href=\"http://dev.icu-project.org/cgi-bin/viewcvs.cgi/*checkout*/icu/source/data/coll/%s.txt\">%s.txt</a></td></tr>\n", locale, locale);
     
     UVersionInfo version;
     ucol_getVersion(COLLATOR_, version);
@@ -1383,13 +1425,13 @@ void outputListHTMLHeader(FILE *file)
     fprintf(file, "<head>\n");
     fprintf(file, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n");
     fprintf(file, "<meta http-equiv=\"Content-Language\" content=\"en-us\">\n");
-    fprintf(file, "<title>Collation Charts</title>\n");
+    fprintf(file, "<title>ICU Collation Charts</title>\n");
     fprintf(file, "<base target=\"main\">\n");
     fprintf(file, "</head>\n");
-    fprintf(file, "<body>\n");
-    fprintf(file, "<h2 align=center>Collation Charts</h2>\n");
+    fprintf(file, "<body bgcolor=#FFFFFF>\n");
+    fprintf(file, "<h2 align=center>ICU Collation Charts</h2>\n");
     fprintf(file, "<p align=center>\n");
-    fprintf(file, "<a href=http://www.unicode.org/charts/uca/ target=new>UCA Charts</a> | ");
+    fprintf(file, "<a href=http://www.unicode.org/charts/collation/ target=new>UCA Charts</a><br>");
 }
 
 /**
@@ -1399,6 +1441,7 @@ void outputListHTMLHeader(FILE *file)
 void outputListHTMLFooter(FILE *file)
 {
     fprintf(file, "</p>\n");
+	//fprintf(file, "<center><image src=http://oss.software.ibm.com/icu/images/w24.gif></center>\n");
     fprintf(file, "</body>\n");
     fprintf(file, "</html>\n");
 }
@@ -1413,6 +1456,8 @@ void serializeScripts() {
     if (options[4].doesOccur) {
         strcpy(filename, options[4].value);
         dirlength = appendDirSeparator(filename);
+    } else {
+      filename[0] = 0;
     }
 
     const char    *locale;
@@ -1431,25 +1476,24 @@ void serializeScripts() {
     }
 
     outputListHTMLHeader(list);
-
+    fprintf(list, "<blockquote>\n");
     while (TRUE) {
         UErrorCode error = U_ZERO_ERROR;
         COLLATOR_ = ucol_open(locale, &error);
         if (U_FAILURE(error)) {
             fprintf(stdout, "Collator creation failed:");
             fprintf(stdout, u_errorName(error));
-            return;
+            break;
         }
         if ((error != U_USING_FALLBACK_WARNING && // not tailored
             error != U_USING_DEFAULT_WARNING) ||
             checkLocaleForLanguage(locale)) {
-            fprintf(list, "<a href=%s.html>%s</a> | ", locale, locale);
-            setAttributes(COLLATOR_, &error);
-            setNormalization(COLLATOR_);
+            fprintf(list, "<a href=%s.html>%s</a> ", locale, locale);
+	        setAttributes(COLLATOR_, &error);
             if (U_FAILURE(error)) {
                fprintf(stdout, "Collator attribute setting failed:");
                fprintf(stdout, u_errorName(error));
-               return;
+               break;
             }
 
             UScriptCode scriptcode[32];
@@ -1457,7 +1501,7 @@ void serializeScripts() {
                                                    &error);
             if (U_FAILURE(error)) {
                 fprintf(stdout, "Error getting lcale scripts\n");
-                return;
+                break;
             }
 
             strcat(filename, locale);
@@ -1465,10 +1509,18 @@ void serializeScripts() {
             OUTPUT_ = fopen(filename, "w");
             if (OUTPUT_ == NULL) {
                 fprintf(stdout, "Cannot open file:%s\n", filename);
-                return;
+                break;
             }
             outputHTMLHeader(locale, scriptcode, scriptcount);
-            serializeScripts(scriptcode, scriptcount);
+            fprintf(stdout, "%s\n", locale);
+
+            if(options[12].doesOccur) {
+              // use whole scripts
+                serializeScripts(scriptcode, scriptcount);
+            } else {
+              // use exemplar chars
+              serializeScripts(scriptcode, scriptcount, locale);
+            }
             fclose(OUTPUT_);
         }
         ucol_close(COLLATOR_);
@@ -1480,7 +1532,8 @@ void serializeScripts() {
         }
         locale = ucol_getAvailable(localelist);
     }
-    fprintf(list, "<a href=help.html>help</a>");
+    fprintf(list, "<br><a href=help.html>help</a><br>");
+    fprintf(list, "</blockquote>\n");
     outputListHTMLFooter(list);
     fclose(list);
 }
@@ -1501,7 +1554,7 @@ int main(int argc, char *argv[]) {
         fprintf(stdout, "\n");
     }
     if (argc < 0 || options[0].doesOccur || options[1].doesOccur) {
-        fprintf(stdout, "Usage: strperf options...\n"
+        fprintf(stdout, "Usage: dumpce options...\n"
                         "--help\n"
                         "    Display this message.\n"
                         "--locale name|all\n"
@@ -1521,9 +1574,12 @@ int main(int argc, char *argv[]) {
                         "--scripts\n" 
                         "    Codepoints from all scripts are sorted and serialized.\n"
                         "--reducehan\n" 
-                        "    Only 200 Han script characters will be displayed with the use of --scripts.\n");
+                        "    Only 200 Han script characters will be displayed with the use of --scripts.\n"
+                        "--wholescripts\n"
+                        "    Show collation order for whole scripts instead of just for exemplar characters of a locale\n\n");
 
-        fprintf(stdout, "Example: dumpce --serialize --locale af --destdir /temp --attribute UCOL_STRENGTH=UCOL_DEFAULT_STRENGTH,4=17\n\n");
+        fprintf(stdout, "Example to generate *.txt files : dumpce --serialize --locale af --destdir /temp --attribute UCOL_STRENGTH=UCOL_DEFAULT_STRENGTH,4=17\n\n");
+        fprintf(stdout, "Example to generate *.html files for oss web display: dumpce --scripts --destdir /temp --reducehan\n");
         return argc < 0 ? U_ILLEGAL_ARGUMENT_ERROR : U_ZERO_ERROR;
     }
 
@@ -1532,10 +1588,6 @@ int main(int argc, char *argv[]) {
         fprintf(stdout, "attributes %s\n", options[6].value);
         parseAttributes();
     }
-    if (options[8].doesOccur) {
-        fprintf(stdout, "normalization mode %s\n", options[8].value);
-        parseNormalization();
-    }
     if (options[3].doesOccur) {
         serialize();
     }
